@@ -79,21 +79,31 @@ static pmix_event_t stdinsig, parentdied;
 static pmix_iof_read_event_t stdinev;
 static pmix_proc_t myparent;
 
-static void pdiedfn(int sd, short args, void *cbdata)
+static void _pdiedcb(pmix_status_t status, void *cbdata)
+{
+    pmix_cb_t *cb = (pmix_cb_t*)cbdata;
+    PMIX_HIDE_UNUSED_PARAMS(status);
+    PMIX_RELEASE(cb);
+}
+
+static void pdiedfn(int fd, short flags, void *arg)
 {
-    pmix_info_t info[2];
     pmix_proc_t keepalive;
-    PMIX_HIDE_UNUSED_PARAMS(sd, args, cbdata);
+    pmix_cb_t *cb;
+    PMIX_HIDE_UNUSED_PARAMS(fd, flags, arg);
 
     PMIX_LOAD_PROCID(&keepalive, "PMIX_KEEPALIVE_PIPE", PMIX_RANK_UNDEF);
 
-    PMIX_INFO_LOAD(&info[0], PMIX_EVENT_NON_DEFAULT, NULL, PMIX_BOOL);
-    PMIX_INFO_LOAD(&info[1], PMIX_EVENT_AFFECTED_PROC, &keepalive, PMIX_PROC);
-
     /* generate a job-terminated event */
+    cb = PMIX_NEW(pmix_cb_t);
+    cb->ninfo = 2;
+    PMIX_INFO_CREATE(cb->info, cb->ninfo);
+    PMIX_INFO_LOAD(&cb->info[0], PMIX_EVENT_NON_DEFAULT, NULL, PMIX_BOOL);
+    PMIX_INFO_LOAD(&cb->info[1], PMIX_EVENT_AFFECTED_PROC, &keepalive, PMIX_PROC);
+    cb->infocopy = true;  // ensure cleanup
     PMIx_Notify_event(PMIX_ERR_JOB_TERMINATED, &pmix_globals.myid,
-                      PMIX_RANGE_PROC_LOCAL, info, 2,
-                      NULL, NULL);
+                      PMIX_RANGE_PROC_LOCAL, cb->info, cb->ninfo,
+                      _pdiedcb, (void*)cb);
 }
 
 static void _notify_complete(pmix_status_t status, void *cbdata)
@@ -669,19 +679,24 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc, pmix_info_t info[], size_t nin
 
     pmix_output_verbose(2, pmix_globals.debug_output, "pmix: init called");
 
+    /* setup a rank_info object for us */
+    pmix_globals.mypeer->info = PMIX_NEW(pmix_rank_info_t);
+    if (NULL == pmix_globals.mypeer->info) {
+        PMIX_RELEASE_THREAD(&pmix_global_lock);
+        return PMIX_ERR_NOMEM;
+    }
     if (PMIX_PEER_IS_CLIENT(pmix_globals.mypeer)) {
         /* if we are a client, then we need to pickup the
          * rest of the envar-based server assignments */
         pmix_globals.pindex = -1;
-        /* setup a rank_info object for us */
-        pmix_globals.mypeer->info = PMIX_NEW(pmix_rank_info_t);
-        if (NULL == pmix_globals.mypeer->info) {
-            PMIX_RELEASE_THREAD(&pmix_global_lock);
-            return PMIX_ERR_NOMEM;
-        }
         pmix_globals.mypeer->info->pname.nspace = strdup(pmix_globals.myid.nspace);
         pmix_globals.mypeer->info->pname.rank = pmix_globals.myid.rank;
     }
+    pmix_globals.mypeer->info->realuid = pmix_globals.realuid;
+    pmix_globals.mypeer->info->uid = pmix_globals.uid;
+    pmix_globals.mypeer->info->realgid = pmix_globals.realgid;
+    pmix_globals.mypeer->info->gid = pmix_globals.gid;
+    pmix_globals.mypeer->info->pid = pmix_globals.pid;
 
     /* select our bfrops compat module */
     pmix_globals.mypeer->nptr->compat.bfrops = pmix_bfrops_base_assign_module(NULL);
@@ -1015,8 +1030,11 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc, pmix_info_t info[], size_t nin
         }
 
         /* start listening for connections */
-        if (PMIX_SUCCESS != pmix_ptl_base_start_listening(info, ninfo)) {
-            pmix_show_help("help-pmix-server.txt", "listener-thread-start", true);
+        rc = pmix_ptl_base_start_listening(info, ninfo);
+        if (PMIX_SUCCESS != rc) {
+            if (PMIX_ERR_SILENT != rc) {
+                pmix_show_help("help-pmix-server.txt", "listener-thread-start", true);
+            }
             return PMIX_ERR_INIT;
         }
     }
