@@ -43,6 +43,7 @@
 #include "src/event/event-internal.h"
 #include "src/hwloc/hwloc-internal.h"
 #include "src/pmix/pmix-internal.h"
+#include "src/util/pmix_if.h"
 #include "src/util/pmix_os_path.h"
 #include "src/util/pmix_environ.h"
 
@@ -97,7 +98,7 @@ int prte_ess_base_prted_setup(void)
 {
     int ret = PRTE_ERROR;
     int fd;
-    char log_file[PATH_MAX];
+    char log_file[PRTE_PATH_MAX];
     char *error = NULL;
     char *uri = NULL;
     char *tmp = NULL;
@@ -273,7 +274,7 @@ int prte_ess_base_prted_setup(void)
          */
 
         /* define a log file name in the session directory */
-        snprintf(log_file, PATH_MAX, "output-prted-%s-%s.log",
+        snprintf(log_file, PRTE_PATH_MAX, "output-prted-%s-%s.log",
                  prte_process_info.myproc.nspace,
                  prte_process_info.nodename);
         log_path = pmix_os_path(false, prte_process_info.top_session_dir, log_file, NULL);
@@ -303,6 +304,10 @@ int prte_ess_base_prted_setup(void)
         goto error;
     }
 
+    /* add network aliases to our list of alias hostnames - must
+     * wait until after we init PMIx before getting them */
+    pmix_ifgetaliases(&prte_process_info.aliases);
+
     /* Setup the communication infrastructure */
     if (PRTE_SUCCESS
         != (ret = pmix_mca_base_framework_open(&prte_prtereachable_base_framework,
