@@ -39,8 +39,9 @@ int main(int argc, char **argv)
 {
     int num_messages, message_size, rc;
 
-    if(argc < 3) {
-        printf("Usage: mpirun -np 2 --mca mpi_spc_attach all --mca mpi_spc_dump_enabled true ./spc_example [num_messages] [message_size]\n");
+    if (argc < 3) {
+        printf("Usage: mpirun -n 2 --mca mpi_spc_attach all --mca mpi_spc_dump_enabled true "
+               "./spc_example [num_messages] [message_size]\n");
         return -1;
     } else {
         num_messages = atoi(argv[1]);
@@ -54,8 +55,9 @@ int main(int argc, char **argv)
     char name[256], description[256];
 
     /* Counter names to be read by ranks 0 and 1 */
-    char *counter_names[] = {"runtime_spc_OMPI_BYTES_SENT_USER",
-                             "runtime_spc_OMPI_BYTES_RECEIVED_USER" };
+    /* (See also: ompi_spc_counters_t for list) */
+    char *counter_names[] = {"runtime_spc_OMPI_SPC_BYTES_SENT_USER",
+                             "runtime_spc_OMPI_SPC_BYTES_RECEIVED_USER" };
 
     MPI_Init(NULL, NULL);
     MPI_T_init_thread(MPI_THREAD_SINGLE, &provided);
@@ -67,7 +69,7 @@ int main(int argc, char **argv)
         MPI_Abort(MPI_COMM_WORLD, -1);
     }
 
-    /* Determine the MPI_T pvar indices for the OMPI_BYTES_SENT/RECIEVED_USER SPCs */
+    /* Determine the MPI_T pvar indices for the OMPI_BYTES_SENT/RECEIVED_USER SPCs */
     index = -1;
     MPI_T_pvar_get_num(&num);
     for(i = 0; i < num; i++) {
