@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -12,6 +13,11 @@
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2017      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2018-2019 Triad National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2022      Amazon.com, Inc. or its affiliates.
+ *                         All Rights reserved.
+ * Copyright (c) 2023      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -55,7 +61,7 @@ enum ompi_attribute_type_t {
                       * with 1 so that we can have it initialized to 0
                       * using memset in the constructor */
     TYPE_ATTR,       /**< The attribute belongs to datatype object */
-    WIN_ATTR         /**< The attribute belongs to a win object */
+    WIN_ATTR,        /**< The attribute belongs to a win object */
 };
 typedef enum ompi_attribute_type_t ompi_attribute_type_t;
 
@@ -64,50 +70,22 @@ typedef enum ompi_attribute_type_t ompi_attribute_type_t;
    delete. These will only be used here and not in the front end
    functions. */
 
-typedef void (ompi_fint_copy_attr_function)(MPI_Fint *oldobj,
-                                                    MPI_Fint *keyval,
-                                                    MPI_Fint *extra_state,
-                                                    MPI_Fint *attr_in,
-                                                    MPI_Fint *attr_out,
-                                                    ompi_fortran_logical_t *flag,
-                                                    MPI_Fint *ierr);
-typedef void (ompi_fint_delete_attr_function)(MPI_Fint *obj,
-                                                      MPI_Fint *keyval,
-                                                      MPI_Fint *attr_in,
-                                                      MPI_Fint *extra_state,
-                                                      MPI_Fint *ierr);
+typedef void (*ompi_fint_copy_attr_function)(MPI_Fint *oldobj, MPI_Fint *keyval,
+                                             MPI_Fint *extra_state, MPI_Fint *attr_in,
+                                             MPI_Fint *attr_out, ompi_fortran_logical_t *flag,
+                                             MPI_Fint *ierr);
+typedef void (*ompi_fint_delete_attr_function)(MPI_Fint *obj, MPI_Fint *keyval, MPI_Fint *attr_in,
+                                               MPI_Fint *extra_state, MPI_Fint *ierr);
 
 /* New-style MPI-2 Fortran function pointer declarations for copy and
    delete. These will only be used here and not in the front end
    functions. */
 
-typedef void (ompi_aint_copy_attr_function)(MPI_Fint *oldobj,
-                                            MPI_Fint *keyval,
-                                            void *extra_state,
-                                            void *attr_in,
-                                            void *attr_out,
-                                            ompi_fortran_logical_t *flag,
-                                            MPI_Fint *ierr);
-typedef void (ompi_aint_delete_attr_function)(MPI_Fint *obj,
-                                              MPI_Fint *keyval,
-                                              void *attr_in,
-                                              void *extra_state,
-                                              MPI_Fint *ierr);
-/*
- * Internally the copy function for all kinds of MPI objects has one more
- * argument, the pointer to the new object. Therefore, we can do on the
- * flight modifications of the new communicator based on attributes stored
- * on the main communicator.
- */
-typedef int (MPI_Comm_internal_copy_attr_function)(MPI_Comm, int, void *,
-                                                   void *, void *, int *,
-                                                   MPI_Comm);
-typedef int (MPI_Type_internal_copy_attr_function)(MPI_Datatype, int, void *,
-                                                   void *, void *, int *,
-                                                   MPI_Datatype);
-typedef int (MPI_Win_internal_copy_attr_function)(MPI_Win, int, void *,
-                                                  void *, void *, int *,
-                                                  MPI_Win);
+typedef void (*ompi_aint_copy_attr_function)(MPI_Fint *oldobj, MPI_Fint *keyval, void *extra_state,
+                                             void *attr_in, void *attr_out,
+                                             ompi_fortran_logical_t *flag, MPI_Fint *ierr);
+typedef void (*ompi_aint_delete_attr_function)(MPI_Fint *obj, MPI_Fint *keyval, void *attr_in,
+                                               void *extra_state, MPI_Fint *ierr);
 
 typedef void (ompi_attribute_keyval_destructor_fn_t)(int);
 
@@ -120,19 +98,17 @@ union ompi_attribute_fn_ptr_union_t {
     MPI_Type_delete_attr_function          *attr_datatype_delete_fn;
     MPI_Win_delete_attr_function           *attr_win_delete_fn;
 
-    MPI_Comm_internal_copy_attr_function   *attr_communicator_copy_fn;
-    MPI_Type_internal_copy_attr_function   *attr_datatype_copy_fn;
-    MPI_Win_internal_copy_attr_function    *attr_win_copy_fn;
+    MPI_Comm_copy_attr_function            *attr_communicator_copy_fn;
+    MPI_Type_copy_attr_function            *attr_datatype_copy_fn;
+    MPI_Win_copy_attr_function             *attr_win_copy_fn;
 
     /* For Fortran old MPI-1 callback functions */
-
-    ompi_fint_delete_attr_function *attr_fint_delete_fn;
-    ompi_fint_copy_attr_function   *attr_fint_copy_fn;
+    ompi_fint_delete_attr_function          attr_fint_delete_fn;
+    ompi_fint_copy_attr_function            attr_fint_copy_fn;
 
     /* For Fortran new MPI-2 callback functions */
-
-    ompi_aint_delete_attr_function *attr_aint_delete_fn;
-    ompi_aint_copy_attr_function   *attr_aint_copy_fn;
+    ompi_aint_delete_attr_function          attr_aint_delete_fn;
+    ompi_aint_copy_attr_function            attr_aint_copy_fn;
 };
 
 typedef union ompi_attribute_fn_ptr_union_t ompi_attribute_fn_ptr_union_t;
@@ -201,22 +177,26 @@ int ompi_attr_hash_init(opal_hash_table_t **hash)
 }
 
 /**
- * Initialize the main attribute hash that stores the keyvals and meta data
+ * Increase the reference count on the attributes subsystem.  Instantiate subsys if
+ * not yet instantiated.
  *
  * @return OMPI return code
  */
 
-int ompi_attr_init(void);
+int ompi_attr_get_ref(void);
+
 
 /**
- * Destroy the main attribute hash that stores the keyvals and meta data
+ * Decrease the reference count on the attributes subsystem.  Attributes subsystem
+ * resources are released when the count drops to zero.
+ *
+ * @return OMPI return code
  */
 
-int ompi_attr_finalize(void);
-
+int ompi_attr_put_ref(void);
 
 /**
- * Create a new key for use by attribute of Comm/Win/Datatype
+ * Create a new key for use by attribute of Comm/Win/Datatype/Instance
  *
  * @param type           Type of attribute (COMM/WIN/DTYPE) (IN)
  * @param copy_attr_fn   Union variable containing the function pointer
@@ -289,7 +269,7 @@ int ompi_attr_free_keyval(ompi_attribute_type_t type, int *key,
  * Set an attribute on the comm/win/datatype in a form valid for C.
  *
  * @param type           Type of attribute (COMM/WIN/DTYPE) (IN)
- * @param object         The actual Comm/Win/Datatype object (IN)
+ * @param object         The actual Comm/Win/Datatype/Instance object (IN)
  * @param attr_hash      The attribute hash table hanging on the object(IN/OUT)
  * @param key            Key val for the attribute (IN)
  * @param attribute      The actual attribute pointer (IN)
@@ -318,7 +298,7 @@ int ompi_attr_set_c(ompi_attribute_type_t type, void *object,
  * Set an int predefined attribute in a form valid for C.
  *
  * @param type           Type of attribute (COMM/WIN/DTYPE) (IN)
- * @param object         The actual Comm/Win/Datatype object (IN)
+ * @param object         The actual Comm/Win/Datatype/Instance object (IN)
  * @param attr_hash      The attribute hash table hanging on the object(IN/OUT)
  * @param key            Key val for the attribute (IN)
  * @param attribute      The actual attribute value (IN)
@@ -348,7 +328,7 @@ int ompi_attr_set_int(ompi_attribute_type_t type, void *object,
  * Fortran MPI-1.
  *
  * @param type           Type of attribute (COMM/WIN/DTYPE) (IN)
- * @param object         The actual Comm/Win/Datatype object (IN)
+ * @param object         The actual Comm/Win/Datatype/Instance object (IN)
  * @param attr_hash      The attribute hash table hanging on the object(IN/OUT)
  * @param key            Key val for the attribute (IN)
  * @param attribute      The actual attribute pointer (IN)
@@ -379,7 +359,7 @@ OMPI_DECLSPEC int ompi_attr_set_fint(ompi_attribute_type_t type, void *object,
  * Fortran MPI-2.
  *
  * @param type           Type of attribute (COMM/WIN/DTYPE) (IN)
- * @param object         The actual Comm/Win/Datatype object (IN)
+ * @param object         The actual Comm/Win/Datatype/Instance object (IN)
  * @param attr_hash      The attribute hash table hanging on the object(IN/OUT)
  * @param key            Key val for the attribute (IN)
  * @param attribute      The actual attribute pointer (IN)
@@ -488,7 +468,7 @@ OMPI_DECLSPEC int ompi_attr_get_aint(opal_hash_table_t *attr_hash, int key,
 /**
  * Delete an attribute on the comm/win/datatype
  * @param type           Type of attribute (COMM/WIN/DTYPE) (IN)
- * @param object         The actual Comm/Win/Datatype object (IN)
+ * @param object         The actual Comm/Win/Datatype/Instance object (IN)
  * @param attr_hash      The attribute hash table hanging on the object(IN)
  * @param key            Key val for the attribute (IN)
  * @param predefined     Whether the key is predefined or not 0/1 (IN)
@@ -503,7 +483,7 @@ int ompi_attr_delete(ompi_attribute_type_t type, void *object,
 
 /**
  * This to be used from functions like MPI_*_DUP in order to copy all
- * the attributes from the old Comm/Win/Dtype object to a new
+ * the attributes from the old Comm/Win/Dtype/Instance object to a new
  * object.
  * @param type         Type of attribute (COMM/WIN/DTYPE) (IN)
  * @param old_object   The old COMM/WIN/DTYPE object (IN)
@@ -520,7 +500,7 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
 
 
 /**
- * This to be used to delete all the attributes from the Comm/Win/Dtype
+ * This to be used to delete all the attributes from the Comm/Win/Dtype/Instance
  * object in one shot
  * @param type         Type of attribute (COMM/WIN/DTYPE) (IN)
  * @param object       The COMM/WIN/DTYPE object (IN)
@@ -536,11 +516,23 @@ int ompi_attr_delete_all(ompi_attribute_type_t type, void *object,
 /**
  * \internal
  *
- * Create all the predefined attributes
+ * Create all the predefined attribute keys
+ * @note This routine is invoked when creating a session 
+ *       so must be thread safe.
  *
  * @returns OMPI_SUCCESS
  */
-int ompi_attr_create_predefined(void);
+int ompi_attr_create_predefined_keyvals(void);
+
+/**
+ * \internal
+ *
+ * Cache and release the predefined attribute keys used in the World Model
+ *
+ * @returns OMPI_SUCCESS
+ */
+int ompi_attr_set_predefined_keyvals_for_wm(void);
+void ompi_attr_delete_predefined_keyvals_for_wm(void);
 
 /**
  * \internal
