@@ -45,7 +45,7 @@ __ompi_datatype_create_from_args( int32_t* i, ptrdiff_t * a,
                                   ompi_datatype_t** d, int32_t type );
 
 typedef struct __dt_args {
-    int32_t            ref_count;
+    opal_atomic_int32_t ref_count;
     int32_t            create_type;
     size_t             total_pack_size;
     int32_t            ci;
@@ -71,57 +71,45 @@ typedef struct __dt_args {
 #define OMPI_DATATYPE_ALIGN_PTR(PTR, TYPE)
 #endif  /* OPAL_ALIGN_WORD_SIZE_INTEGERS */
 
-/**
- * Some architectures require 64 bits pointers (to pointers) to
- * be 64 bits aligned. As in the ompi_datatype_args_t structure we have
- * 2 such array of pointers and one to an array of ints, if we start by
- * setting the 64 bits aligned one we will not have any trouble. Problem
- * originally reported on SPARC 64.
- */
-#define ALLOC_ARGS(PDATA, IC, AC, DC)                                   \
-    do {                                                                \
-        int length = sizeof(ompi_datatype_args_t) + (IC) * sizeof(int) + \
-            (AC) * sizeof(ptrdiff_t) + (DC) * sizeof(MPI_Datatype); \
-        char* buf = (char*)malloc( length );                            \
-        ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)buf;       \
-        pArgs->ci = (IC);                                               \
-        pArgs->ca = (AC);                                               \
-        pArgs->cd = (DC);                                               \
-        buf += sizeof(ompi_datatype_args_t);                            \
-        if( pArgs->ca == 0 ) pArgs->a = NULL;                           \
-        else {                                                          \
-            pArgs->a = (ptrdiff_t*)buf;                         \
-            buf += pArgs->ca * sizeof(ptrdiff_t);               \
-        }                                                               \
-        if( pArgs->cd == 0 ) pArgs->d = NULL;                           \
-        else {                                                          \
-            pArgs->d = (ompi_datatype_t**)buf;                          \
-            buf += pArgs->cd * sizeof(MPI_Datatype);                    \
-        }                                                               \
-        if( pArgs->ci == 0 ) pArgs->i = NULL;                           \
-        else pArgs->i = (int*)buf;                                      \
-        pArgs->ref_count = 1;                                           \
-        pArgs->total_pack_size = (4 + (IC) + (DC)) * sizeof(int) +      \
-            (AC) * sizeof(ptrdiff_t);                                   \
-        (PDATA)->args = (void*)pArgs;                                   \
-        (PDATA)->packed_description = NULL;                             \
-    } while(0)
-
-
 int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
                                 int32_t ci, const int32_t** i,
                                 int32_t ca, const ptrdiff_t* a,
                                 int32_t cd, ompi_datatype_t* const * d, int32_t type)
 {
     int pos;
-    ompi_datatype_args_t* pArgs;
 
     assert( NULL == pData->args );
-    ALLOC_ARGS( pData, ci, ca, cd );
-
-    pArgs = (ompi_datatype_args_t*)pData->args;
+    int length = sizeof(ompi_datatype_args_t) + ci * sizeof(int) +
+                 ca * sizeof(ptrdiff_t) + cd * sizeof(MPI_Datatype);
+    char* buf = (char*)malloc( length );
+    ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)buf;
+    pArgs->ci = ci; pArgs->i = NULL;
+    pArgs->ca = ca; pArgs->a = NULL;
+    pArgs->cd = cd; pArgs->d = NULL;
     pArgs->create_type = type;
 
+    /**
+     * Some architectures require 64 bits pointers (to pointers) to
+     * be 64 bits aligned. As in the ompi_datatype_args_t structure we have
+     * 2 such array of pointers and one to an array of ints, if we start by
+     * setting the 64 bits aligned one we will not have any trouble. Problem
+     * originally reported on SPARC 64.
+     */
+    buf += sizeof(ompi_datatype_args_t);
+    if( 0 != pArgs->ca ) {
+        pArgs->a = (ptrdiff_t*)buf;
+        buf += pArgs->ca * sizeof(ptrdiff_t);
+    }
+    if( 0 != pArgs->cd ) {
+        pArgs->d = (ompi_datatype_t**)buf;
+        buf += pArgs->cd * sizeof(MPI_Datatype);
+    }
+    if( 0 != pArgs->ci ) pArgs->i = (int*)buf;
+
+    pArgs->ref_count = 1;
+    pArgs->total_pack_size = (4 + ci) * sizeof(int) +
+                             cd * sizeof(MPI_Datatype) +
+                             ca * sizeof(ptrdiff_t);
     switch(type) {
 
     case MPI_COMBINER_DUP:
@@ -229,7 +217,7 @@ int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
         pArgs->d[pos] = d[pos];
         if( !(ompi_datatype_is_predefined(d[pos])) ) {
             /* We handle a user defined datatype. We should make sure that the
-             * user will not have the oportunity to destroy it before all derived
+             * user will not have the opportunity to destroy it before all derived
              * datatypes are destroyed. As we keep pointers to every datatype
              * (for MPI_Type_get_content and MPI_Type_get_envelope) we have to make
              * sure that those datatype will be available if the user ask for them.
@@ -243,10 +231,12 @@ int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
         pArgs->total_pack_size += sizeof(int);  /* each data has an ID */
     }
 
+    pData->args = (void*)pArgs;
+    pData->packed_description = 0;
+
     return OMPI_SUCCESS;
 }
 
-
 int32_t ompi_datatype_print_args( const ompi_datatype_t* pData )
 {
     int32_t i;
@@ -375,8 +365,8 @@ int32_t ompi_datatype_copy_args( const ompi_datatype_t* source_data,
 {
     ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)source_data->args;
 
-    /* Increase the reference count of the datatype enveloppe. This
-     * prevent us from making extra copies for the enveloppe (which is mostly
+    /* Increase the reference count of the datatype envelope. This
+     * prevent us from making extra copies for the envelope (which is mostly
      * a read only memory).
      */
     if( NULL != pArgs ) {
@@ -401,7 +391,7 @@ int32_t ompi_datatype_release_args( ompi_datatype_t* pData )
     OPAL_THREAD_ADD_FETCH32(&pArgs->ref_count, -1);
     if( 0 == pArgs->ref_count ) {
         /* There are some duplicated datatypes around that have a pointer to this
-         * args. We will release them only when the last datatype will dissapear.
+         * args. We will release them only when the last datatype will disappear.
          */
         for( i = 0; i < pArgs->cd; i++ ) {
             if( !(ompi_datatype_is_predefined(pArgs->d[i])) ) {
@@ -458,7 +448,7 @@ static inline int __ompi_datatype_pack_description( ompi_datatype_t* datatype,
     position = (int*)next_packed;
     next_packed += sizeof(int) * args->cd;
 
-    /* copy the aray of counts (32 bits aligned) */
+    /* copy the array of counts (32 bits aligned) */
     memcpy( next_packed, args->i, sizeof(int) * args->ci );
     next_packed += args->ci * sizeof(int);
 
@@ -485,12 +475,12 @@ int ompi_datatype_get_pack_description( ompi_datatype_t* datatype,
 {
     ompi_datatype_args_t* args = (ompi_datatype_args_t*)datatype->args;
     int next_index = OMPI_DATATYPE_MAX_PREDEFINED;
-    void *packed_description = datatype->packed_description;
+    void *packed_description = (void *) datatype->packed_description;
     void* recursive_buffer;
 
     if (NULL == packed_description) {
         void *_tmp_ptr = NULL;
-        if (opal_atomic_compare_exchange_strong_ptr (&datatype->packed_description, (void *) &_tmp_ptr, (void *) 1)) {
+        if (opal_atomic_compare_exchange_strong_ptr (&datatype->packed_description, (intptr_t *) &_tmp_ptr, 1)) {
             if( ompi_datatype_is_predefined(datatype) ) {
                 packed_description = malloc(2 * sizeof(int));
             } else if( NULL == args ) {
@@ -512,10 +502,10 @@ int ompi_datatype_get_pack_description( ompi_datatype_t* datatype,
             }
 
             opal_atomic_wmb ();
-            datatype->packed_description = packed_description;
+            datatype->packed_description = (intptr_t) packed_description;
         } else {
             /* another thread beat us to it */
-            packed_description = datatype->packed_description;
+            packed_description = (void *) datatype->packed_description;
         }
     }
 
@@ -523,11 +513,11 @@ int ompi_datatype_get_pack_description( ompi_datatype_t* datatype,
         struct timespec interval = {.tv_sec = 0, .tv_nsec = 1000};
 
         /* wait until the packed description is updated */
-        while ((void *) 1 == datatype->packed_description) {
+        while (1 == datatype->packed_description) {
             nanosleep (&interval, NULL);
         }
 
-        packed_description = datatype->packed_description;
+        packed_description = (void *) datatype->packed_description;
     }
 
     *packed_buffer = (const void *) packed_description;
@@ -536,7 +526,7 @@ int ompi_datatype_get_pack_description( ompi_datatype_t* datatype,
 
 size_t ompi_datatype_pack_description_length( ompi_datatype_t* datatype )
 {
-    void *packed_description = datatype->packed_description;
+    void *packed_description = (void *) datatype->packed_description;
 
     if( ompi_datatype_is_predefined(datatype) ) {
         return 2 * sizeof(int);
@@ -841,9 +831,7 @@ ompi_datatype_t* ompi_datatype_get_single_predefined_type_from_args( ompi_dataty
                 return NULL;
             }
         }
-#if OMPI_ENABLE_MPI1_COMPAT
         if (current_predef != MPI_LB && current_predef != MPI_UB) {
-#endif
             if( NULL == predef ) {  /* This is the first iteration */
                 predef = current_predef;
             } else {
@@ -857,9 +845,7 @@ ompi_datatype_t* ompi_datatype_get_single_predefined_type_from_args( ompi_dataty
                     return NULL;
                 }
             }
-#if OMPI_ENABLE_MPI1_COMPAT
         }
-#endif
     }
     return predef;
 }
