@@ -3,7 +3,11 @@
 # Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
 # Copyright (c) 2016-2019 Research Organization for Information Science
 #                         and Technology (RIST).  All rights reserved.
-# Copyright (c) 2016      FUJITSU LIMITED.  All rights reserved.
+# Copyright (c) 2016-2018 FUJITSU LIMITED.  All rights reserved.
+# Copyright (c) 2020      The University of Tennessee and The University
+#                         of Tennessee Research Foundation.  All rights
+#                         reserved.
+# Copyright (c) 2022      IBM Corporation.  All rights reserved.
 # $COPYRIGHT$
 #
 # Additional copyrights may follow
@@ -67,6 +71,30 @@ sub write_file {
 
 #----------------------------------------------------------------------------
 
+# Read a value for a specified key from the file specified.
+
+sub read_value_from_file {
+    my ($filename, $key) = @_;
+    my $value;
+
+    open(FILE_IN, $filename) || die "Couldn't open $filename";
+    while(my $line = <FILE_IN>) {
+        if( $line =~ /^$key=(.+)/ ) {
+            $value = $1;
+            last;
+        }
+    }
+    close(FILE_IN);
+
+    if(!defined($value)) {
+        die "Did not find the string \"$key\" in the file $filename"
+    }
+
+    return $value;
+}
+
+#----------------------------------------------------------------------------
+
 print "creating Fortran header files (with common constants)...\n";
 
 # Find the OMPI topdir.  It is likely the pwd.
@@ -93,6 +121,7 @@ $handles->{MPI_COMM_SELF} = 1;
 $handles->{MPI_GROUP_EMPTY} = 1;
 $handles->{MPI_ERRORS_ARE_FATAL} = 1;
 $handles->{MPI_ERRORS_RETURN} = 2;
+$handles->{MPI_ERRORS_ABORT} = 3;
 
 $handles->{MPI_MAX} =  1;
 $handles->{MPI_MIN} =  2;
@@ -119,6 +148,7 @@ $handles->{MPI_OP_NULL} = 0;
 $handles->{MPI_REQUEST_NULL} = 0;
 $handles->{MPI_WIN_NULL} = 0;
 $handles->{MPI_MESSAGE_NULL} = 0;
+$handles->{MPI_SESSION_NULL} = 0;
 
 $handles->{MPI_BYTE} =  1;
 $handles->{MPI_PACKED} =  2;
@@ -195,6 +225,7 @@ $handles->{MPI_C_FLOAT_COMPLEX} = $handles->{MPI_C_COMPLEX};
 $handles->{MPI_C_DOUBLE_COMPLEX} = 70;
 $handles->{MPI_C_LONG_DOUBLE_COMPLEX} = 71;
 $handles->{MPI_COUNT} = 72;
+$handles->{MPI_COMPLEX4} = 73;
 
 $handles->{MPI_MESSAGE_NO_PROC} = 1;
 
@@ -210,8 +241,8 @@ $io_handles->{MPI_FILE_NULL} = 0;
 
 my $constants;
 
-$constants->{MPI_VERSION} = 3;
-$constants->{MPI_SUBVERSION} = 1;
+$constants->{MPI_VERSION} = read_value_from_file("$topdir/VERSION", "mpi_standard_version");
+$constants->{MPI_SUBVERSION} = read_value_from_file("$topdir/VERSION", "mpi_standard_subversion");
 
 $constants->{MPI_ANY_SOURCE} = -1;
 $constants->{MPI_ANY_TAG} = -1;
@@ -237,6 +268,7 @@ $constants->{MPI_WIN_SIZE} = 8;
 $constants->{MPI_WIN_DISP_UNIT} = 9;
 $constants->{MPI_WIN_CREATE_FLAVOR} = 10;
 $constants->{MPI_WIN_MODEL} = 11;
+$constants->{MPI_FT} = 12;
 $constants->{MPI_WIN_FLAVOR_CREATE} = 1;
 $constants->{MPI_WIN_FLAVOR_ALLOCATE} = 2;
 $constants->{MPI_WIN_FLAVOR_DYNAMIC} = 3;
@@ -311,6 +343,7 @@ $constants->{MPI_ERR_NOT_SAME} = 40;
 $constants->{MPI_ERR_NO_SPACE} = 41;
 $constants->{MPI_ERR_NO_SUCH_FILE} = 42;
 $constants->{MPI_ERR_PORT} = 43;
+$constants->{MPI_ERR_PROC_ABORTED} = 74;
 $constants->{MPI_ERR_QUOTA} = 44;
 $constants->{MPI_ERR_READ_ONLY} = 45;
 $constants->{MPI_ERR_RMA_CONFLICT} = 46;
@@ -321,6 +354,11 @@ $constants->{MPI_ERR_SPAWN} = 50;
 $constants->{MPI_ERR_UNSUPPORTED_DATAREP} = 51;
 $constants->{MPI_ERR_UNSUPPORTED_OPERATION} = 52;
 $constants->{MPI_ERR_WIN} = 53;
+# these errors are MPI, but we keep it with high values
+# until standardized so as to avoid changing the ABI
+$constants->{MPI_ERR_PROC_FAILED} = 75;
+$constants->{MPI_ERR_PROC_FAILED_PENDING} = 76;
+$constants->{MPI_ERR_REVOKED} = 77;
 # these error codes will never be returned by a fortran function
 # since there are no fortran bindings for MPI_T
 $constants->{MPI_T_ERR_MEMORY} = 54;
@@ -342,6 +380,8 @@ $constants->{MPI_ERR_RMA_ATTACH} = 69;
 $constants->{MPI_ERR_RMA_FLAVOR} = 70;
 $constants->{MPI_ERR_RMA_SHARED} = 71;
 $constants->{MPI_T_ERR_INVALID} = 72;
+$constants->{MPI_ERR_SESSION} = 78;
+$constants->{MPI_ERR_VALUE_TOO_LARGE} = 79;
 $constants->{MPI_ERR_LASTCODE} = 92;
 
 $constants->{MPI_IDENT} = 0;
@@ -382,6 +422,8 @@ $constants->{OMPI_COMM_TYPE_BOARD} = 8;
 $constants->{OMPI_COMM_TYPE_HOST} = 9;
 $constants->{OMPI_COMM_TYPE_CU} = 10;
 $constants->{OMPI_COMM_TYPE_CLUSTER} = 11;
+$constants->{MPI_COMM_TYPE_HW_UNGUIDED} = 12;
+$constants->{MPI_COMM_TYPE_HW_GUIDED} = 13;
 
 #----------------------------------------------------------------------------
 
@@ -415,7 +457,7 @@ my $header = '! -*- fortran -*-
 ! Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
 !                         University Research and Technology
 !                         Corporation.  All rights reserved.
-! Copyright (c) 2004-2010 The University of Tennessee and The University
+! Copyright (c) 2004-2016 The University of Tennessee and The University
 !                         of Tennessee Research Foundation.  All rights
 !                         reserved.
 ! Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
@@ -423,7 +465,7 @@ my $header = '! -*- fortran -*-
 ! Copyright (c) 2004-2005 The Regents of the University of California.
 !                         All rights reserved.
 ! Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
-! Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+! Copyright (c) 2009-2012 Oak Ridge National Labs.  All rights reserved.
 ! Copyright (c) 2016      Research Organization for Information Science
 !                         and Technology (RIST). All rights reserved.
 ! $COPYRIGHT$
@@ -468,65 +510,7 @@ write_fortran_file($header, $io_constants, $lio_constants,
 
 # Create preprocessor files
 
-my $output = '/* WARNING! THIS IS A GENERATED FILE!!
- * ANY EDITS YOU PUT HERE WILL BE LOST!
- * Instead, edit topdir/ompi/include/mpif-values.pl
- */
-
-/*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
- *                         University Research and Technology
- *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
- *                         of Tennessee Research Foundation.  All rights
- *                         reserved.
- * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
- *                         University of Stuttgart.  All rights reserved.
- * Copyright (c) 2004-2005 The Regents of the University of California.
- *                         All rights reserved.
- * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2009-2012 Los Alamos National Security, LLC.
- *                         All rights reserved.
- * Copyright (c) 2016      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
- * $COPYRIGHT$
- *
- * Additional copyrights may follow
- *
- * $HEADER$
- */
-
-#ifndef USE_MPI_F08_CONSTANTS_H
-#define USE_MPI_F08_CONSTANTS_H
-
-';
-
-foreach my $key (sort(keys(%{$constants}))) {
-    $output .= "#define OMPI_$key $constants->{$key}\n";
-}
-$output .= "\n";
-foreach my $key (sort(keys(%{$handles}))) {
-    $output .= "#define OMPI_$key $handles->{$key}\n";
-}
-
-foreach my $key (sort(keys(%{$io_constants}))) {
-    $output .= "#define OMPI_$key $io_constants->{$key}\n";
-}
-foreach my $key (sort(keys(%{$lio_constants}))) {
-    $output .= "#define OMPI_$key $lio_constants->{$key}\n";
-}
-$output .= "\n";
-foreach my $key (sort(keys(%{$io_handles}))) {
-    $output .= "#define OMPI_$key $io_handles->{$key}\n";
-}
-$output .= "\n";
-$output .= "#endif /* USE_MPI_F08_CONSTANTS_H */\n";
-
-write_file("$topdir/ompi/mpi/fortran/use-mpi-f08/constants.h", $output);
-
-$output = '! WARNING! THIS IS A GENERATED FILE!!
+my $output = '! WARNING! THIS IS A GENERATED FILE!!
 ! ANY EDITS YOU PUT HERE WILL BE LOST!
 ! Instead, edit topdir/ompi/include/mpif-values.pl
 !
@@ -535,7 +519,7 @@ $output = '! WARNING! THIS IS A GENERATED FILE!!
 ! Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
 !                         University Research and Technology
 !                         Corporation.  All rights reserved.
-! Copyright (c) 2004-2006 The University of Tennessee and The University
+! Copyright (c) 2004-2016 The University of Tennessee and The University
 !                         of Tennessee Research Foundation.  All rights
 !                         reserved.
 ! Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
@@ -544,7 +528,7 @@ $output = '! WARNING! THIS IS A GENERATED FILE!!
 !                         All rights reserved.
 ! Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
 ! Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
-! Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+! Copyright (c) 2009-2012 Oak Ridge National Labs.  All rights reserved.
 ! Copyright (c) 2009-2012 Los Alamos National Security, LLC.
 !                         All rights reserved.
 ! Copyright (c) 2016-2019 Research Organization for Information Science
