@@ -13,7 +13,7 @@
  * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2008-2016 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2014      NVIDIA Corporation.  All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
@@ -220,7 +220,7 @@ static int mca_bml_r2_endpoint_add_btl (struct ompi_proc_t *proc, mca_bml_base_e
 
     if ((btl_flags & (MCA_BTL_FLAGS_PUT | MCA_BTL_FLAGS_GET | MCA_BTL_FLAGS_SEND)) == 0) {
         /* If no protocol specified, we have 2 choices: we ignore the BTL
-         * as we don't know which protocl to use, or we suppose that all
+         * as we don't know which protocol to use, or we suppose that all
          * BTLs support the send protocol. This is really a btl error as
          * these flags should have been sanitized by the btl. */
         btl_flags |= MCA_BTL_FLAGS_SEND;
@@ -233,12 +233,14 @@ static int mca_bml_r2_endpoint_add_btl (struct ompi_proc_t *proc, mca_bml_base_e
 
         if (!bml_btl || bml_btl->btl->btl_exclusivity <= btl->btl_exclusivity) {
             /* this btl has higher exclusivity than an existing btl or none exists */
-
-            opal_output_verbose(1, opal_btl_base_framework.framework_output,
-                                "mca: bml: Using %s btl for send to %s on node %s",
-                                btl->btl_component->btl_version.mca_component_name,
-                                OMPI_NAME_PRINT(&proc->super.proc_name),
-                                proc->super.proc_hostname);
+            if (0 < opal_output_get_verbosity(opal_btl_base_framework.framework_output)) {
+                char *errhost = opal_get_proc_hostname(&proc->super);
+                opal_output(0, "mca: bml: Using %s btl for send to %s on node %s",
+                            btl->btl_component->btl_version.mca_component_name,
+                            OMPI_NAME_PRINT(&proc->super.proc_name),
+                            errhost);
+                free(errhost);
+            }
 
             /* cache the endpoint on the proc */
             if (NULL == bml_btl || (bml_btl->btl->btl_exclusivity <= btl->btl_exclusivity)) {
@@ -252,15 +254,16 @@ static int mca_bml_r2_endpoint_add_btl (struct ompi_proc_t *proc, mca_bml_base_e
                  * calculate the bitwise OR of the btl flags
                  */
                 bml_endpoint->btl_flags_or |= bml_btl->btl_flags;
-            } else {
-                opal_output_verbose(20, opal_btl_base_framework.framework_output,
-                                    "mca: bml: Not using %s btl for send to %s on node %s "
-                                    "because %s btl has higher exclusivity (%d > %d)",
-                                    btl->btl_component->btl_version.mca_component_name,
-                                    OMPI_NAME_PRINT(&proc->super.proc_name), proc->super.proc_hostname,
-                                    bml_btl->btl->btl_component->btl_version.mca_component_name,
-                                    bml_btl->btl->btl_exclusivity,
-                                    btl->btl_exclusivity);
+            } else if (19 < opal_output_get_verbosity(opal_btl_base_framework.framework_output)) {
+                char *errhost = opal_get_proc_hostname(&proc->super);
+                opal_output(0, "mca: bml: Not using %s btl for send to %s on node %s "
+                            "because %s btl has higher exclusivity (%d > %d)",
+                            btl->btl_component->btl_version.mca_component_name,
+                            OMPI_NAME_PRINT(&proc->super.proc_name), errhost,
+                            bml_btl->btl->btl_component->btl_version.mca_component_name,
+                            bml_btl->btl->btl_exclusivity,
+                            btl->btl_exclusivity);
+                free(errhost);
             }
 
             btl_in_use = true;
@@ -424,14 +427,16 @@ static int mca_bml_r2_add_proc (struct ompi_proc_t *proc)
         OBJ_RELEASE(bml_endpoint);
         /* no btl is available for this proc */
         if (mca_bml_r2.show_unreach_errors) {
+            char *errhost = opal_get_proc_hostname(&proc->super);
+            char *localhost = opal_get_proc_hostname(&ompi_proc_local_proc->super);
             opal_show_help ("help-mca-bml-r2.txt", "unreachable proc", true,
                             OMPI_NAME_PRINT(&(ompi_proc_local_proc->super.proc_name)),
-                            (NULL != ompi_proc_local_proc->super.proc_hostname ?
-                             ompi_proc_local_proc->super.proc_hostname : "unknown!"),
+                            localhost,
                             OMPI_NAME_PRINT(&(proc->super.proc_name)),
-                            (NULL != proc->super.proc_hostname ?
-                             proc->super.proc_hostname : "unknown!"),
+                            errhost,
                             btl_names);
+            free(errhost);
+            free(localhost);
         }
 
         return OMPI_ERR_UNREACH;
@@ -578,14 +583,16 @@ static int mca_bml_r2_add_procs( size_t nprocs,
         if (NULL == proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML]) {
             ret = OMPI_ERR_UNREACH;
             if (mca_bml_r2.show_unreach_errors) {
+                char *errhost = opal_get_proc_hostname(&proc->super);
+                char *localhost = opal_get_proc_hostname(&ompi_proc_local_proc->super);
                 opal_show_help("help-mca-bml-r2.txt", "unreachable proc", true,
                                OMPI_NAME_PRINT(&(ompi_proc_local_proc->super.proc_name)),
-                               (NULL != ompi_proc_local_proc->super.proc_hostname ?
-                                ompi_proc_local_proc->super.proc_hostname : "unknown!"),
+                               localhost,
                                OMPI_NAME_PRINT(&(proc->super.proc_name)),
-                               (NULL != proc->super.proc_hostname ?
-                                proc->super.proc_hostname : "unknown!"),
+                               errhost,
                                btl_names);
+                free(errhost);
+                free(localhost);
             }
 
             break;
@@ -627,7 +634,7 @@ static int mca_bml_r2_del_procs(size_t nprocs,
             }
 
             /* The reference stored in btl_eager and btl_rdma will automatically
-             * dissapear once the btl_array destructor is called. Thus, there is
+             * disappear once the btl_array destructor is called. Thus, there is
              * no need for extra cleaning here.
              */
         }
@@ -909,7 +916,7 @@ static int mca_bml_r2_register( mca_btl_base_tag_t tag,
 {
     mca_btl_base_active_message_trigger[tag].cbfunc = cbfunc;
     mca_btl_base_active_message_trigger[tag].cbdata = data;
-    /* Give an oportunity to the BTLs to do something special
+    /* Give an opportunity to the BTLs to do something special
      * for each registration.
      */
     {
@@ -980,7 +987,6 @@ mca_bml_r2_module_t mca_bml_r2 = {
         .bml_register = mca_bml_r2_register,
         .bml_register_error = mca_bml_r2_register_error,
         .bml_finalize = mca_bml_r2_finalize,
-        .bml_ft_event = mca_bml_r2_ft_event,
     },
 };
 
