@@ -3,6 +3,9 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2020      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2022      IBM Corporation. All rights reserved
+ * Copyright (c) 2023      Jeffrey M. Squyres.  All rights reserved.
+ * Copyright (c) 2024      NVIDIA CORPORATION. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -36,8 +39,9 @@ int ompi_coll_adapt_ireduce_register(void)
 
     mca_coll_adapt_component.adapt_ireduce_algorithm = 1;
     mca_base_component_var_register(c, "reduce_algorithm",
-                                    "Algorithm of reduce, 1: binomial, 2: in_order_binomial, 3: binary, 4: pipeline, 5: chain, 6: linear", MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
-                                    OPAL_INFO_LVL_5, MCA_BASE_VAR_SCOPE_READONLY,
+                                    "Algorithm of reduce, 1: binomial, 2: in_order_binomial, 3: binary, 4: pipeline, 5: chain, 6: linear",
+                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
+                                    OPAL_INFO_LVL_5, MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_ireduce_algorithm);
     if( (mca_coll_adapt_component.adapt_ireduce_algorithm < 0) ||
         (mca_coll_adapt_component.adapt_ireduce_algorithm > OMPI_COLL_ADAPT_ALGORITHM_COUNT) ) {
@@ -47,58 +51,58 @@ int ompi_coll_adapt_ireduce_register(void)
     mca_coll_adapt_component.adapt_ireduce_segment_size = 163740;
     mca_base_component_var_register(c, "reduce_segment_size",
                                     "Segment size in bytes used by default for reduce algorithms. Only has meaning if algorithm is forced and supports segmenting. 0 bytes means no segmentation.",
-                                    MCA_BASE_VAR_TYPE_SIZE_T, NULL, 0, 0,
+                                    MCA_BASE_VAR_TYPE_SIZE_T, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                     OPAL_INFO_LVL_5,
-                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_ireduce_segment_size);
 
     mca_coll_adapt_component.adapt_ireduce_max_send_requests = 2;
     mca_base_component_var_register(c, "reduce_max_send_requests",
                                     "Maximum number of send requests",
-                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                     OPAL_INFO_LVL_5,
-                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_ireduce_max_send_requests);
 
     mca_coll_adapt_component.adapt_ireduce_max_recv_requests = 3;
     mca_base_component_var_register(c, "reduce_max_recv_requests",
                                     "Maximum number of receive requests per peer",
-                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                     OPAL_INFO_LVL_5,
-                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_ireduce_max_recv_requests);
 
     mca_coll_adapt_component.adapt_inbuf_free_list_min = 10;
     mca_base_component_var_register(c, "inbuf_free_list_min",
                                     "Minimum number of segment in inbuf free list",
-                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                     OPAL_INFO_LVL_5,
-                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_inbuf_free_list_min);
 
     mca_coll_adapt_component.adapt_inbuf_free_list_max = 10000;
     mca_base_component_var_register(c, "inbuf_free_list_max",
                                     "Maximum number of segment in inbuf free list",
-                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                     OPAL_INFO_LVL_5,
-                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_inbuf_free_list_max);
 
 
     mca_coll_adapt_component.adapt_inbuf_free_list_inc = 10;
     mca_base_component_var_register(c, "inbuf_free_list_inc",
                                     "Number of segments to allocate when growing the inbuf free list",
-                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                     OPAL_INFO_LVL_5,
-                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    MCA_BASE_VAR_SCOPE_ALL,
                                     &mca_coll_adapt_component.adapt_inbuf_free_list_inc);
 
     mca_coll_adapt_component.adapt_ireduce_synchronous_send = true;
     (void) mca_base_component_var_register(c, "reduce_synchronous_send",
                                            "Whether to use synchronous send operations during setup of reduce operations",
-                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
                                            OPAL_INFO_LVL_9,
-                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           MCA_BASE_VAR_SCOPE_ALL,
                                            &mca_coll_adapt_component.adapt_ireduce_synchronous_send);
 
     mca_coll_adapt_component.adapt_ireduce_context_free_list = NULL;
@@ -221,7 +225,7 @@ static int send_cb(ompi_request_t * req)
                          context->peer, context->seg_index));
     int err;
 
-    opal_atomic_sub_fetch_32(&(context->con->ongoing_send), 1);
+    opal_atomic_sub_fetch_32((opal_atomic_int32_t*)&(context->con->ongoing_send), 1);
 
     /* Send a new segment */
     ompi_coll_adapt_item_t *item =
@@ -242,7 +246,7 @@ static int send_cb(ompi_request_t * req)
         send_context->peer = context->peer;
         send_context->con = context->con;
 
-        opal_atomic_add_fetch_32(&(context->con->ongoing_send), 1);
+        opal_atomic_add_fetch_32((opal_atomic_int32_t*)&(context->con->ongoing_send), 1);
 
         int send_count = send_context->con->seg_count;
         if (item->id == (send_context->con->num_segs - 1)) {
@@ -267,11 +271,11 @@ static int send_cb(ompi_request_t * req)
         /* Release the item */
         OBJ_RELEASE(item);
 
-        /* Set the send call back */
+        /* Set the send callback */
         ompi_request_set_callback(send_req, send_cb, send_context);
     }
 
-    int32_t num_sent = opal_atomic_add_fetch_32(&(context->con->num_sent_segs), 1);
+    int32_t num_sent = opal_atomic_add_fetch_32((opal_atomic_int32_t*)&(context->con->num_sent_segs), 1);
     OPAL_OUTPUT_VERBOSE((30, mca_coll_adapt_component.adapt_output,
                          "[%d]: In send_cb, root = %d, num_sent = %d, num_segs = %d\n",
                          context->con->rank, context->con->tree->tree_root, num_sent,
@@ -296,7 +300,7 @@ static int send_cb(ompi_request_t * req)
 static int recv_cb(ompi_request_t * req)
 {
     ompi_coll_adapt_reduce_context_t *context = (ompi_coll_adapt_reduce_context_t *) req->req_complete_cb_data;
-    int32_t new_id = opal_atomic_add_fetch_32(&(context->con->next_recv_segs[context->child_id]), 1);
+    int32_t new_id = opal_atomic_add_fetch_32((opal_atomic_int32_t*)&(context->con->next_recv_segs[context->child_id]), 1);
     int err;
 
     OPAL_OUTPUT_VERBOSE((30, mca_coll_adapt_component.adapt_output,
@@ -345,7 +349,7 @@ static int recv_cb(ompi_request_t * req)
         if (MPI_SUCCESS != err) {
             return err;
         }
-        /* Set the receive call back */
+        /* Set the receive callback */
         ompi_request_set_callback(recv_req, recv_cb, recv_context);
     }
 
@@ -420,7 +424,7 @@ static int recv_cb(ompi_request_t * req)
             send_context->seg_index = item->id;
             send_context->peer = context->con->tree->tree_prev;
             send_context->con = context->con;
-            opal_atomic_add_fetch_32(&(context->con->ongoing_send), 1);
+            opal_atomic_add_fetch_32((opal_atomic_int32_t*)&(context->con->ongoing_send), 1);
 
             int send_count = send_context->con->seg_count;
             if (item->id == (send_context->con->num_segs - 1)) {
@@ -441,12 +445,12 @@ static int recv_cb(ompi_request_t * req)
             }
             OBJ_RELEASE(item);
 
-            /* Set the send call back */
+            /* Set the send callback */
             ompi_request_set_callback(send_req, send_cb, send_context);
         }
     }
 
-    int32_t num_recv_segs = opal_atomic_add_fetch_32(&(context->con->num_recv_segs), 1);
+    int32_t num_recv_segs = opal_atomic_add_fetch_32((opal_atomic_int32_t*)&(context->con->num_recv_segs), 1);
     OPAL_OUTPUT_VERBOSE((30, mca_coll_adapt_component.adapt_output,
                          "[%d]: In recv_cb, tree = %p, root = %d, num_recv = %d, num_segs = %d, num_child = %d\n",
                          context->con->rank, (void *) context->con->tree,
@@ -504,7 +508,8 @@ int ompi_coll_adapt_ireduce(const void *sbuf, void *rbuf, int count, struct ompi
 
 
     return ompi_coll_adapt_ireduce_generic(sbuf, rbuf, count, dtype, op, root, comm, request, module,
-                                           adapt_module_cached_topology(module, comm, root, mca_coll_adapt_component.adapt_ireduce_algorithm),
+                                           ompi_coll_adapt_module_cached_topology(module, comm, root,
+                                        		                                            mca_coll_adapt_component.adapt_ireduce_algorithm),
                                            mca_coll_adapt_component.adapt_ireduce_segment_size);
 
 }
@@ -550,7 +555,7 @@ int ompi_coll_adapt_ireduce_generic(const void *sbuf, void *rbuf, int count,
                             mca_coll_adapt_component.adapt_context_free_list_max,
                             mca_coll_adapt_component.adapt_context_free_list_inc,
                             NULL, 0, NULL, NULL, NULL);
-        if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&mca_coll_adapt_component.adapt_ireduce_context_free_list,
+        if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR((opal_atomic_intptr_t *)&mca_coll_adapt_component.adapt_ireduce_context_free_list,
                                                      &(intptr_t){0}, fl) ) {
             OBJ_RELEASE(fl);
         }
@@ -617,7 +622,7 @@ int ompi_coll_adapt_ireduce_generic(const void *sbuf, void *rbuf, int count,
                             mca_coll_adapt_component.adapt_inbuf_free_list_inc,
                             NULL, 0, NULL, NULL, NULL);
         /* Set up next_recv_segs */
-        con->next_recv_segs = (int32_t *) malloc(sizeof(int32_t) * tree->tree_nextsize);
+        con->next_recv_segs = (int32_t *) malloc(sizeof(opal_atomic_int32_t) * tree->tree_nextsize);
         ompi_coll_adapt_inbuf_t *temp_inbuf =
             (ompi_coll_adapt_inbuf_t *) opal_free_list_wait_st(&con->inbuf_list);
         con->distance = (char *) temp_inbuf->buff - lower_bound - (char *) temp_inbuf;       //address of inbuf->buff to address of inbuf
@@ -661,7 +666,6 @@ int ompi_coll_adapt_ireduce_generic(const void *sbuf, void *rbuf, int count,
             con->next_recv_segs[i] = min - 1;
         }
 
-        int num_recvs = 0;
         for (int32_t seg_index = 0; seg_index < min; seg_index++)
         {
             /* For each child */
@@ -709,8 +713,6 @@ int ompi_coll_adapt_ireduce_generic(const void *sbuf, void *rbuf, int count,
                 }
                 /* Set the recv callback */
                 ompi_request_set_callback(recv_req, recv_cb, context);
-
-                ++num_recvs;
             }
         }
     }
@@ -741,7 +743,7 @@ int ompi_coll_adapt_ireduce_generic(const void *sbuf, void *rbuf, int count,
                 (ompi_coll_adapt_reduce_context_t *)opal_free_list_wait(mca_coll_adapt_component.adapt_ireduce_context_free_list);
             context->buff = (char *) sbuf + (ptrdiff_t) seg_index * (ptrdiff_t) segment_increment;
             context->seg_index = seg_index;
-            /* Actural rank of the peer */
+            /* Actual rank of the peer */
             context->peer = tree->tree_prev;
             context->con = con;
             context->inbuf = NULL;
