@@ -17,6 +17,7 @@
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2018      Siberian State University of Telecommunications
  *                         and Information Science. All rights reserved.
+ * Copyright (c) 2022      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -424,7 +425,7 @@ ompi_coll_base_allreduce_intra_ring(const void *sbuf, void *rbuf, int count,
        - wait on block (r + 1)
        - compute on block (r + 1)
        - send block (r + 1) to rank (r + 1)
-       Note that we must be careful when computing the begining of buffers and
+       Note that we must be careful when computing the beginning of buffers and
        for send operations and computation we must compute the exact block size.
     */
     send_to = (rank + 1) % size;
@@ -717,7 +718,7 @@ ompi_coll_base_allreduce_intra_ring_segmented(const void *sbuf, void *rbuf, int
            - wait on block (r + 1)
            - compute on block (r + 1)
            - send block (r + 1) to rank (r + 1)
-           Note that we must be careful when computing the begining of buffers and
+           Note that we must be careful when computing the beginning of buffers and
            for send operations and computation we must compute the exact block size.
         */
         send_to = (rank + 1) % size;
@@ -982,7 +983,9 @@ int ompi_coll_base_allreduce_intra_redscat_allgather(
 
     /* Find nearest power-of-two less than or equal to comm_size */
     int nsteps = opal_hibit(comm_size, comm->c_cube_dim + 1);   /* ilog2(comm_size) */
-    assert(nsteps >= 0);
+    if (-1 == nsteps) {
+        return MPI_ERR_ARG;
+    }
     int nprocs_pof2 = 1 << nsteps;                              /* flp2(comm_size) */
 
     if (count < nprocs_pof2 || !ompi_op_is_commute(op)) {
@@ -1122,7 +1125,7 @@ int ompi_coll_base_allreduce_intra_redscat_allgather(
 
         for (int mask = 1; mask < nprocs_pof2; mask <<= 1) {
             /*
-             * On each iteration: rindex[step] = sindex[step] -- begining of the
+             * On each iteration: rindex[step] = sindex[step] -- beginning of the
              * current window. Length of the current window is storded in wsize.
              */
             int vdest = vrank ^ mask;
