@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2020 The University of Tennessee and The University
+ * Copyright (c) 2004-2022 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,6 +11,10 @@
  *                         All rights reserved.
  * Copyright (c) 2014-2020 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2023      Jeffrey M. Squyres.  All rights reserved.
+ *
+ * Copyright (c) 2024      NVIDIA CORPORATION. All rights reserved.
+ *
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -78,6 +82,26 @@ int ompi_coll_base_sendrecv_actual( const void* sendbuf, size_t scount,
     if (MPI_STATUS_IGNORE != status) {
         status->MPI_ERROR = err;
     }
+    if( MPI_REQUEST_NULL != req ) {
+#if OPAL_ENABLE_FT_MPI
+        if( MPI_ERR_PROC_FAILED == req->req_status.MPI_ERROR
+         || MPI_ERR_PROC_FAILED_PENDING == req->req_status.MPI_ERROR
+         || MPI_ERR_REVOKED == req->req_status.MPI_ERROR ) {
+            /* We cannot just 'free' and forget, as the PML/BTLS would still
+             * be updating the request buffer after we return from the MPI
+             * call!
+             * For other errors that do not have a well defined post-error
+             * behavior, calling the cancel/wait could deadlock, so we just
+             * free, as this is the best that can be done in this case. */
+            ompi_request_cancel(req);
+            ompi_request_wait(&req, MPI_STATUS_IGNORE);
+            if( MPI_ERR_PROC_FAILED_PENDING == err ) {
+                err = MPI_ERR_PROC_FAILED;
+            }
+        } else /* this 'else' intentionally spills outside the ifdef */
+#endif /* OPAL_ENABLE_FT_MPI */
+        ompi_request_free(&req);
+    }
     return (err);
 }
 
@@ -107,14 +131,29 @@ int ompi_rounddown(int num, int factor)
     return num * factor;    /* floor(num / factor) * factor */
 }
 
-static void release_objs_callback(struct ompi_coll_base_nbc_request_t *request) {
-    if (NULL != request->data.objs.objs[0]) {
-        OBJ_RELEASE(request->data.objs.objs[0]);
-        request->data.objs.objs[0] = NULL;
+/**
+ * Release all objects and arrays stored into the nbc_request.
+ * The release_arrays are temporary memory to stored the values
+ * converted from Fortran, and should disappear in same time as the
+ * request itself.
+ */
+static void
+release_objs_callback(struct ompi_coll_base_nbc_request_t *request)
+{
+    if (NULL != request->data.refcounted.objs.objs[0]) {
+        OBJ_RELEASE(request->data.refcounted.objs.objs[0]);
+        request->data.refcounted.objs.objs[0] = NULL;
     }
-    if (NULL != request->data.objs.objs[1]) {
-        OBJ_RELEASE(request->data.objs.objs[1]);
-        request->data.objs.objs[1] = NULL;
+    if (NULL != request->data.refcounted.objs.objs[1]) {
+        OBJ_RELEASE(request->data.refcounted.objs.objs[1]);
+        request->data.refcounted.objs.objs[1] = NULL;
+    }
+    for(int i = 0; i < OMPI_REQ_NB_RELEASE_ARRAYS; i++ ) {
+        if (NULL == request->data.release_arrays[i]) {
+            break;
+        }
+        free(request->data.release_arrays[i]);
+        request->data.release_arrays[i] = NULL;
     }
 }
 
@@ -148,12 +187,12 @@ int ompi_coll_base_retain_op( ompi_request_t *req, ompi_op_t *op,
     }
     if (!ompi_op_is_intrinsic(op)) {
         OBJ_RETAIN(op);
-        request->data.op.op = op;
+        request->data.refcounted.op.op = op;
         retain = true;
     }
     if (!ompi_datatype_is_predefined(type)) {
         OBJ_RETAIN(type);
-        request->data.op.datatype = type;
+        request->data.refcounted.op.datatype = type;
         retain = true;
     }
     if (OPAL_UNLIKELY(retain)) {
@@ -187,12 +226,12 @@ int ompi_coll_base_retain_datatypes( ompi_request_t *req, ompi_datatype_t *stype
     }
     if (NULL != stype && !ompi_datatype_is_predefined(stype)) {
         OBJ_RETAIN(stype);
-        request->data.types.stype = stype;
+        request->data.refcounted.types.stype = stype;
         retain = true;
     }
     if (NULL != rtype && !ompi_datatype_is_predefined(rtype)) {
         OBJ_RETAIN(rtype);
-        request->data.types.rtype = rtype;
+        request->data.refcounted.types.rtype = rtype;
         retain = true;
     }
     if (OPAL_UNLIKELY(retain)) {
@@ -209,29 +248,25 @@ int ompi_coll_base_retain_datatypes( ompi_request_t *req, ompi_datatype_t *stype
     return OMPI_SUCCESS;
 }
 
-static void release_vecs_callback(ompi_coll_base_nbc_request_t *request) {
-    ompi_communicator_t *comm = request->super.req_mpi_object.comm;
-    int scount, rcount;
-    if (OMPI_COMM_IS_TOPO(comm)) {
-        (void)mca_topo_base_neighbor_count (comm, &rcount, &scount);
-    } else {
-        scount = rcount = OMPI_COMM_IS_INTER(comm)?ompi_comm_remote_size(comm):ompi_comm_size(comm);
-    }
-    if (NULL != request->data.vecs.stypes) {
-        for (int i=0; i<scount; i++) {
-            if (NULL != request->data.vecs.stypes[i]) {
-                OMPI_DATATYPE_RELEASE_NO_NULLIFY(request->data.vecs.stypes[i]);
+static void release_vecs_callback(ompi_coll_base_nbc_request_t *request)
+{
+    if (NULL != request->data.refcounted.vecs.stypes) {
+        for (int i = 0; i < request->data.refcounted.vecs.scount; i++) {
+            if (NULL != request->data.refcounted.vecs.stypes[i] &&
+                !ompi_datatype_is_predefined(request->data.refcounted.vecs.stypes[i])) {
+                OMPI_DATATYPE_RELEASE_NO_NULLIFY(request->data.refcounted.vecs.stypes[i]);
             }
         }
-        request->data.vecs.stypes = NULL;
+        request->data.refcounted.vecs.stypes = NULL;
     }
-    if (NULL != request->data.vecs.rtypes) {
-        for (int i=0; i<rcount; i++) {
-            if (NULL != request->data.vecs.rtypes[i]) {
-                OMPI_DATATYPE_RELEASE_NO_NULLIFY(request->data.vecs.rtypes[i]);
+    if (NULL != request->data.refcounted.vecs.rtypes) {
+        for (int i = 0; i < request->data.refcounted.vecs.rcount; i++) {
+            if (NULL != request->data.refcounted.vecs.rtypes[i] &&
+                !ompi_datatype_is_predefined(request->data.refcounted.vecs.rtypes[i])) {
+                OMPI_DATATYPE_RELEASE_NO_NULLIFY(request->data.refcounted.vecs.rtypes[i]);
             }
         }
-        request->data.vecs.rtypes = NULL;
+        request->data.refcounted.vecs.rtypes = NULL;
     }
 }
 
@@ -257,35 +292,47 @@ static int free_vecs_callback(struct ompi_request_t **rptr) {
 }
 
 int ompi_coll_base_retain_datatypes_w( ompi_request_t *req,
-                                       ompi_datatype_t *const stypes[], ompi_datatype_t *const rtypes[]) {
+                                       ompi_datatype_t * const stypes[],
+                                       ompi_datatype_t * const rtypes[],
+                                       bool use_topo)
+{
     ompi_coll_base_nbc_request_t *request = (ompi_coll_base_nbc_request_t *)req;
-    bool retain = false;
     ompi_communicator_t *comm = request->super.req_mpi_object.comm;
     int scount, rcount;
+
     if (REQUEST_COMPLETE(req)) {
         return OMPI_SUCCESS;
     }
-    if (OMPI_COMM_IS_TOPO(comm)) {
+
+    if (use_topo && OMPI_COMM_IS_TOPO(comm)) {
         (void)mca_topo_base_neighbor_count (comm, &rcount, &scount);
     } else {
         scount = rcount = OMPI_COMM_IS_INTER(comm)?ompi_comm_remote_size(comm):ompi_comm_size(comm);
     }
 
-    for (int i=0; i<scount; i++) {
-        if (NULL != stypes && NULL != stypes[i] && !ompi_datatype_is_predefined(stypes[i])) {
-            OBJ_RETAIN(stypes[i]);
-            retain = true;
+    request->data.refcounted.vecs.scount = 0;  /* default value */
+    if (NULL != stypes) {
+        for (int i = 0; i < scount; i++) {
+            if (NULL != stypes[i] && !ompi_datatype_is_predefined(stypes[i])) {
+                OBJ_RETAIN(stypes[i]);
+                request->data.refcounted.vecs.scount = i;  /* last valid type */
+            }
         }
     }
-    for (int i=0; i<rcount; i++) {
-        if (NULL != rtypes && NULL != rtypes[i] && !ompi_datatype_is_predefined(rtypes[i])) {
-            OBJ_RETAIN(rtypes[i]);
-            retain = true;
+    request->data.refcounted.vecs.rcount = 0;  /* default value */
+    if (NULL != rtypes) {
+        for (int i = 0; i < rcount; i++) {
+            if (NULL != rtypes[i] && !ompi_datatype_is_predefined(rtypes[i])) {
+                OBJ_RETAIN(rtypes[i]);
+                request->data.refcounted.vecs.rcount = i;  /* last valid type */
+            }
         }
     }
-    if (OPAL_UNLIKELY(retain)) {
-        request->data.vecs.stypes = stypes;
-        request->data.vecs.rtypes = rtypes;
+    if (OPAL_LIKELY(request->data.refcounted.vecs.scount | request->data.refcounted.vecs.rcount) ) {
+        request->data.refcounted.vecs.stypes = (ompi_datatype_t **) stypes;
+        request->data.refcounted.vecs.rtypes = (ompi_datatype_t **) rtypes;
+        request->data.refcounted.vecs.scount = scount;
+        request->data.refcounted.vecs.rcount = rcount;
         if (req->req_persistent) {
             request->cb.req_free = req->req_free;
             req->req_free = free_vecs_callback;
@@ -299,15 +346,16 @@ int ompi_coll_base_retain_datatypes_w( ompi_request_t *req,
     return OMPI_SUCCESS;
 }
 
-static void nbc_req_cons(ompi_coll_base_nbc_request_t *req)
+static void nbc_req_constructor(ompi_coll_base_nbc_request_t *req)
 {
     req->cb.req_complete_cb = NULL;
     req->req_complete_cb_data = NULL;
-    req->data.objs.objs[0] = NULL;
-    req->data.objs.objs[1] = NULL;
+    req->data.refcounted.objs.objs[0] = NULL;
+    req->data.refcounted.objs.objs[1] = NULL;
+    req->data.release_arrays[0] = NULL;
 }
 
-OBJ_CLASS_INSTANCE(ompi_coll_base_nbc_request_t, ompi_request_t, nbc_req_cons, NULL);
+OBJ_CLASS_INSTANCE(ompi_coll_base_nbc_request_t, ompi_request_t, nbc_req_constructor, NULL);
 
 /* File reading functions */
 static void skiptonewline (FILE *fptr, int *fileline)
@@ -354,7 +402,7 @@ int ompi_coll_base_file_getnext_long(FILE *fptr, int *fileline, long* val)
 
 int ompi_coll_base_file_getnext_string(FILE *fptr, int *fileline, char** val)
 {
-    char trash, token[32];
+    char trash, token[33];
     int rc;
 
     *val = NULL;  /* security in case we fail */
@@ -437,6 +485,26 @@ int ompi_coll_base_file_peek_next_char_is(FILE *fptr, int *fileline, int expecte
     } while (1);
 }
 
+/**
+ * return non-zero if the next non-space to read on the current line is a digit.
+ * otherwise return 0.
+ */
+int ompi_coll_base_file_peek_next_char_isdigit(FILE *fptr)
+{
+    do {
+        int next = fgetc(fptr);
+
+        if ((' ' == next) || ('\t' == next)) {
+            continue; /* discard space and tab. keep everything else */
+        }
+
+        ungetc(next, fptr); /* put the char back into the stream */
+
+        return isdigit(next); /* report back whether or not next is a digit */
+
+    } while (1);
+}
+
 /**
  * There are certainly simpler implementation for this function when performance
  * is not a critical point. But, as this function is used during the collective
