@@ -3,6 +3,7 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2014-2015 NVIDIA Corporation.  All rights reserved.
+ * Copyright (c) 2022      Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -17,7 +18,6 @@
 
 #include "ompi/op/op.h"
 #include "opal/datatype/opal_convertor.h"
-#include "opal/datatype/opal_datatype_cuda.h"
 
 int mca_coll_cuda_exscan(const void *sbuf, void *rbuf, int count,
                          struct ompi_datatype_t *dtype,
@@ -32,23 +32,30 @@ int mca_coll_cuda_exscan(const void *sbuf, void *rbuf, int count,
     int rc;
 
     bufsize = opal_datatype_span(&dtype->super, count, &gap);
+    rc = mca_coll_cuda_check_buf((void *)sbuf);
+    if (rc < 0) {
+        return rc;
+    }
 
-    if ((MPI_IN_PLACE != sbuf) && (opal_cuda_check_bufs((char *)sbuf, NULL))) {
+    if ((MPI_IN_PLACE != sbuf) && rc > 0) {
         sbuf1 = (char*)malloc(bufsize);
         if (NULL == sbuf1) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
-        opal_cuda_memcpy_sync(sbuf1, sbuf, bufsize);
+        mca_coll_cuda_memcpy(sbuf1, sbuf, bufsize);
         sbuf = sbuf1 - gap;
     }
-
-    if (opal_cuda_check_bufs(rbuf, NULL)) {
+    rc = mca_coll_cuda_check_buf(rbuf);
+    if (rc < 0) {
+        return rc;
+    }
+    if (rc > 0) {
         rbuf1 = (char*)malloc(bufsize);
         if (NULL == rbuf1) {
             if (NULL != sbuf1) free(sbuf1);
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
-        opal_cuda_memcpy_sync(rbuf1, rbuf, bufsize);
+        mca_coll_cuda_memcpy(rbuf1, rbuf, bufsize);
         rbuf2 = rbuf; /* save away original buffer */
         rbuf = rbuf1 - gap;
     }
@@ -60,7 +67,7 @@ int mca_coll_cuda_exscan(const void *sbuf, void *rbuf, int count,
     }
     if (NULL != rbuf1) {
         rbuf = rbuf2;
-        opal_cuda_memcpy_sync(rbuf, rbuf1, bufsize);
+        mca_coll_cuda_memcpy(rbuf, rbuf1, bufsize);
         free(rbuf1);
     }
     return rc;
