@@ -3,7 +3,8 @@
  * Copyright (c) 2018-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2020      Bull S.A.S. All rights reserved.
+ * Copyright (c) 2022      IBM Corporation. All rights reserved
+ * Copyright (c) 2020-2022 Bull S.A.S. All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -12,6 +13,11 @@
  * $HEADER$
  */
 
+/*
+ *@file
+ * Implementation of configuration file parser to set collective components to use.
+ */
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -23,6 +29,7 @@
 #include "coll_han.h"
 #include "coll_han_dynamic.h"
 #include "coll_han_dynamic_file.h"
+#include "coll_han_algorithms.h"
 
 #include "ompi/mca/coll/base/coll_base_util.h"
 
@@ -35,6 +42,12 @@ static void check_dynamic_rules(void);
 /* Current file line for verbose message */
 static int fileline = 1;
 
+/*
+ * File parsing function. Allocated memory depending on the number of rules.
+ * This functions expects a file formatted as described in coll_han_dynamic_file.h.
+ * The configuration is then used by coll/han component to determine which module to
+ * use at each topological level.
+ */
 int
 mca_coll_han_init_dynamic_rules(void)
 {
@@ -46,30 +59,32 @@ mca_coll_han_init_dynamic_rules(void)
     /* Loop counters */
     int i, j, k, l;
 
-    /* Collective informations */
-    long nb_coll, coll_id;
+    /* Collective information */
+    long nb_coll;
+    COLLTYPE_T coll_id;
+    int algorithm_id;
     char * coll_name = NULL;
+    char * algorithm_name = NULL;
     collective_rule_t *coll_rules;
 
-    /* Topo informations */
+    /* Topo information */
     long nb_topo, topo_lvl;
     topologic_rule_t *topo_rules;
 
-    /* Configuration informations */
+    /* Configuration information */
     long nb_rules, conf_size;
     configuration_rule_t *conf_rules;
 
-    /* Message size informations */
+    /* Message size information */
     long nb_msg_size;
     size_t msg_size;
     msg_size_rule_t *msg_size_rules;
 
-    /* Component informations */
+    /* Component information */
     long component;
 
     /* If the dynamic rules are not used, do not even read the file */
     if(!mca_coll_han_component.use_dynamic_file_rules) {
-        nb_coll = 0;
         return OMPI_SUCCESS;
     }
 
@@ -81,7 +96,8 @@ mca_coll_han_init_dynamic_rules(void)
         return OMPI_SUCCESS;
     }
 
-    if( NULL == (fptr = fopen(fname, "r")) ) {
+    fptr = fopen(fname, "r");
+    if( NULL == fptr ) {
         opal_output_verbose(5, mca_coll_han_component.han_output,
                             "coll:han:mca_coll_han_init_dynamic_rules cannot open dynamic file provided by "
                             "coll_han_dynamic_rules_filename=%s. Make sure it provides the  full path and "
@@ -115,6 +131,7 @@ mca_coll_han_init_dynamic_rules(void)
     /* Iterates on collective rules */
     for( i = 0 ; i < nb_coll ; i++ ) {
         coll_rules[i].nb_topologic_levels = 0;
+        coll_rules[i].topologic_rules = NULL;
         mca_coll_han_component.dynamic_rules.nb_collectives = i+1;
 
         /* Get the collective identifier */
@@ -147,7 +164,7 @@ mca_coll_han_init_dynamic_rules(void)
         if(!mca_coll_han_is_coll_dynamic_implemented(coll_id)) {
             opal_output_verbose(5, mca_coll_han_component.han_output,
                                 "coll:han:mca_coll_han_init_dynamic_rules found an error on dynamic rules file %s "
-                                "read collective id %ld at line %d but this collective is not implemented yet. "
+                                "read collective id %d at line %d but this collective is not implemented yet. "
                                 "This is not an error but this set of rules will not be used\n",
                                 fname, coll_id, fileline);
         }
@@ -165,7 +182,7 @@ mca_coll_han_init_dynamic_rules(void)
             goto file_reading_error;
         }
 
-        /* Store the collective rule informations */
+        /* Store the collective rule information */
         coll_rules[i].nb_topologic_levels = nb_topo;
         coll_rules[i].collective_id = (COLLTYPE_T)coll_id;
 
@@ -188,17 +205,33 @@ mca_coll_han_init_dynamic_rules(void)
         /* Iterates on topologic rules */
         for( j = 0 ; j < nb_topo ; j++ ) {
             topo_rules[j].nb_rules = 0;
+            topo_rules[j].configuration_rules = NULL;
             coll_rules[i].nb_topologic_levels = j+1;
 
             /* Get the topologic level identifier */
-            if( (getnext_long(fptr, &topo_lvl) < 0) || (topo_lvl < INTRA_NODE) || (topo_lvl >= NB_TOPO_LVL) ) {
+            char *topo_lvl_name = NULL;
+            if( getnext_string(fptr, &topo_lvl_name) < 0 ) {
                 opal_output_verbose(5, mca_coll_han_component.han_output,
                                     "coll:han:mca_coll_han_init_dynamic_rules found an error on dynamic rules file %s "
-                                    "at line %d: an invalid topo level %ld is given or the reader encountered an unexpected EOF. "
-                                    "Topologic level must be at least %d and less than %d\n",
-                                    fname, fileline, topo_lvl, INTRA_NODE, NB_TOPO_LVL);
+                                    "at line %d: cannot read the name/id of a topo level\n",
+                                    fname, fileline);
                 goto file_reading_error;
             }
+            topo_lvl = mca_coll_han_topo_lvl_name_to_id(topo_lvl_name);
+            if (topo_lvl < 0) {
+                char *endp;
+                topo_lvl = (int)strtol(topo_lvl_name, &endp, 10);
+                if (('\0' != *endp ) || (topo_lvl < INTRA_NODE) || (topo_lvl >= NB_TOPO_LVL)) {
+                    opal_output_verbose(5, mca_coll_han_component.han_output,
+                                        "coll:han:mca_coll_han_init_dynamic_rules found an error on dynamic rules file %s "
+                                        "at line %d: unknown topo level '%s'\n",
+                                        fname, fileline, topo_lvl_name);
+                    free(topo_lvl_name);
+                    topo_lvl_name = NULL;
+                    goto file_reading_error;
+                }
+            }
+            free (topo_lvl_name);
 
             /*
              * The first information of a topologic rule
@@ -214,7 +247,7 @@ mca_coll_han_init_dynamic_rules(void)
                 goto file_reading_error;
             }
 
-            /* Store the topologic rule informations */
+            /* Store the topologic rule information */
             topo_rules[j].collective_id = coll_id;
             topo_rules[j].topologic_level = (TOPO_LVL_T)topo_lvl;
             topo_rules[j].nb_rules = nb_rules;
@@ -238,6 +271,7 @@ mca_coll_han_init_dynamic_rules(void)
             /* Iterate on configuration rules */
             for( k = 0; k < nb_rules; k++ ) {
                 conf_rules[k].nb_msg_size = 0;
+                conf_rules[k].msg_size_rules = NULL;
                 topo_rules[j].nb_rules = k+1;
 
                 /* Get the configuration size */
@@ -320,15 +354,53 @@ mca_coll_han_init_dynamic_rules(void)
                                             "least %d and less than %d\n",
                                             fname, fileline, target_comp_name, SELF, COMPONENTS_COUNT);
                         free(target_comp_name);
+                        target_comp_name = NULL;
                         goto file_reading_error;
                     }
 
+                    /* Get the optionnal algorithm for han  */
+                    algorithm_id = 0; // default for all collectives
+                    if ((component == HAN) && (1 == ompi_coll_base_file_peek_next_char_is(fptr, &fileline, '@')) ) {
+
+                        if( getnext_string(fptr, &algorithm_name) < 0 ) {
+                            opal_output_verbose(5, mca_coll_han_component.han_output,
+                                                "coll:han:mca_coll_han_init_dynamic_rules found an error on dynamic rules file %s "
+                                                "at line %d: cannot read the name/id of an algorithm\n",
+                                                fname, fileline);
+                            free(target_comp_name);
+                            target_comp_name = NULL;
+                            goto file_reading_error;
+                        }
+                        algorithm_id = mca_coll_han_algorithm_name_to_id(coll_id, algorithm_name);
+                        if (algorithm_id < 0) {
+                            char *endp;
+                            algorithm_id = (int)strtol(algorithm_name, &endp, 10);
+                            char endc = *endp;
+                            free(algorithm_name);
+                            algorithm_name = NULL;
+                            if (('\0' != endc ) || !mca_coll_han_algorithm_id_is_valid(coll_id, algorithm_id)) {
+                                opal_output_verbose(5, mca_coll_han_component.han_output,
+                                                    "coll:han:mca_coll_han_init_dynamic_rules found an error on dynamic rules file %s "
+                                                    "at line %d: unknown algorithm '%s' for %s\n",
+                                                    fname, fileline, algorithm_name, coll_name);
+                                free(target_comp_name);
+                                target_comp_name = NULL;
+                                goto file_reading_error;
+                            }
+                        }
+                        opal_output_verbose(5, mca_coll_han_component.han_output,
+                                            "coll:han:mca_coll_han_init_dynamic_rules found for coll=%s msg_size=%ld : algorithm '%s' %d\n",
+                                            coll_name, msg_size, algorithm_name, algorithm_id);
+                    }
+
+
                     /* Store message size rule information */
                     msg_size_rules[l].collective_id = coll_id;
                     msg_size_rules[l].topologic_level = topo_lvl;
                     msg_size_rules[l].configuration_size = conf_size;
                     msg_size_rules[l].msg_size = msg_size;
                     msg_size_rules[l].component = (COMPONENT_T)component;
+                    msg_size_rules[l].algorithm_id = algorithm_id;
 
                     nb_entries++;
                     /* do we have the optional segment length */
@@ -383,6 +455,7 @@ mca_coll_han_init_dynamic_rules(void)
     fclose(fptr);
 
     check_dynamic_rules();
+    free(algorithm_name);
     return OMPI_SUCCESS;
 
 cannot_allocate:
@@ -395,6 +468,7 @@ cannot_allocate:
     if( NULL != coll_name ) {
         free(coll_name);
     }
+    free(algorithm_name);
     fclose (fptr);
     /* We disable the module, we don't need to keep the rules */
     mca_coll_han_free_dynamic_rules();
@@ -410,12 +484,16 @@ file_reading_error:
     if( NULL != coll_name ) {
         free(coll_name);
     }
+    free(algorithm_name);
     fclose (fptr);
     /* We disable the module, we don't need to keep the rules */
     mca_coll_han_free_dynamic_rules();
     return OMPI_SUCCESS;
 }
 
+/*
+ * Memory free all the rules parsed in the file
+ */
 void
 mca_coll_han_free_dynamic_rules(void)
 {
@@ -423,7 +501,7 @@ mca_coll_han_free_dynamic_rules(void)
     int i, j, k;
 
     /* Loop ranges */
-    int nb_coll, nb_topo, nb_conf;
+    int nb_coll, nb_conf;
 
     /* Aliases */
     collective_rule_t *coll_rules;
@@ -434,7 +512,7 @@ mca_coll_han_free_dynamic_rules(void)
     coll_rules = mca_coll_han_component.dynamic_rules.collective_rules;
 
     for(i=0 ; i<nb_coll ; i++) {
-        nb_topo = coll_rules[i].nb_topologic_levels;
+        int nb_topo = coll_rules[i].nb_topologic_levels;
         topo_rules = coll_rules[i].topologic_rules;
 
         for(j=0 ; j<nb_topo ; j++) {
@@ -472,26 +550,25 @@ static void check_dynamic_rules(void)
     /* Loop counters */
     int i, j, k, l;
 
-    /* Collective informations */
+    /* Collective information */
     int nb_coll;
     COLLTYPE_T coll_id;
     collective_rule_t *coll_rules;
 
-    /* Topo informations */
-    int nb_topo;
+    /* Topo information */
     TOPO_LVL_T topo_lvl;
     topologic_rule_t *topo_rules;
 
-    /* Configuration informations */
+    /* Configuration information */
     int nb_rules, conf_size;
     configuration_rule_t *conf_rules;
 
-    /* Message size informations */
+    /* Message size information */
     int nb_msg_size;
     size_t msg_size;
     msg_size_rule_t *msg_size_rules;
 
-    /* Component informations */
+    /* Component information */
     COMPONENT_T component;
 
     nb_coll = mca_coll_han_component.dynamic_rules.nb_collectives;
@@ -499,7 +576,7 @@ static void check_dynamic_rules(void)
 
     for( i = 0; i < nb_coll; i++ ) {
         coll_id = coll_rules[i].collective_id;
-        nb_topo = coll_rules[i].nb_topologic_levels;
+        int nb_topo = coll_rules[i].nb_topologic_levels;
         topo_rules = coll_rules[i].topologic_rules;
 
         for( j = 0; j < nb_topo; j++ ) {
@@ -547,29 +624,29 @@ static void check_dynamic_rules(void)
     }
 }
 
+/* Print configurations parsed from the file */
 void mca_coll_han_dump_dynamic_rules(void)
 {
     int nb_entries = 0;
 
-    /* Collective informations */
+    /* Collective information */
     int nb_coll;
     COLLTYPE_T coll_id;
     collective_rule_t *coll_rules;
 
-    /* Topo informations */
-    int nb_topo;
+    /* Topo information */
     TOPO_LVL_T topo_lvl;
     topologic_rule_t *topo_rules;
 
-    /* Configuration informations */
+    /* Configuration information */
     int nb_rules, conf_size;
     configuration_rule_t *conf_rules;
 
-    /* Message size informations */
+    /* Message size information */
     int nb_msg_size, msg_size;
     msg_size_rule_t *msg_size_rules;
 
-    /* Component informations */
+    /* Component information */
     COMPONENT_T component;
 
     nb_coll = mca_coll_han_component.dynamic_rules.nb_collectives;
@@ -577,7 +654,7 @@ void mca_coll_han_dump_dynamic_rules(void)
 
     for(int i = 0; i < nb_coll; i++ ) {
         coll_id = coll_rules[i].collective_id;
-        nb_topo = coll_rules[i].nb_topologic_levels;
+        int nb_topo = coll_rules[i].nb_topologic_levels;
         topo_rules = coll_rules[i].topologic_rules;
 
         for(int j = 0; j < nb_topo; j++ ) {
@@ -597,10 +674,10 @@ void mca_coll_han_dump_dynamic_rules(void)
                     opal_output(mca_coll_han_component.han_output,
                                 "coll:han:dump_dynamic_rules %d collective %d (%s) "
                                 "topology level %d (%s) configuration size %d "
-                                "mesage size %d -> collective component %d (%s)\n",
+                                "message size %d -> collective component %d (%s)\n",
                                 nb_entries, coll_id, mca_coll_base_colltype_to_str(coll_id),
                                 topo_lvl, mca_coll_han_topo_lvl_to_str(topo_lvl), conf_size,
-                                msg_size, component, available_components[component].component_name);
+                                msg_size, component, ompi_coll_han_available_components[component].component_name);
 
                     nb_entries++;
                 }
