@@ -13,6 +13,7 @@
  * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2023      Jeffrey M. Squyres.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -49,6 +50,7 @@ mca_coll_inter_allreduce_inter(const void *sbuf, void *rbuf, int count,
 {
     int err, rank, root = 0;
     char *tmpbuf = NULL, *pml_buffer = NULL;
+    const char *source;
     ptrdiff_t gap, span;
 
     rank = ompi_comm_rank(comm);
@@ -58,20 +60,21 @@ mca_coll_inter_allreduce_inter(const void *sbuf, void *rbuf, int count,
 
     tmpbuf = (char *) malloc(span);
     if (NULL == tmpbuf) {
-	return OMPI_ERR_OUT_OF_RESOURCE;
+        return OMPI_ERR_OUT_OF_RESOURCE;
     }
     pml_buffer = tmpbuf - gap;
+    source = (MPI_IN_PLACE == sbuf) ? rbuf : sbuf;
 
-    err = comm->c_local_comm->c_coll->coll_reduce(sbuf, pml_buffer, count,
-						 dtype, op, root,
-						 comm->c_local_comm,
-                                                 comm->c_local_comm->c_coll->coll_reduce_module);
+    err = comm->c_local_comm->c_coll->coll_reduce(source, pml_buffer, count,
+                                                  dtype, op, root,
+                                                  comm->c_local_comm,
+                                                  comm->c_local_comm->c_coll->coll_reduce_module);
     if (OMPI_SUCCESS != err) {
-	goto exit;
+        goto exit;
     }
 
     if (rank == root) {
-	/* Do a send-recv between the two root procs. to avoid deadlock */
+        /* Do a send-recv between the two root procs. to avoid deadlock */
         err = ompi_coll_base_sendrecv_actual(pml_buffer, count, dtype, 0,
                                              MCA_COLL_BASE_TAG_ALLREDUCE,
                                              rbuf, count, dtype, 0,
@@ -84,8 +87,8 @@ mca_coll_inter_allreduce_inter(const void *sbuf, void *rbuf, int count,
 
     /* bcast the message to all the local processes */
     err = comm->c_local_comm->c_coll->coll_bcast(rbuf, count, dtype,
-						root, comm->c_local_comm,
-                                                comm->c_local_comm->c_coll->coll_bcast_module);
+                                                 root, comm->c_local_comm,
+                                                 comm->c_local_comm->c_coll->coll_bcast_module);
     if (OMPI_SUCCESS != err) {
             goto exit;
     }
