@@ -61,7 +61,7 @@ int NBC_Reduce_args_compare(NBC_Reduce_args *a, NBC_Reduce_args *b, void *param)
 /* the non-blocking reduce */
 static int nbc_reduce_init(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
                            MPI_Op op, int root, struct ompi_communicator_t *comm, ompi_request_t ** request,
-                           struct mca_coll_base_module_2_3_0_t *module, bool persistent) {
+                           mca_coll_base_module_t *module, bool persistent) {
   int rank, p, res, segsize;
   size_t size;
   MPI_Aint ext;
@@ -239,7 +239,7 @@ static int nbc_reduce_init(const void* sendbuf, void* recvbuf, int count, MPI_Da
 
 int ompi_coll_libnbc_ireduce(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
                              MPI_Op op, int root, struct ompi_communicator_t *comm, ompi_request_t ** request,
-                             struct mca_coll_base_module_2_3_0_t *module) {
+                             mca_coll_base_module_t *module) {
     int res = nbc_reduce_init(sendbuf, recvbuf, count, datatype, op, root,
                               comm, request, module, false);
     if (OPAL_LIKELY(OMPI_SUCCESS != res)) {
@@ -257,7 +257,7 @@ int ompi_coll_libnbc_ireduce(const void* sendbuf, void* recvbuf, int count, MPI_
 
 static int nbc_reduce_inter_init(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
                                  MPI_Op op, int root, struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                 struct mca_coll_base_module_2_3_0_t *module, bool persistent) {
+                                 mca_coll_base_module_t *module, bool persistent) {
   int rank, res, rsize;
   NBC_Schedule *schedule;
   ompi_coll_libnbc_module_t *libnbc_module = (ompi_coll_libnbc_module_t*) module;
@@ -305,7 +305,7 @@ static int nbc_reduce_inter_init(const void* sendbuf, void* recvbuf, int count,
 
 int ompi_coll_libnbc_ireduce_inter(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
                                    MPI_Op op, int root, struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                   struct mca_coll_base_module_2_3_0_t *module) {
+                                   mca_coll_base_module_t *module) {
     int res = nbc_reduce_inter_init(sendbuf, recvbuf, count, datatype, op, root,
                                     comm, request, module, false);
     if (OPAL_LIKELY(OMPI_SUCCESS != res)) {
@@ -781,7 +781,7 @@ static inline int red_sched_redscat_gather(
 
         for (int mask = 1; mask < nprocs_pof2; mask <<= 1) {
             /*
-             * On each iteration: rindex[step] = sindex[step] -- begining of the
+             * On each iteration: rindex[step] = sindex[step] -- beginning of the
              * current window. Length of the current window is storded in wsize.
              */
             int vdest = vrank ^ mask;
@@ -939,7 +939,7 @@ static inline int red_sched_redscat_gather(
 
 int ompi_coll_libnbc_reduce_init(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
                                  MPI_Op op, int root, struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
-                                 struct mca_coll_base_module_2_3_0_t *module) {
+                                 mca_coll_base_module_t *module) {
     int res = nbc_reduce_init(sendbuf, recvbuf, count, datatype, op, root,
                               comm, request, module, true);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
@@ -951,7 +951,7 @@ int ompi_coll_libnbc_reduce_init(const void* sendbuf, void* recvbuf, int count,
 
 int ompi_coll_libnbc_reduce_inter_init(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype,
                                        MPI_Op op, int root, struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
-                                       struct mca_coll_base_module_2_3_0_t *module) {
+                                       mca_coll_base_module_t *module) {
     int res = nbc_reduce_inter_init(sendbuf, recvbuf, count, datatype, op, root,
                                     comm, request, module, true);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
