@@ -53,7 +53,7 @@ int NBC_Scan_args_compare(NBC_Scan_args *a, NBC_Scan_args *b, void *param) {
 
 static int nbc_scan_init(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op,
                          struct ompi_communicator_t *comm, ompi_request_t ** request,
-                         struct mca_coll_base_module_2_3_0_t *module, bool persistent) {
+                         mca_coll_base_module_t *module, bool persistent) {
     int rank, p, res;
     ptrdiff_t gap, span;
     NBC_Schedule *schedule;
@@ -314,7 +314,7 @@ static inline int scan_sched_recursivedoubling(
 
 int ompi_coll_libnbc_iscan(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op,
                            struct ompi_communicator_t *comm, ompi_request_t ** request,
-                           struct mca_coll_base_module_2_3_0_t *module) {
+                           mca_coll_base_module_t *module) {
     int res = nbc_scan_init(sendbuf, recvbuf, count, datatype, op,
                             comm, request, module, false);
     if (OPAL_LIKELY(OMPI_SUCCESS != res)) {
@@ -332,7 +332,7 @@ int ompi_coll_libnbc_iscan(const void* sendbuf, void* recvbuf, int count, MPI_Da
 
 int ompi_coll_libnbc_scan_init(const void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op,
                                struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
-                               struct mca_coll_base_module_2_3_0_t *module) {
+                               mca_coll_base_module_t *module) {
     int res = nbc_scan_init(sendbuf, recvbuf, count, datatype, op,
                             comm, request, module, true);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
