@@ -38,9 +38,9 @@ BEGIN_C_DECLS
    great while.  Use a "goto" label for expdiency to exit loops. */
 #define SPIN_CONDITION_MAX 100000
 #define SPIN_CONDITION(cond, exit_label) \
-  do { int i; \
+  do { \
        if (cond) goto exit_label; \
-       for (i = 0; i < SPIN_CONDITION_MAX; ++i) { \
+       for (int spin_cond_i = 0; spin_cond_i < SPIN_CONDITION_MAX; ++spin_cond_i) { \
            if (cond) { goto exit_label; } \
        } \
        opal_progress(); \
@@ -55,7 +55,7 @@ BEGIN_C_DECLS
      */
     typedef struct mca_coll_sm_component_t {
         /** Base coll component */
-        mca_coll_base_component_2_0_0_t super;
+        mca_coll_base_component_2_4_0_t super;
 
         /** MCA parameter: Priority of this component */
         int sm_priority;
@@ -114,7 +114,7 @@ BEGIN_C_DECLS
     typedef struct mca_coll_sm_in_use_flag_t {
         /** Number of processes currently using this set of
             segments */
-        volatile uint32_t mcsiuf_num_procs_using;
+        opal_atomic_uint32_t mcsiuf_num_procs_using;
         /** Must match data->mcb_count */
         volatile uint32_t mcsiuf_operation_count;
     } mca_coll_sm_in_use_flag_t;
@@ -152,7 +152,7 @@ BEGIN_C_DECLS
         /** Pointer to my parent's barrier control pages (will be NULL
             for communicator rank 0; odd index pages are "in", even
             index pages are "out") */
-        uint32_t *mcb_barrier_control_parent;
+        opal_atomic_uint32_t *mcb_barrier_control_parent;
 
         /** Pointers to my childrens' barrier control pages (they're
             contiguous in memory, so we only point to the base -- the
@@ -201,7 +201,7 @@ BEGIN_C_DECLS
     /**
      * Global component instance
      */
-    OMPI_MODULE_DECLSPEC extern mca_coll_sm_component_t mca_coll_sm_component;
+    OMPI_DECLSPEC extern mca_coll_sm_component_t mca_coll_sm_component;
 
     /*
      * coll module functions
@@ -316,8 +316,6 @@ BEGIN_C_DECLS
 				   struct ompi_communicator_t *comm,
 				   mca_coll_base_module_t *module);
 
-    int mca_coll_sm_ft_event(int state);
-
 /**
  * Global variables used in the macros (essentially constants, so
  * these are thread safe)
@@ -358,7 +356,7 @@ extern uint32_t mca_coll_sm_one;
  * Macro to release an in-use flag from this process
  */
 #define FLAG_RELEASE(flag) \
-    (void)opal_atomic_add(&(flag)->mcsiuf_num_procs_using, -1)
+    opal_atomic_add(&(flag)->mcsiuf_num_procs_using, -1)
 
 /**
  * Macro to copy a single segment in from a user buffer to a shared
@@ -410,7 +408,7 @@ extern uint32_t mca_coll_sm_one;
     } while (0)
 
 /**
- * Macro for childen to wait for parent notification (use real rank).
+ * Macro for children to wait for parent notification (use real rank).
  * Save the value passed and then reset it when done.  Used in fan out
  * operations.
  */
