@@ -15,7 +15,8 @@
  *                         All rights reserved.
  * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2015      Intel, Inc. All rights reserved.
+ * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -51,12 +52,13 @@
 #include "opal_stdint.h"
 #include "opal/mca/hwloc/base/base.h"
 #include "opal/util/os_path.h"
+#include "opal/util/printf.h"
 
 #include "ompi/communicator/communicator.h"
 #include "ompi/group/group.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/base.h"
-#include "ompi/mca/rte/rte.h"
+#include "ompi/runtime/ompi_rte.h"
 #include "ompi/proc/proc.h"
 #include "coll_sm.h"
 
@@ -173,16 +175,18 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
        are not on this node, then we don't want to run */
     if (OMPI_COMM_IS_INTER(comm) || 1 == ompi_comm_size(comm) || ompi_group_have_remote_peers (comm->c_local_group)) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:comm_query (%d/%s): intercomm, comm is too small, or not all peers local; disqualifying myself", comm->c_contextid, comm->c_name);
+                            "coll:sm:comm_query (%s/%s): intercomm, comm is too small, or not all peers local; disqualifying myself",
+			    ompi_comm_print_cid (comm), comm->c_name);
 	return NULL;
     }
 
     /* Get the priority level attached to this module. If priority is less
      * than or equal to 0, then the module is unavailable. */
     *priority = mca_coll_sm_component.sm_priority;
-    if (mca_coll_sm_component.sm_priority <= 0) {
+    if (mca_coll_sm_component.sm_priority < 0) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:comm_query (%d/%s): priority too low; disqualifying myself", comm->c_contextid, comm->c_name);
+                            "coll:sm:comm_query (%s/%s): priority too low; disqualifying myself",
+			    ompi_comm_print_cid (comm), comm->c_name);
 	return NULL;
     }
 
@@ -193,7 +197,6 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
 
     /* All is good -- return a module */
     sm_module->super.coll_module_enable = sm_module_enable;
-    sm_module->super.ft_event        = mca_coll_sm_ft_event;
     sm_module->super.coll_allgather  = NULL;
     sm_module->super.coll_allgatherv = NULL;
     sm_module->super.coll_allreduce  = mca_coll_sm_allreduce_intra;
@@ -212,8 +215,8 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
     sm_module->super.coll_scatterv   = NULL;
 
     opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                        "coll:sm:comm_query (%d/%s): pick me! pick me!",
-                        comm->c_contextid, comm->c_name);
+                        "coll:sm:comm_query (%s/%s): pick me! pick me!",
+                        ompi_comm_print_cid (comm), comm->c_name);
     return &(sm_module->super);
 }
 
@@ -227,8 +230,8 @@ static int sm_module_enable(mca_coll_base_module_t *module,
     if (NULL == comm->c_coll->coll_reduce ||
         NULL == comm->c_coll->coll_reduce_module) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:enable (%d/%s): no underlying reduce; disqualifying myself",
-                            comm->c_contextid, comm->c_name);
+                            "coll:sm:enable (%s/%s): no underlying reduce; disqualifying myself",
+			    ompi_comm_print_cid (comm), comm->c_name);
         return OMPI_ERROR;
     }
 
@@ -264,8 +267,8 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
                c->sm_comm_num_segments * 3);
     if (NULL == maffinity) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:enable (%d/%s): malloc failed (1)",
-                            comm->c_contextid, comm->c_name);
+                            "coll:sm:enable (%s/%s): malloc failed (1)",
+                            ompi_comm_print_cid (comm), comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
@@ -291,8 +294,8 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     if (NULL == data) {
         free(maffinity);
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:enable (%d/%s): malloc failed (2)",
-                            comm->c_contextid, comm->c_name);
+                            "coll:sm:enable (%s/%s): malloc failed (2)",
+                            ompi_comm_print_cid (comm), comm->c_name);
         return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
     }
     data->mcb_operation_count = 0;
@@ -362,7 +365,7 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     /* Once the communicator is bootstrapped, setup the pointers into
        the per-communicator shmem data segment.  First, setup the
        barrier buffers.  There are 2 sets of barrier buffers (because
-       there can never be more than one outstanding barrier occuring
+       there can never be more than one outstanding barrier occurring
        at any timie).  Setup pointers to my control buffers, my
        parents, and [the beginning of] my children (note that the
        children are contiguous, so having the first pointer and the
@@ -372,7 +375,7 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     data->mcb_barrier_control_me = (uint32_t*)
         (base + (rank * control_size * num_barrier_buffers * 2));
     if (data->mcb_tree[rank].mcstn_parent) {
-        data->mcb_barrier_control_parent = (uint32_t*)
+        data->mcb_barrier_control_parent = (opal_atomic_uint32_t*)
             (base +
              (data->mcb_tree[rank].mcstn_parent->mcstn_id * control_size *
               num_barrier_buffers * 2));
@@ -467,31 +470,30 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
 
     /* Wait for everyone in this communicator to attach and setup */
     opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                        "coll:sm:enable (%d/%s): waiting for peers to attach",
-                        comm->c_contextid, comm->c_name);
+                        "coll:sm:enable (%s/%s): waiting for peers to attach",
+                        ompi_comm_print_cid (comm), comm->c_name);
     SPIN_CONDITION(size == data->sm_bootstrap_meta->module_seg->seg_inited, seg_init_exit);
 
     /* Once we're all here, remove the mmap file; it's not needed anymore */
     if (0 == rank) {
         unlink(data->sm_bootstrap_meta->shmem_ds.seg_name);
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:enable (%d/%s): removed mmap file %s",
-                            comm->c_contextid, comm->c_name,
+                            "coll:sm:enable (%s/%s): removed mmap file %s",
+                            ompi_comm_print_cid (comm), comm->c_name,
                             data->sm_bootstrap_meta->shmem_ds.seg_name);
     }
 
     /* All done */
 
     opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                        "coll:sm:enable (%d/%s): success!",
-                        comm->c_contextid, comm->c_name);
+                        "coll:sm:enable (%s/%s): success!",
+                        ompi_comm_print_cid (comm), comm->c_name);
     return OMPI_SUCCESS;
 }
 
 static int bootstrap_comm(ompi_communicator_t *comm,
                           mca_coll_sm_module_t *module)
 {
-    int i;
     char *shortpath, *fullpath;
     mca_coll_sm_component_t *c = &mca_coll_sm_component;
     mca_coll_sm_comm_t *data = module->sm_comm_data;
@@ -507,10 +509,10 @@ static int bootstrap_comm(ompi_communicator_t *comm,
     /* Make the rendezvous filename for this communicators shmem data
        segment.  The CID is not guaranteed to be unique among all
        procs on this node, so also pair it with the PID of the proc
-       with the lowest ORTE name to form a unique filename. */
+       with the lowest PMIx name to form a unique filename. */
     proc = ompi_group_peer_lookup(comm->c_local_group, 0);
     lowest_name = OMPI_CAST_RTE_NAME(&proc->super.proc_name);
-    for (i = 1; i < comm_size; ++i) {
+    for (int i = 1; i < comm_size; ++i) {
         proc = ompi_group_peer_lookup(comm->c_local_group, i);
         if (ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL,
                                           OMPI_CAST_RTE_NAME(&proc->super.proc_name),
@@ -518,12 +520,12 @@ static int bootstrap_comm(ompi_communicator_t *comm,
             lowest_name = OMPI_CAST_RTE_NAME(&proc->super.proc_name);
         }
     }
-    asprintf(&shortpath, "coll-sm-cid-%d-name-%s.mmap", comm->c_contextid,
+    opal_asprintf(&shortpath, "coll-sm-cid-%s-name-%s.mmap", ompi_comm_print_cid (comm),
              OMPI_NAME_PRINT(lowest_name));
     if (NULL == shortpath) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:enable:bootstrap comm (%d/%s): asprintf failed",
-                            comm->c_contextid, comm->c_name);
+                            "coll:sm:enable:bootstrap comm (%s/%s): asprintf failed",
+                            ompi_comm_print_cid (comm), comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
     fullpath = opal_os_path(false, ompi_process_info.job_session_dir,
@@ -531,8 +533,8 @@ static int bootstrap_comm(ompi_communicator_t *comm,
     free(shortpath);
     if (NULL == fullpath) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:sm:enable:bootstrap comm (%d/%s): opal_os_path failed",
-                            comm->c_contextid, comm->c_name);
+                            "coll:sm:enable:bootstrap comm (%s/%s): opal_os_path failed",
+                            ompi_comm_print_cid (comm), comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
@@ -562,14 +564,14 @@ static int bootstrap_comm(ompi_communicator_t *comm,
         (num_segments * (comm_size * control_size * 2)) +
         (num_segments * (comm_size * frag_size));
     opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                        "coll:sm:enable:bootstrap comm (%d/%s): attaching to %" PRIsize_t " byte mmap: %s",
-                        comm->c_contextid, comm->c_name, size, fullpath);
+                        "coll:sm:enable:bootstrap comm (%s/%s): attaching to %" PRIsize_t " byte mmap: %s",
+                        ompi_comm_print_cid (comm), comm->c_name, size, fullpath);
     if (0 == ompi_comm_rank (comm)) {
         data->sm_bootstrap_meta = mca_common_sm_module_create_and_attach (size, fullpath, sizeof(mca_common_sm_seg_header_t), 8);
         if (NULL == data->sm_bootstrap_meta) {
             opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                "coll:sm:enable:bootstrap comm (%d/%s): mca_common_sm_init_group failed",
-                comm->c_contextid, comm->c_name);
+                "coll:sm:enable:bootstrap comm (%s/%s): mca_common_sm_init_group failed",
+				ompi_comm_print_cid (comm), comm->c_name);
             free(fullpath);
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
@@ -588,24 +590,3 @@ static int bootstrap_comm(ompi_communicator_t *comm,
     free(fullpath);
     return OMPI_SUCCESS;
 }
-
-
-int mca_coll_sm_ft_event(int state) {
-    if(OPAL_CRS_CHECKPOINT == state) {
-        ;
-    }
-    else if(OPAL_CRS_CONTINUE == state) {
-        ;
-    }
-    else if(OPAL_CRS_RESTART == state) {
-        ;
-    }
-    else if(OPAL_CRS_TERM == state ) {
-        ;
-    }
-    else {
-        ;
-    }
-
-    return OMPI_SUCCESS;
-}
