@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2009-2017 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,8 +30,7 @@
  *	Accepts:	- same as MPI_Reduce_scatter()
  *	Returns:	- MPI_SUCCESS or error code
  */
-int mca_coll_sync_reduce_scatter(const void *sbuf, void *rbuf,
-                                 const int *rcounts,
+int mca_coll_sync_reduce_scatter(const void *sbuf, void *rbuf, const int *rcounts,
                                  struct ompi_datatype_t *dtype,
                                  struct ompi_op_t *op,
                                  struct ompi_communicator_t *comm,
@@ -41,11 +40,10 @@ int mca_coll_sync_reduce_scatter(const void *sbuf, void *rbuf,
 
     if (s->in_operation) {
         return s->c_coll.coll_reduce_scatter(sbuf, rbuf, rcounts,
-                                            dtype, op, comm,
-                                            s->c_coll.coll_reduce_scatter_module);
-    } else {
-        COLL_SYNC(s, s->c_coll.coll_reduce_scatter(sbuf, rbuf, rcounts,
-                                                   dtype, op, comm,
-                                                   s->c_coll.coll_reduce_scatter_module));
+                                             dtype, op, comm,
+                                             s->c_coll.coll_reduce_scatter_module);
     }
+    COLL_SYNC(s, s->c_coll.coll_reduce_scatter(sbuf, rbuf, rcounts,
+                                               dtype, op, comm,
+                                               s->c_coll.coll_reduce_scatter_module));
 }
