@@ -5,10 +5,13 @@
  *                         reserved.
  * Copyright (c) 2013-2017 Inria.  All rights reserved.
  * Copyright (c) 2015      Bull SAS.  All rights reserved.
- * Copyright (c) 2016-2017 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2017      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2016-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2017-2018 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2019      Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2022      IBM Corporation. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -16,14 +19,16 @@
  * $HEADER$
  */
 
-#include <ompi_config.h>
+#include "ompi_config.h"
 #include "common_monitoring.h"
 #include "common_monitoring_coll.h"
-#include <ompi/constants.h>
-#include <ompi/communicator/communicator.h>
-#include <opal/mca/base/mca_base_component_repository.h>
-#include <opal/class/opal_hash_table.h>
-#include <opal/util/output.h>
+#include "ompi/constants.h"
+#include "ompi/communicator/communicator.h"
+#include "opal/mca/base/mca_base_component_repository.h"
+#include "opal/class/opal_hash_table.h"
+#include "opal/util/output.h"
+#include "opal/util/printf.h"
+#include "opal/runtime/opal.h"
 #include <math.h>
 
 #if SIZEOF_LONG_LONG == SIZEOF_SIZE_T
@@ -34,7 +39,7 @@
 
 /*** Monitoring specific variables ***/
 /* Keep tracks of how many components are currently using the common part */
-static int32_t mca_common_monitoring_hold = 0;
+static opal_atomic_int32_t mca_common_monitoring_hold = 0;
 /* Output parameters */
 int mca_common_monitoring_output_stream_id = -1;
 static opal_output_stream_t mca_common_monitoring_output_stream_obj = {
@@ -51,7 +56,7 @@ static opal_output_stream_t mca_common_monitoring_output_stream_obj = {
 };
 
 /*** MCA params to mark the monitoring as enabled. ***/
-/* This signals that the monitoring will highjack the PML, OSC and COLL */
+/* This signals that the monitoring will hijack the PML, OSC and COLL */
 int mca_common_monitoring_enabled = 0;
 int mca_common_monitoring_current_state = 0;
 /* Signals there will be an output of the monitored data at component close */
@@ -60,26 +65,26 @@ static int mca_common_monitoring_output_enabled = 0;
 static char* mca_common_monitoring_initial_filename = "";
 static char* mca_common_monitoring_current_filename = NULL;
 
-/* array for stroring monitoring data*/
-static size_t* pml_data = NULL;
-static size_t* pml_count = NULL;
-static size_t* filtered_pml_data = NULL;
-static size_t* filtered_pml_count = NULL;
-static size_t* osc_data_s = NULL;
-static size_t* osc_count_s = NULL;
-static size_t* osc_data_r = NULL;
-static size_t* osc_count_r = NULL;
-static size_t* coll_data = NULL;
-static size_t* coll_count = NULL;
+/* array for storing monitoring data*/
+static opal_atomic_size_t* pml_data = NULL;
+static opal_atomic_size_t* pml_count = NULL;
+static opal_atomic_size_t* filtered_pml_data = NULL;
+static opal_atomic_size_t* filtered_pml_count = NULL;
+static opal_atomic_size_t* osc_data_s = NULL;
+static opal_atomic_size_t* osc_count_s = NULL;
+static opal_atomic_size_t* osc_data_r = NULL;
+static opal_atomic_size_t* osc_count_r = NULL;
+static opal_atomic_size_t* coll_data = NULL;
+static opal_atomic_size_t* coll_count = NULL;
 
-static size_t* size_histogram = NULL;
+static opal_atomic_size_t* size_histogram = NULL;
 static const int max_size_histogram = 66;
 static double log10_2 = 0.;
 
 static int rank_world = -1;
 static int nprocs_world = 0;
 
-opal_hash_table_t *common_monitoring_translation_ht = NULL;
+opal_hash_table_t *ompi_common_monitoring_translation_ht = NULL;
 
 /* Reset all the monitoring arrays */
 static void mca_common_monitoring_reset ( void );
@@ -87,35 +92,35 @@ static void mca_common_monitoring_reset ( void );
 /* Flushes the monitored data and reset the values */
 static int mca_common_monitoring_flush (int fd, char* filename);
 
-/* Retreive the PML recorded count of messages sent */
+/* Retrieve the PML recorded count of messages sent */
 static int mca_common_monitoring_get_pml_count (const struct mca_base_pvar_t *pvar,
                                                 void *value, void *obj_handle);
 
-/* Retreive the PML recorded amount of data sent */
+/* Retrieve the PML recorded amount of data sent */
 static int mca_common_monitoring_get_pml_size (const struct mca_base_pvar_t *pvar,
                                                void *value, void *obj_handle);
 
-/* Retreive the OSC recorded count of messages sent */
+/* Retrieve the OSC recorded count of messages sent */
 static int mca_common_monitoring_get_osc_sent_count (const struct mca_base_pvar_t *pvar,
                                                      void *value, void *obj_handle);
 
-/* Retreive the OSC recorded amount of data sent */
+/* Retrieve the OSC recorded amount of data sent */
 static int mca_common_monitoring_get_osc_sent_size (const struct mca_base_pvar_t *pvar,
                                                     void *value, void *obj_handle);
 
-/* Retreive the OSC recorded count of messages received */
+/* Retrieve the OSC recorded count of messages received */
 static int mca_common_monitoring_get_osc_recv_count (const struct mca_base_pvar_t *pvar,
                                                      void *value, void *obj_handle);
 
-/* Retreive the OSC recorded amount of data received */
+/* Retrieve the OSC recorded amount of data received */
 static int mca_common_monitoring_get_osc_recv_size (const struct mca_base_pvar_t *pvar,
                                                     void *value, void *obj_handle);
 
-/* Retreive the COLL recorded count of messages sent */
+/* Retrieve the COLL recorded count of messages sent */
 static int mca_common_monitoring_get_coll_count (const struct mca_base_pvar_t *pvar,
                                                  void *value, void *obj_handle);
 
-/* Retreive the COLL recorded amount of data sent */
+/* Retrieve the COLL recorded amount of data sent */
 static int mca_common_monitoring_get_coll_size (const struct mca_base_pvar_t *pvar,
                                                 void *value, void *obj_handle);
 
@@ -211,18 +216,18 @@ int mca_common_monitoring_init( void )
     if( !mca_common_monitoring_enabled ) return OMPI_ERROR;
     if( 1 < opal_atomic_add_fetch_32(&mca_common_monitoring_hold, 1) ) return OMPI_SUCCESS; /* Already initialized */
 
-    char hostname[OPAL_MAXHOSTNAMELEN] = "NA";
+    const char *hostname;
     /* Initialize constant */
     log10_2 = log10(2.);
     /* Open the opal_output stream */
-    gethostname(hostname, sizeof(hostname));
-    asprintf(&mca_common_monitoring_output_stream_obj.lds_prefix,
+    hostname = opal_gethostname();
+    opal_asprintf(&mca_common_monitoring_output_stream_obj.lds_prefix,
              "[%s:%06d] monitoring: ", hostname, getpid());
     mca_common_monitoring_output_stream_id =
         opal_output_open(&mca_common_monitoring_output_stream_obj);
     /* Initialize proc translation hashtable */
-    common_monitoring_translation_ht = OBJ_NEW(opal_hash_table_t);
-    opal_hash_table_init(common_monitoring_translation_ht, 2048);
+    ompi_common_monitoring_translation_ht = OBJ_NEW(opal_hash_table_t);
+    opal_hash_table_init(ompi_common_monitoring_translation_ht, 2048);
     return OMPI_SUCCESS;
 }
 
@@ -230,9 +235,9 @@ void mca_common_monitoring_finalize( void )
 {
     if( ! mca_common_monitoring_enabled || /* Don't release if not last */
         0 < opal_atomic_sub_fetch_32(&mca_common_monitoring_hold, 1) ) return;
-    
+
     OPAL_MONITORING_PRINT_INFO("common_component_finish");
-    /* Dump monitoring informations */
+    /* Dump monitoring information */
     mca_common_monitoring_flush(mca_common_monitoring_output_enabled,
                                 mca_common_monitoring_current_filename);
     /* Disable all monitoring */
@@ -241,9 +246,9 @@ void mca_common_monitoring_finalize( void )
     opal_output_close(mca_common_monitoring_output_stream_id);
     free(mca_common_monitoring_output_stream_obj.lds_prefix);
     /* Free internal data structure */
-    free(pml_data);  /* a single allocation */
-    opal_hash_table_remove_all( common_monitoring_translation_ht );
-    OBJ_RELEASE(common_monitoring_translation_ht);
+    free((void *) pml_data);  /* a single allocation */
+    opal_hash_table_remove_all( ompi_common_monitoring_translation_ht );
+    OBJ_RELEASE(ompi_common_monitoring_translation_ht);
     mca_common_monitoring_coll_finalize();
     if( NULL != mca_common_monitoring_current_filename ) {
         free(mca_common_monitoring_current_filename);
@@ -251,7 +256,7 @@ void mca_common_monitoring_finalize( void )
     }
 }
 
-void mca_common_monitoring_register(void*pml_monitoring_component)
+int mca_common_monitoring_register(void)
 {
     /* Because we are playing tricks with the component close, we should not
      * use mca_base_component_var_register but instead stay with the basic
@@ -280,7 +285,6 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
                                 &mca_common_monitoring_output_enabled);
 
     (void)mca_base_var_register("ompi", "pml", "monitoring", "filename",
-                                /*&mca_common_monitoring_component.pmlm_version, "filename",*/
                                 "The name of the file where the monitoring information "
                                 "should be saved (the filename will be extended with the "
                                 "process rank and the \".prof\" extension). If this field "
@@ -297,10 +301,9 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
      * filename to be handled by the framework. It's easier to deal
      * with the string lifetime.
      */
-    if( NULL != mca_common_monitoring_initial_filename )
+    if( NULL != mca_common_monitoring_initial_filename ) {
         mca_common_monitoring_current_filename = strdup(mca_common_monitoring_initial_filename);
-
-    /* Register PVARs */
+    }
 
     /* PML PVARs */
     (void)mca_base_pvar_register("ompi", "pml", "monitoring", "flush", "Flush the monitoring "
@@ -336,7 +339,7 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
                                  MCA_BASE_PVAR_FLAG_READONLY | MCA_BASE_PVAR_FLAG_IWG,
                                  mca_common_monitoring_get_osc_sent_count, NULL,
                                  mca_common_monitoring_comm_size_notify, NULL);
-    
+
     (void)mca_base_pvar_register("ompi", "osc", "monitoring", "messages_sent_size", "Size of "
                                  "messages sent through the OSC framework with each peer.",
                                  OPAL_INFO_LVL_4, MPI_T_PVAR_CLASS_SIZE,
@@ -385,7 +388,7 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
                                  MCA_BASE_PVAR_FLAG_READONLY | MCA_BASE_PVAR_FLAG_IWG,
                                  mca_common_monitoring_coll_get_o2a_count, NULL,
                                  mca_common_monitoring_coll_messages_notify, NULL);
-    
+
     (void)mca_base_pvar_register("ompi", "coll", "monitoring", "o2a_size", "Size of messages "
                                  "exchanged as one-to-all operations in a communicator.",
                                  OPAL_INFO_LVL_4, MPI_T_PVAR_CLASS_AGGREGATE,
@@ -401,7 +404,7 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
                                  MCA_BASE_PVAR_FLAG_READONLY | MCA_BASE_PVAR_FLAG_IWG,
                                  mca_common_monitoring_coll_get_a2o_count, NULL,
                                  mca_common_monitoring_coll_messages_notify, NULL);
-    
+
     (void)mca_base_pvar_register("ompi", "coll", "monitoring", "a2o_size", "Size of messages "
                                  "exchanged as all-to-one operations in a communicator.",
                                  OPAL_INFO_LVL_4, MPI_T_PVAR_CLASS_AGGREGATE,
@@ -417,7 +420,7 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
                                  MCA_BASE_PVAR_FLAG_READONLY | MCA_BASE_PVAR_FLAG_IWG,
                                  mca_common_monitoring_coll_get_a2a_count, NULL,
                                  mca_common_monitoring_coll_messages_notify, NULL);
-    
+
     (void)mca_base_pvar_register("ompi", "coll", "monitoring", "a2a_size", "Size of messages "
                                  "exchanged as all-to-all operations in a communicator.",
                                  OPAL_INFO_LVL_4, MPI_T_PVAR_CLASS_AGGREGATE,
@@ -425,6 +428,8 @@ void mca_common_monitoring_register(void*pml_monitoring_component)
                                  MCA_BASE_PVAR_FLAG_READONLY | MCA_BASE_PVAR_FLAG_IWG,
                                  mca_common_monitoring_coll_get_a2a_size, NULL,
                                  mca_common_monitoring_coll_messages_notify, NULL);
+
+    return OMPI_SUCCESS;
 }
 
 /**
@@ -446,7 +451,7 @@ int mca_common_monitoring_add_procs(struct ompi_proc_t **procs,
 
     if( NULL == pml_data ) {
         int array_size = (10 + max_size_histogram) * nprocs_world;
-        pml_data           = (size_t*)calloc(array_size, sizeof(size_t));
+        pml_data           = (opal_atomic_size_t*)calloc(array_size, sizeof(size_t));
         pml_count          = pml_data + nprocs_world;
         filtered_pml_data  = pml_count + nprocs_world;
         filtered_pml_count = filtered_pml_data + nprocs_world;
@@ -480,7 +485,7 @@ int mca_common_monitoring_add_procs(struct ompi_proc_t **procs,
 
             key = *((uint64_t*)&tmp);
             /* save the rank of the process in MPI_COMM_WORLD in the hash using the proc_name as the key */
-            if( OPAL_SUCCESS != opal_hash_table_set_value_uint64(common_monitoring_translation_ht,
+            if( OPAL_SUCCESS != opal_hash_table_set_value_uint64(ompi_common_monitoring_translation_ht,
                                                                  key, (void*)(uintptr_t)peer_rank) ) {
                 return OMPI_ERR_OUT_OF_RESOURCE;  /* failed to allocate memory or growing the hash table */
             }
@@ -493,7 +498,7 @@ int mca_common_monitoring_add_procs(struct ompi_proc_t **procs,
 static void mca_common_monitoring_reset( void )
 {
     int array_size = (10 + max_size_histogram) * nprocs_world;
-    memset(pml_data, 0, array_size * sizeof(size_t));
+    memset((void *) pml_data, 0, array_size * sizeof(size_t));
     mca_common_monitoring_coll_reset();
 }
 
@@ -510,7 +515,7 @@ void mca_common_monitoring_record_pml(int world_rank, size_t data_size, int tag)
             log2_size = max_size_histogram - 2;
         opal_atomic_add_fetch_size_t(&size_histogram[world_rank * max_size_histogram + log2_size + 1], 1);
     }
-        
+
     /* distinguishses positive and negative tags if requested */
     if( (tag < 0) && (mca_common_monitoring_filter()) ) {
         opal_atomic_add_fetch_size_t(&filtered_pml_data[world_rank], data_size);
@@ -775,7 +780,7 @@ static int mca_common_monitoring_flush(int fd, char* filename)
             OPAL_MONITORING_PRINT_ERR("Error while flushing: no filename provided");
             return OMPI_ERROR;
         } else {
-            asprintf(&tmpfn, "%s.%" PRId32 ".prof", filename, rank_world);
+            opal_asprintf(&tmpfn, "%s.%" PRId32 ".prof", filename, rank_world);
             pf = fopen(tmpfn, "w");
             free(tmpfn);
         }
