@@ -4,7 +4,7 @@
  *                         reserved.
  * Copyright (c) 2013-2017 Inria.  All rights reserved.
  * Copyright (c) 2013-2015 Bull SAS.  All rights reserved.
- * Copyright (c) 2016      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2016-2020 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2017      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -27,7 +27,7 @@ Contact the authors for questions.
 
 To be run as:
 
-mpirun -np 4 \
+mpirun -n 4 \
     --mca pml_monitoring_enable 1 \
     -x LD_PRELOAD=ompi_install_dir/lib/ompi_monitoring_prof.so \
     ./my_app
@@ -42,10 +42,30 @@ writing 4x4 matrix to monitoring_avg.mat
 
 */
 
+#include "ompi_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <mpi.h>
 #include <string.h>
+#include <stdbool.h>
+
+#if OMPI_BUILD_FORTRAN_BINDINGS
+// Set these #defines in the same way that
+// ompi/mpi/fortran/mpif-h/Makefile.am does when compiling the real
+// Fortran mpif.h bindings.  They set behaviors in the Fortran header
+// files so that we can compile properly.
+#define OMPI_BUILD_MPI_PROFILING 0
+#define OMPI_COMPILING_FORTRAN_WRAPPERS 1
+#endif
+
+#include "opal/mca/threads/thread_usage.h"
+
+#include "ompi/include/mpi.h"
+#include "ompi/mpi/fortran/base/constants.h"
+#include "ompi/mpi/fortran/base/fint_2_int.h"
+#if OMPI_BUILD_FORTRAN_BINDINGS
+#include "ompi/mpi/fortran/mpif-h/bindings.h"
+#endif
 
 static MPI_T_pvar_session session;
 static int comm_world_size;
@@ -92,7 +112,7 @@ int MPI_Init(int* argc, char*** argv)
 
     MPIT_result = MPI_T_init_thread(MPI_THREAD_SINGLE, &provided);
     if (MPIT_result != MPI_SUCCESS) {
-        fprintf(stderr, "ERROR : failed to intialize MPI_T interface, preventing to get monitoring results: check your OpenMPI installation\n");
+        fprintf(stderr, "ERROR : failed to initialize MPI_T interface, preventing to get monitoring results: check your OpenMPI installation\n");
         PMPI_Abort(MPI_COMM_WORLD, MPIT_result);
     }
 
