@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2015 University of Houston. All rights reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -23,8 +25,7 @@
 #include "mpi.h"
 #include "ompi/constants.h"
 #include "ompi/mca/fcoll/fcoll.h"
-#include "ompi/mca/io/ompio/io_ompio.h"
-#include "ompi/mca/io/io.h"
+#include "ompi/mca/common/ompio/common_ompio.h"
 #include "math.h"
 #include <unistd.h>
 
@@ -32,11 +33,11 @@
 /* this component just directly calls the individual I/O operations */
 
 int
-mca_fcoll_individual_file_read_all (mca_io_ompio_file_t *fh,
+mca_fcoll_individual_file_read_all (ompio_file_t *fh,
                                     void *buf,
                                     int count,
                                     struct ompi_datatype_t *datatype,
                                     ompi_status_public_t *status)
 {
-    return ompio_io_ompio_file_read( fh, buf, count, datatype, status);
+    return mca_common_ompio_file_read( fh, buf, count, datatype, status);
 }
