@@ -16,6 +16,7 @@
  * Copyright (c) 2015      University of Houston. All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2019 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,6 +30,7 @@
 #include "mpi.h"
 #include "ompi/mca/mca.h"
 #include "ompi/request/request.h"
+#include "ompi/info/info.h"
 
 /*
  * Forward declaration for private data on io components and modules.
@@ -89,14 +91,14 @@ typedef int (*mca_io_base_component_file_unquery_fn_t)
     (struct ompi_file_t *file, struct mca_io_base_file_t *private_data);
 
 typedef int (*mca_io_base_component_file_delete_query_fn_t)
-    (const char *filename, struct ompi_info_t *info,
+    (const char *filename, struct opal_info_t *info,
      struct mca_io_base_delete_t **private_data,
      bool *usable, int *priority);
 typedef int (*mca_io_base_component_file_delete_select_fn_t)
-    (const char *filename, struct ompi_info_t *info,
+    (const char *filename, struct opal_info_t *info,
      struct mca_io_base_delete_t *private_data);
 typedef int (*mca_io_base_component_file_delete_unselect_fn_t)
-    (const char *filename, struct ompi_info_t *info,
+    (const char *filename, struct opal_info_t *info,
      struct mca_io_base_delete_t *private_data);
 
 typedef int (*mca_io_base_component_register_datarep_fn_t)(
@@ -140,7 +142,7 @@ typedef union mca_io_base_components_t mca_io_base_components_t;
 
 typedef int (*mca_io_base_module_file_open_fn_t)
     (struct ompi_communicator_t *comm, const char *filename, int amode,
-     struct ompi_info_t *info, struct ompi_file_t *fh);
+     struct opal_info_t *info, struct ompi_file_t *fh);
 typedef int (*mca_io_base_module_file_close_fn_t)(struct ompi_file_t *fh);
 
 typedef int (*mca_io_base_module_file_set_size_fn_t)
@@ -159,7 +161,7 @@ typedef int (*mca_io_base_module_file_get_info_fn_t)
 typedef int (*mca_io_base_module_file_set_view_fn_t)
     (struct ompi_file_t *fh, MPI_Offset disp, struct ompi_datatype_t *etype,
      struct ompi_datatype_t *filetype, const char *datarep,
-     struct ompi_info_t *info);
+     struct opal_info_t *info);
 typedef int (*mca_io_base_module_file_get_view_fn_t)
     (struct ompi_file_t *fh, MPI_Offset *disp,
      struct ompi_datatype_t **etype, struct ompi_datatype_t **filetype,
