@@ -13,6 +13,7 @@
  * Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -44,7 +45,7 @@
       int i;                                                             \
       type *a = (type *) in;                                             \
       type *b = (type *) out;                                            \
-      for (i = 0; i < *count; ++i) {                                     \
+      for (i = *count; i > 0; i--) {                                     \
           *(b++) op *(a++);                                              \
       }                                                                  \
   }
@@ -64,7 +65,7 @@
       int i;                                                             \
       type *a = (type *) in;                                             \
       type *b = (type *) out;                                            \
-      for (i = 0; i < *count; ++i) {                                     \
+      for (i = *count; i > 0; i--) {                                     \
           *(b) = current_func(*(b), *(a));                               \
           ++b;                                                           \
           ++a;                                                           \
@@ -92,7 +93,7 @@
         int i;                                                          \
         ompi_op_predefined_##type_name##_t *a = (ompi_op_predefined_##type_name##_t*) in; \
         ompi_op_predefined_##type_name##_t *b = (ompi_op_predefined_##type_name##_t*) out; \
-        for (i = 0; i < *count; ++i, ++a, ++b) {                        \
+        for (i = *count; i > 0; i--, ++a, ++b) {                        \
             if (a->v op b->v) {                                         \
                 b->v = a->v;                                            \
                 b->k = a->k;                                            \
@@ -116,7 +117,7 @@
       int i;                                                             \
       type (*a)[2] = (type (*)[2]) in;                                   \
       type (*b)[2] = (type (*)[2]) out;                                  \
-      for (i = 0; i < *count; ++i, ++a, ++b) {                           \
+      for (i = *count; i > 0; i--, ++a, ++b) {                           \
           (*b)[0] += (*a)[0];                                            \
           (*b)[1] += (*a)[1];                                            \
       }                                                                  \
@@ -137,7 +138,7 @@
       type (*a)[2] = (type (*)[2]) in;                                   \
       type (*b)[2] = (type (*)[2]) out;                                  \
       type c[2];                                                         \
-      for (i = 0; i < *count; ++i, ++a, ++b) {                           \
+      for (i = *count; i > 0; i--, ++a, ++b) {                           \
           c[0] = (*a)[0] * (*b)[0] - (*a)[1] * (*b)[1];                  \
           c[1] = (*a)[0] * (*b)[1] + (*a)[1] * (*b)[0];                  \
           (*b)[0] = c[0];                                                \
@@ -183,11 +184,14 @@ FUNC_FUNC(max, fortran_integer8, ompi_fortran_integer8_t)
 FUNC_FUNC(max, fortran_integer16, ompi_fortran_integer16_t)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+FUNC_FUNC(max, short_float, short float)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+FUNC_FUNC(max, short_float, opal_short_float_t)
+#endif
 FUNC_FUNC(max, float, float)
 FUNC_FUNC(max, double, double)
-#if HAVE_LONG_DOUBLE
 FUNC_FUNC(max, long_double, long double)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 FUNC_FUNC(max, fortran_real, ompi_fortran_real_t)
 #endif
@@ -246,11 +250,14 @@ FUNC_FUNC(min, fortran_integer8, ompi_fortran_integer8_t)
 FUNC_FUNC(min, fortran_integer16, ompi_fortran_integer16_t)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+FUNC_FUNC(min, short_float, short float)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+FUNC_FUNC(min, short_float, opal_short_float_t)
+#endif
 FUNC_FUNC(min, float, float)
 FUNC_FUNC(min, double, double)
-#if HAVE_LONG_DOUBLE
 FUNC_FUNC(min, long_double, long double)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 FUNC_FUNC(min, fortran_real, ompi_fortran_real_t)
 #endif
@@ -306,11 +313,14 @@ OP_FUNC(sum, fortran_integer8, ompi_fortran_integer8_t, +=)
 OP_FUNC(sum, fortran_integer16, ompi_fortran_integer16_t, +=)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+OP_FUNC(sum, short_float, short float, +=)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+OP_FUNC(sum, short_float, opal_short_float_t, +=)
+#endif
 OP_FUNC(sum, float, float, +=)
 OP_FUNC(sum, double, double, +=)
-#if HAVE_LONG_DOUBLE
 OP_FUNC(sum, long_double, long double, +=)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 OP_FUNC(sum, fortran_real, ompi_fortran_real_t, +=)
 #endif
@@ -330,15 +340,14 @@ OP_FUNC(sum, fortran_real8, ompi_fortran_real8_t, +=)
 OP_FUNC(sum, fortran_real16, ompi_fortran_real16_t, +=)
 #endif
 /* Complex */
-#if HAVE_FLOAT__COMPLEX
+#if defined(HAVE_SHORT_FLOAT__COMPLEX)
+OP_FUNC(sum, c_short_float_complex, short float _Complex, +=)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_COMPLEX_T)
+COMPLEX_SUM_FUNC(c_short_float_complex, opal_short_float_t)
+#endif
 OP_FUNC(sum, c_float_complex, float _Complex, +=)
-#endif
-#if HAVE_DOUBLE__COMPLEX
 OP_FUNC(sum, c_double_complex, double _Complex, +=)
-#endif
-#if HAVE_LONG_DOUBLE__COMPLEX
 OP_FUNC(sum, c_long_double_complex, long double _Complex, +=)
-#endif
 
 /*************************************************************************
  * Product
@@ -376,11 +385,14 @@ OP_FUNC(prod, fortran_integer8, ompi_fortran_integer8_t, *=)
 OP_FUNC(prod, fortran_integer16, ompi_fortran_integer16_t, *=)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+OP_FUNC(prod, short_float, short float, *=)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+OP_FUNC(prod, short_float, opal_short_float_t, *=)
+#endif
 OP_FUNC(prod, float, float, *=)
 OP_FUNC(prod, double, double, *=)
-#if HAVE_LONG_DOUBLE
 OP_FUNC(prod, long_double, long double, *=)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 OP_FUNC(prod, fortran_real, ompi_fortran_real_t, *=)
 #endif
@@ -400,15 +412,14 @@ OP_FUNC(prod, fortran_real8, ompi_fortran_real8_t, *=)
 OP_FUNC(prod, fortran_real16, ompi_fortran_real16_t, *=)
 #endif
 /* Complex */
-#if HAVE_FLOAT__COMPLEX
+#if defined(HAVE_SHORT_FLOAT__COMPLEX)
+OP_FUNC(prod, c_short_float_complex, short float _Complex, *=)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_COMPLEX_T)
+COMPLEX_PROD_FUNC(c_short_float_complex, opal_short_float_t)
+#endif
 OP_FUNC(prod, c_float_complex, float _Complex, *=)
-#endif
-#if HAVE_DOUBLE__COMPLEX
 OP_FUNC(prod, c_double_complex, double _Complex, *=)
-#endif
-#if HAVE_LONG_DOUBLE__COMPLEX
 OP_FUNC(prod, c_long_double_complex, long double _Complex, *=)
-#endif
 
 /*************************************************************************
  * Logical AND
@@ -624,9 +635,7 @@ LOC_STRUCT(double_int, double, int)
 LOC_STRUCT(long_int, long, int)
 LOC_STRUCT(2int, int, int)
 LOC_STRUCT(short_int, short, int)
-#if HAVE_LONG_DOUBLE
 LOC_STRUCT(long_double_int, long double, int)
-#endif
 LOC_STRUCT(unsigned_long, unsigned long, int)
 
 /*************************************************************************
@@ -647,9 +656,7 @@ LOC_FUNC(maxloc, double_int, >)
 LOC_FUNC(maxloc, long_int, >)
 LOC_FUNC(maxloc, 2int, >)
 LOC_FUNC(maxloc, short_int, >)
-#if HAVE_LONG_DOUBLE
 LOC_FUNC(maxloc, long_double_int, >)
-#endif
 
 /*************************************************************************
  * Min location
@@ -669,9 +676,7 @@ LOC_FUNC(minloc, double_int, <)
 LOC_FUNC(minloc, long_int, <)
 LOC_FUNC(minloc, 2int, <)
 LOC_FUNC(minloc, short_int, <)
-#if HAVE_LONG_DOUBLE
 LOC_FUNC(minloc, long_double_int, <)
-#endif
 
 
 /*
@@ -688,7 +693,7 @@ LOC_FUNC(minloc, long_double_int, <)
         type *a1 = (type *) in1;                                        \
         type *a2 = (type *) in2;                                        \
         type *b = (type *) out;                                         \
-        for (i = 0; i < *count; ++i) {                                  \
+        for (i = *count; i > 0; i--) {                                  \
             *(b++) =  *(a1++) op *(a2++);                               \
         }                                                               \
     }
@@ -710,7 +715,7 @@ LOC_FUNC(minloc, long_double_int, <)
         type *a1 = (type *) in1;                                        \
         type *a2 = (type *) in2;                                        \
         type *b = (type *) out;                                         \
-        for (i = 0; i < *count; ++i) {                                  \
+        for (i = *count; i > 0; i--) {                                  \
             *(b) = current_func(*(a1), *(a2));                          \
             ++b;                                                        \
             ++a1;                                                       \
@@ -743,7 +748,7 @@ LOC_FUNC(minloc, long_double_int, <)
       ompi_op_predefined_##type_name##_t *a1 = (ompi_op_predefined_##type_name##_t*) in1; \
       ompi_op_predefined_##type_name##_t *a2 = (ompi_op_predefined_##type_name##_t*) in2; \
       ompi_op_predefined_##type_name##_t *b = (ompi_op_predefined_##type_name##_t*) out; \
-      for (i = 0; i < *count; ++i, ++a1, ++a2, ++b ) {                  \
+      for (i = *count; i > 0; i--, ++a1, ++a2, ++b ) {                  \
           if (a1->v op a2->v) {                                         \
               b->v = a1->v;                                             \
               b->k = a1->k;                                             \
@@ -773,7 +778,7 @@ LOC_FUNC(minloc, long_double_int, <)
       type (*a1)[2] = (type (*)[2]) in1;                                 \
       type (*a2)[2] = (type (*)[2]) in2;                                 \
       type (*b)[2] = (type (*)[2]) out;                                  \
-      for (i = 0; i < *count; ++i, ++a1, ++a2, ++b) {                    \
+      for (i = *count; i > 0; i--, ++a1, ++a2, ++b) {                    \
           (*b)[0] = (*a1)[0] + (*a2)[0];                                 \
           (*b)[1] = (*a1)[1] + (*a2)[1];                                 \
       }                                                                  \
@@ -795,7 +800,7 @@ LOC_FUNC(minloc, long_double_int, <)
       type (*a1)[2] = (type (*)[2]) in1;                                 \
       type (*a2)[2] = (type (*)[2]) in2;                                 \
       type (*b)[2] = (type (*)[2]) out;                                  \
-      for (i = 0; i < *count; ++i, ++a1, ++a2, ++b) {                    \
+      for (i = *count; i > 0; i--, ++a1, ++a2, ++b) {                    \
           (*b)[0] = (*a1)[0] * (*a2)[0] - (*a1)[1] * (*a2)[1];           \
           (*b)[1] = (*a1)[0] * (*a2)[1] + (*a1)[1] * (*a2)[0];           \
       }                                                                  \
@@ -839,11 +844,14 @@ FUNC_FUNC_3BUF(max, fortran_integer8, ompi_fortran_integer8_t)
 FUNC_FUNC_3BUF(max, fortran_integer16, ompi_fortran_integer16_t)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+FUNC_FUNC_3BUF(max, short_float, short float)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+FUNC_FUNC_3BUF(max, short_float, opal_short_float_t)
+#endif
 FUNC_FUNC_3BUF(max, float, float)
 FUNC_FUNC_3BUF(max, double, double)
-#if HAVE_LONG_DOUBLE
 FUNC_FUNC_3BUF(max, long_double, long double)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 FUNC_FUNC_3BUF(max, fortran_real, ompi_fortran_real_t)
 #endif
@@ -902,11 +910,14 @@ FUNC_FUNC_3BUF(min, fortran_integer8, ompi_fortran_integer8_t)
 FUNC_FUNC_3BUF(min, fortran_integer16, ompi_fortran_integer16_t)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+FUNC_FUNC_3BUF(min, short_float, short float)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+FUNC_FUNC_3BUF(min, short_float, opal_short_float_t)
+#endif
 FUNC_FUNC_3BUF(min, float, float)
 FUNC_FUNC_3BUF(min, double, double)
-#if HAVE_LONG_DOUBLE
 FUNC_FUNC_3BUF(min, long_double, long double)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 FUNC_FUNC_3BUF(min, fortran_real, ompi_fortran_real_t)
 #endif
@@ -962,11 +973,14 @@ OP_FUNC_3BUF(sum, fortran_integer8, ompi_fortran_integer8_t, +)
 OP_FUNC_3BUF(sum, fortran_integer16, ompi_fortran_integer16_t, +)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+OP_FUNC_3BUF(sum, short_float, short float, +)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+OP_FUNC_3BUF(sum, short_float, opal_short_float_t, +)
+#endif
 OP_FUNC_3BUF(sum, float, float, +)
 OP_FUNC_3BUF(sum, double, double, +)
-#if HAVE_LONG_DOUBLE
 OP_FUNC_3BUF(sum, long_double, long double, +)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 OP_FUNC_3BUF(sum, fortran_real, ompi_fortran_real_t, +)
 #endif
@@ -986,15 +1000,14 @@ OP_FUNC_3BUF(sum, fortran_real8, ompi_fortran_real8_t, +)
 OP_FUNC_3BUF(sum, fortran_real16, ompi_fortran_real16_t, +)
 #endif
 /* Complex */
-#if HAVE_FLOAT__COMPLEX
+#if defined(HAVE_SHORT_FLOAT__COMPLEX)
+OP_FUNC_3BUF(sum, c_short_float_complex, short float _Complex, +)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_COMPLEX_T)
+COMPLEX_SUM_FUNC_3BUF(c_short_float_complex, opal_short_float_t)
+#endif
 OP_FUNC_3BUF(sum, c_float_complex, float _Complex, +)
-#endif
-#if HAVE_DOUBLE__COMPLEX
 OP_FUNC_3BUF(sum, c_double_complex, double _Complex, +)
-#endif
-#if HAVE_LONG_DOUBLE__COMPLEX
 OP_FUNC_3BUF(sum, c_long_double_complex, long double _Complex, +)
-#endif
 
 /*************************************************************************
  * Product
@@ -1032,11 +1045,14 @@ OP_FUNC_3BUF(prod, fortran_integer8, ompi_fortran_integer8_t, *)
 OP_FUNC_3BUF(prod, fortran_integer16, ompi_fortran_integer16_t, *)
 #endif
 /* Floating point */
+#if defined(HAVE_SHORT_FLOAT)
+OP_FUNC_3BUF(prod, short_float, short float, *)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T)
+OP_FUNC_3BUF(prod, short_float, opal_short_float_t, *)
+#endif
 OP_FUNC_3BUF(prod, float, float, *)
 OP_FUNC_3BUF(prod, double, double, *)
-#if HAVE_LONG_DOUBLE
 OP_FUNC_3BUF(prod, long_double, long double, *)
-#endif
 #if OMPI_HAVE_FORTRAN_REAL
 OP_FUNC_3BUF(prod, fortran_real, ompi_fortran_real_t, *)
 #endif
@@ -1056,15 +1072,14 @@ OP_FUNC_3BUF(prod, fortran_real8, ompi_fortran_real8_t, *)
 OP_FUNC_3BUF(prod, fortran_real16, ompi_fortran_real16_t, *)
 #endif
 /* Complex */
-#if HAVE_FLOAT__COMPLEX
+#if defined(HAVE_SHORT_FLOAT__COMPLEX)
+OP_FUNC_3BUF(prod, c_short_float_complex, short float _Complex, *)
+#elif defined(HAVE_OPAL_SHORT_FLOAT_COMPLEX_T)
+COMPLEX_PROD_FUNC_3BUF(c_short_float_complex, opal_short_float_t)
+#endif
 OP_FUNC_3BUF(prod, c_float_complex, float _Complex, *)
-#endif
-#if HAVE_DOUBLE__COMPLEX
 OP_FUNC_3BUF(prod, c_double_complex, double _Complex, *)
-#endif
-#if HAVE_LONG_DOUBLE__COMPLEX
 OP_FUNC_3BUF(prod, c_long_double_complex, long double _Complex, *)
-#endif
 
 /*************************************************************************
  * Logical AND
@@ -1280,9 +1295,7 @@ LOC_STRUCT_3BUF(double_int, double, int)
 LOC_STRUCT_3BUF(long_int, long, int)
 LOC_STRUCT_3BUF(2int, int, int)
 LOC_STRUCT_3BUF(short_int, short, int)
-#if HAVE_LONG_DOUBLE
 LOC_STRUCT_3BUF(long_double_int, long double, int)
-#endif
 */
 
 /*************************************************************************
@@ -1303,9 +1316,7 @@ LOC_FUNC_3BUF(maxloc, double_int, >)
 LOC_FUNC_3BUF(maxloc, long_int, >)
 LOC_FUNC_3BUF(maxloc, 2int, >)
 LOC_FUNC_3BUF(maxloc, short_int, >)
-#if HAVE_LONG_DOUBLE
 LOC_FUNC_3BUF(maxloc, long_double_int, >)
-#endif
 
 /*************************************************************************
  * Min location
@@ -1325,20 +1336,18 @@ LOC_FUNC_3BUF(minloc, double_int, <)
 LOC_FUNC_3BUF(minloc, long_int, <)
 LOC_FUNC_3BUF(minloc, 2int, <)
 LOC_FUNC_3BUF(minloc, short_int, <)
-#if HAVE_LONG_DOUBLE
 LOC_FUNC_3BUF(minloc, long_double_int, <)
-#endif
 
 /*
  * Helpful defines, because there's soooo many names!
  *
  * **NOTE** These #define's used to be strictly ordered but the use of
  * designated initializers removed this restrictions. When adding new
- * operators ALWAYS use a designated initalizer!
+ * operators ALWAYS use a designated initializer!
  */
 
 /** C integer ***********************************************************/
-#define C_INTEGER(name, ftype)                                              \
+#define C_INTEGER(name, ftype)                                             \
   [OMPI_OP_BASE_TYPE_INT8_T] = ompi_op_base_##ftype##_##name##_int8_t,     \
   [OMPI_OP_BASE_TYPE_UINT8_T] = ompi_op_base_##ftype##_##name##_uint8_t,   \
   [OMPI_OP_BASE_TYPE_INT16_T] = ompi_op_base_##ftype##_##name##_int16_t,   \
@@ -1443,12 +1452,22 @@ LOC_FUNC_3BUF(minloc, long_double_int, <)
 
 /** Floating point, including all the Fortran reals *********************/
 
+#if defined(HAVE_SHORT_FLOAT) || defined(HAVE_OPAL_SHORT_FLOAT_T)
+#define SHORT_FLOAT(name, ftype) ompi_op_base_##ftype##_##name##_short_float
+#else
+#define SHORT_FLOAT(name, ftype) NULL
+#endif
+#define FLOAT(name, ftype) ompi_op_base_##ftype##_##name##_float
+#define DOUBLE(name, ftype) ompi_op_base_##ftype##_##name##_double
+#define LONG_DOUBLE(name, ftype) ompi_op_base_##ftype##_##name##_long_double
+
 #define FLOATING_POINT(name, ftype)                                                            \
-  [OMPI_OP_BASE_TYPE_FLOAT] = ompi_op_base_##ftype##_##name##_float,                           \
-  [OMPI_OP_BASE_TYPE_DOUBLE] = ompi_op_base_##ftype##_##name##_double,                         \
+  [OMPI_OP_BASE_TYPE_SHORT_FLOAT] = SHORT_FLOAT(name, ftype),                                  \
+  [OMPI_OP_BASE_TYPE_FLOAT] = FLOAT(name, ftype),                                              \
+  [OMPI_OP_BASE_TYPE_DOUBLE] = DOUBLE(name, ftype),                                            \
   FLOATING_POINT_FORTRAN_REAL(name, ftype),                                                    \
   [OMPI_OP_BASE_TYPE_DOUBLE_PRECISION] = FLOATING_POINT_FORTRAN_DOUBLE_PRECISION(name, ftype), \
-  [OMPI_OP_BASE_TYPE_LONG_DOUBLE] = ompi_op_base_##ftype##_##name##_long_double
+  [OMPI_OP_BASE_TYPE_LONG_DOUBLE] = LONG_DOUBLE(name, ftype)
 
 /** Fortran logical *****************************************************/
 
@@ -1465,11 +1484,17 @@ LOC_FUNC_3BUF(minloc, long_double_int, <)
 
 /** Complex *****************************************************/
 
+#if defined(HAVE_SHORT_FLOAT__COMPLEX) || defined(HAVE_OPAL_SHORT_FLOAT_COMPLEX_T)
+#define SHORT_FLOAT_COMPLEX(name, ftype) ompi_op_base_##ftype##_##name##_c_short_float_complex
+#else
+#define SHORT_FLOAT_COMPLEX(name, ftype) NULL
+#endif
 #define FLOAT_COMPLEX(name, ftype) ompi_op_base_##ftype##_##name##_c_float_complex
 #define DOUBLE_COMPLEX(name, ftype) ompi_op_base_##ftype##_##name##_c_double_complex
 #define LONG_DOUBLE_COMPLEX(name, ftype) ompi_op_base_##ftype##_##name##_c_long_double_complex
 
 #define COMPLEX(name, ftype)                                                  \
+    [OMPI_OP_BASE_TYPE_C_SHORT_FLOAT_COMPLEX] = SHORT_FLOAT_COMPLEX(name, ftype), \
     [OMPI_OP_BASE_TYPE_C_FLOAT_COMPLEX] = FLOAT_COMPLEX(name, ftype),         \
     [OMPI_OP_BASE_TYPE_C_DOUBLE_COMPLEX] = DOUBLE_COMPLEX(name, ftype),       \
     [OMPI_OP_BASE_TYPE_C_LONG_DOUBLE_COMPLEX] = LONG_DOUBLE_COMPLEX(name, ftype)
@@ -1599,7 +1624,7 @@ ompi_op_base_handler_fn_t ompi_op_base_functions[OMPI_OP_BASE_FORTRAN_OP_MAX][OM
         [OMPI_OP_BASE_FORTRAN_REPLACE] = {
             /* (MPI_ACCUMULATE is handled differently than the other
                reductions, so just zero out its function
-               impementations here to ensure that users don't invoke
+               implementations here to ensure that users don't invoke
                MPI_REPLACE with any reduction operations other than
                ACCUMULATE) */
             NULL,
@@ -1686,7 +1711,7 @@ ompi_op_base_3buff_handler_fn_t ompi_op_base_3buff_functions[OMPI_OP_BASE_FORTRA
         [OMPI_OP_BASE_FORTRAN_REPLACE] = {
             /* MPI_ACCUMULATE is handled differently than the other
                reductions, so just zero out its function
-               impementations here to ensure that users don't invoke
+               implementations here to ensure that users don't invoke
                MPI_REPLACE with any reduction operations other than
                ACCUMULATE */
             NULL,
