@@ -1,7 +1,7 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2011-2017 Sandia National Laboratories.  All rights reserved.
- * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2011-2022 Sandia National Laboratories.  All rights reserved.
+ * Copyright (c) 2015-2018 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
@@ -9,6 +9,9 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
+ * Copyright (c) 2018-2022 Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2020      High Performance Computing Center Stuttgart,
+ *                         University of Stuttgart.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -18,9 +21,14 @@
 
 #include "ompi_config.h"
 
-#include "ompi/mca/osc/osc.h"
+#include "opal/util/printf.h"
+#include "opal/include/opal/align.h"
+#include "opal/mca/mpool/base/base.h"
+#include "opal/opal_portable_platform.h"
+
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
+#include "ompi/mca/osc/osc.h"
 #include "ompi/request/request.h"
 
 #include "osc_portals4.h"
@@ -62,42 +70,40 @@ ompi_osc_portals4_component_t mca_osc_portals4_component = {
 
 ompi_osc_portals4_module_t ompi_osc_portals4_module_template = {
     {
-        NULL, /* shared_query */
+        .osc_win_attach = ompi_osc_portals4_attach,
+        .osc_win_detach = ompi_osc_portals4_detach,
+        .osc_free = ompi_osc_portals4_free,
 
-        ompi_osc_portals4_attach,
-        ompi_osc_portals4_detach,
-        ompi_osc_portals4_free,
+        .osc_put = ompi_osc_portals4_put,
+        .osc_get = ompi_osc_portals4_get,
+        .osc_accumulate = ompi_osc_portals4_accumulate,
+        .osc_compare_and_swap = ompi_osc_portals4_compare_and_swap,
+        .osc_fetch_and_op = ompi_osc_portals4_fetch_and_op,
+        .osc_get_accumulate = ompi_osc_portals4_get_accumulate,
 
-        ompi_osc_portals4_put,
-        ompi_osc_portals4_get,
-        ompi_osc_portals4_accumulate,
-        ompi_osc_portals4_compare_and_swap,
-        ompi_osc_portals4_fetch_and_op,
-        ompi_osc_portals4_get_accumulate,
+        .osc_rput = ompi_osc_portals4_rput,
+        .osc_rget = ompi_osc_portals4_rget,
+        .osc_raccumulate = ompi_osc_portals4_raccumulate,
+        .osc_rget_accumulate = ompi_osc_portals4_rget_accumulate,
 
-        ompi_osc_portals4_rput,
-        ompi_osc_portals4_rget,
-        ompi_osc_portals4_raccumulate,
-        ompi_osc_portals4_rget_accumulate,
+        .osc_fence = ompi_osc_portals4_fence,
 
-        ompi_osc_portals4_fence,
+        .osc_start = ompi_osc_portals4_start,
+        .osc_complete = ompi_osc_portals4_complete,
+        .osc_post = ompi_osc_portals4_post,
+        .osc_wait = ompi_osc_portals4_wait,
+        .osc_test = ompi_osc_portals4_test,
 
-        ompi_osc_portals4_start,
-        ompi_osc_portals4_complete,
-        ompi_osc_portals4_post,
-        ompi_osc_portals4_wait,
-        ompi_osc_portals4_test,
+        .osc_lock = ompi_osc_portals4_lock,
+        .osc_unlock = ompi_osc_portals4_unlock,
+        .osc_lock_all = ompi_osc_portals4_lock_all,
+        .osc_unlock_all = ompi_osc_portals4_unlock_all,
 
-        ompi_osc_portals4_lock,
-        ompi_osc_portals4_unlock,
-        ompi_osc_portals4_lock_all,
-        ompi_osc_portals4_unlock_all,
-
-        ompi_osc_portals4_sync,
-        ompi_osc_portals4_flush,
-        ompi_osc_portals4_flush_all,
-        ompi_osc_portals4_flush_local,
-        ompi_osc_portals4_flush_local_all,
+        .osc_sync = ompi_osc_portals4_sync,
+        .osc_flush = ompi_osc_portals4_flush,
+        .osc_flush_all = ompi_osc_portals4_flush_all,
+        .osc_flush_local = ompi_osc_portals4_flush_local,
+        .osc_flush_local_all = ompi_osc_portals4_flush_local_all,
     }
 };
 
@@ -108,28 +114,11 @@ ompi_osc_portals4_module_t ompi_osc_portals4_module_template = {
 static bool
 check_config_value_bool(char *key, opal_info_t *info)
 {
-    char *value_string;
-    int value_len, ret, flag, param;
+    int ret, flag, param;
     const bool *flag_value;
     bool result;
-
-    ret = opal_info_get_valuelen(info, key, &value_len, &flag);
-    if (OMPI_SUCCESS != ret) goto info_not_found;
-    if (flag == 0) goto info_not_found;
-    value_len++;
-
-    value_string = (char*)malloc(sizeof(char) * value_len + 1); /* Should malloc 1 char for NUL-termination */
-    if (NULL == value_string) goto info_not_found;
-
-    ret = opal_info_get(info, key, value_len, value_string, &flag);
-    if (OMPI_SUCCESS != ret) {
-        free(value_string);
-        goto info_not_found;
-    }
-    assert(flag != 0);
-    ret = opal_info_value_to_bool(value_string, &result);
-    free(value_string);
-    if (OMPI_SUCCESS != ret) goto info_not_found;
+    ret = opal_info_get_bool(info, key, &result, &flag);
+    if (OMPI_SUCCESS != ret || !flag) goto info_not_found;
     return result;
 
  info_not_found:
@@ -146,37 +135,31 @@ check_config_value_bool(char *key, opal_info_t *info)
 static bool
 check_config_value_equal(char *key, opal_info_t *info, char *value)
 {
-    char *value_string;
-    int value_len, ret, flag, param;
-    const bool *flag_value;
+    int ret, flag, param;
+    const char *mca_value;
     bool result = false;
+    opal_cstring_t *value_string;
 
-    ret = opal_info_get_valuelen(info, key, &value_len, &flag);
-    if (OMPI_SUCCESS != ret) goto info_not_found;
-    if (flag == 0) goto info_not_found;
-    value_len++;
-
-    value_string = (char*)malloc(sizeof(char) * value_len + 1); /* Should malloc 1 char for NUL-termination */
-    if (NULL == value_string) goto info_not_found;
-
-    ret = opal_info_get(info, key, value_len, value_string, &flag);
-    if (OMPI_SUCCESS != ret) {
-        free(value_string);
+    ret = opal_info_get(info, key, &value_string, &flag);
+    if (OMPI_SUCCESS != ret || !flag) {
         goto info_not_found;
     }
-    assert(flag != 0);
-    if (0 == strcmp(value_string, value)) result = true;
-    free(value_string);
+    if (0 == strcmp(value_string->string, value)) {
+        result = true;
+    }
+    OBJ_RELEASE(value_string);
     return result;
 
  info_not_found:
     param = mca_base_var_find("ompi", "osc", "portals4", key);
     if (0 > param) return false;
 
-    ret = mca_base_var_get_value(param, &flag_value, NULL, NULL);
+    ret = mca_base_var_get_value(param, &mca_value, NULL, NULL);
     if (OMPI_SUCCESS != ret) return false;
 
-    if (0 == strcmp(value_string, value)) result = true;
+    if (0 == strcmp(mca_value, value)) {
+        result = true;
+    }
 
     return result;
 }
@@ -365,6 +348,8 @@ component_init(bool enable_progress_threads, bool enable_mpi_threads)
         return ret;
     }
 
+    ompi_osc_base_requires_world = true;
+
     return OMPI_SUCCESS;
 }
 
@@ -406,13 +391,27 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 {
     ompi_osc_portals4_module_t *module = NULL;
     int ret = OMPI_ERROR;
-    int tmp;
+    int tmp, flag;
     ptl_md_t md;
     ptl_me_t me;
     char *name;
+    size_t memory_alignment = OPAL_ALIGN_MIN;
 
     if (MPI_WIN_FLAVOR_SHARED == flavor) return OMPI_ERR_NOT_SUPPORTED;
 
+    if (NULL != info) {
+        opal_cstring_t *align_info_str;
+        opal_info_get(info, "mpi_minimum_memory_alignment",
+                      &align_info_str, &flag);
+        if (flag) {
+            size_t tmp_align = atoll(align_info_str->string);
+            OBJ_RELEASE(align_info_str);
+            if (OPAL_ALIGN_MIN < tmp_align) {
+                memory_alignment = tmp_align;
+            }
+        }
+    }
+
     /* create module structure */
     module = (ompi_osc_portals4_module_t*)
         calloc(1, sizeof(ompi_osc_portals4_module_t));
@@ -424,8 +423,10 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     /* fill in our part */
     if (MPI_WIN_FLAVOR_ALLOCATE == flavor) {
-        module->free_after = *base = malloc(size);
+        *base = mca_mpool_base_default_module->mpool_alloc(mca_mpool_base_default_module, size,
+                                                           memory_alignment, 0);
         if (NULL == *base) goto error;
+        module->free_after = *base;
     } else {
         module->free_after = NULL;
     }
@@ -435,9 +436,9 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     opal_output_verbose(1, ompi_osc_base_framework.framework_output,
                         "portals4 component creating window with id %d",
-                        ompi_comm_get_cid(module->comm));
+                        ompi_comm_get_local_cid(module->comm));
 
-    asprintf(&name, "portals4 window %d", ompi_comm_get_cid(module->comm));
+    opal_asprintf(&name, "portals4 window %d", ompi_comm_get_local_cid(module->comm));
     ompi_win_set_name(win, name);
     free(name);
 
@@ -524,7 +525,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     me.options = PTL_ME_OP_PUT | PTL_ME_OP_GET | PTL_ME_NO_TRUNCATE | PTL_ME_EVENT_SUCCESS_DISABLE;
     me.match_id.phys.nid = PTL_NID_ANY;
     me.match_id.phys.pid = PTL_PID_ANY;
-    me.match_bits = module->comm->c_contextid;
+    me.match_bits = ompi_comm_get_local_cid(module->comm);
     me.ignore_bits = 0;
 
     ret = PtlMEAppend(module->ni_h,
@@ -547,7 +548,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     me.options = PTL_ME_OP_PUT | PTL_ME_OP_GET | PTL_ME_NO_TRUNCATE | PTL_ME_EVENT_SUCCESS_DISABLE;
     me.match_id.phys.nid = PTL_NID_ANY;
     me.match_id.phys.pid = PTL_PID_ANY;
-    me.match_bits = module->comm->c_contextid | OSC_PORTALS4_MB_CONTROL;
+    me.match_bits = ompi_comm_get_local_cid(module->comm) | OSC_PORTALS4_MB_CONTROL;
     me.ignore_bits = 0;
 
     ret = PtlMEAppend(module->ni_h,
@@ -564,7 +565,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     }
 
     module->opcount = 0;
-    module->match_bits = module->comm->c_contextid;
+    module->match_bits = ompi_comm_get_local_cid(module->comm);
     module->atomic_max = (check_config_value_equal("accumulate_ordering", info, "none")) ?
         mca_osc_portals4_component.matching_atomic_max :
         MIN(mca_osc_portals4_component.matching_atomic_max,
@@ -591,7 +592,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     module->passive_target_access_epoch = false;
 
-#if OPAL_ASSEMBLY_ARCH == OPAL_X86_64 || OPAL_ASSEMBLY_ARCH == OPAL_IA32
+#if defined(PLATFORM_ARCH_X86) || defined(PLATFORM_ARCH_X86_64)
     *model = MPI_WIN_UNIFIED;
 #else
     *model = MPI_WIN_SEPARATE;
@@ -616,10 +617,10 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
  error:
     /* BWB: FIX ME: This is all wrong... */
-    if (0 != module->ct_h) PtlCTFree(module->ct_h);
-    if (0 != module->data_me_h) PtlMEUnlink(module->data_me_h);
-    if (0 != module->req_md_h) PtlMDRelease(module->req_md_h);
-    if (0 != module->md_h) PtlMDRelease(module->md_h);
+    if (!PtlHandleIsEqual(module->ct_h, PTL_INVALID_HANDLE)) PtlCTFree(module->ct_h);
+    if (!PtlHandleIsEqual(module->data_me_h, PTL_INVALID_HANDLE)) PtlMEUnlink(module->data_me_h);
+    if (!PtlHandleIsEqual(module->req_md_h, PTL_INVALID_HANDLE)) PtlMDRelease(module->req_md_h);
+    if (!PtlHandleIsEqual(module->md_h, PTL_INVALID_HANDLE)) PtlMDRelease(module->md_h);
     if (NULL != module->comm) ompi_comm_free(&module->comm);
     if (NULL != module) free(module);
 
@@ -656,11 +657,11 @@ ompi_osc_portals4_free(struct ompi_win_t *win)
     PtlMEUnlink(module->control_me_h);
     PtlMEUnlink(module->data_me_h);
     PtlMDRelease(module->md_h);
-    if (module->origin_iovec_md_h != PTL_INVALID_HANDLE) {
+    if (!PtlHandleIsEqual(module->origin_iovec_md_h,PTL_INVALID_HANDLE)) {
         PtlMDRelease(module->origin_iovec_md_h);
         free(module->origin_iovec_list);
     }
-    if (module->result_iovec_md_h != PTL_INVALID_HANDLE) {
+    if (!PtlHandleIsEqual(module->result_iovec_md_h,PTL_INVALID_HANDLE)) {
         PtlMDRelease(module->result_iovec_md_h);
         free(module->result_iovec_list);
     }
@@ -668,7 +669,8 @@ ompi_osc_portals4_free(struct ompi_win_t *win)
     PtlCTFree(module->ct_h);
     if (NULL != module->disp_units) free(module->disp_units);
     ompi_comm_free(&module->comm);
-    if (NULL != module->free_after) free(module->free_after);
+    mca_mpool_base_default_module->mpool_free(mca_mpool_base_default_module,
+                                              module->free_after);
 
     if (!opal_list_is_empty(&module->outstanding_locks)) {
         ret = OMPI_ERR_RMA_SYNC;
@@ -679,37 +681,3 @@ ompi_osc_portals4_free(struct ompi_win_t *win)
 
     return ret;
 }
-
-
-int
-ompi_osc_portals4_set_info(struct ompi_win_t *win, struct opal_info_t *info)
-{
-    ompi_osc_portals4_module_t *module =
-        (ompi_osc_portals4_module_t*) win->w_osc_module;
-
-    /* enforce collectiveness... */
-    return module->comm->c_coll->coll_barrier(module->comm,
-                                             module->comm->c_coll->coll_barrier_module);
-}
-
-
-int
-ompi_osc_portals4_get_info(struct ompi_win_t *win, struct opal_info_t **info_used)
-{
-    ompi_osc_portals4_module_t *module =
-        (ompi_osc_portals4_module_t*) win->w_osc_module;
-
-    opal_info_t *info = OBJ_NEW(opal_info_t);
-    if (NULL == info) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-
-    opal_info_set(info, "no_locks",  (module->state.lock == LOCK_ILLEGAL) ? "true" : "false");
-    if (module->atomic_max < mca_osc_portals4_component.matching_atomic_max) {
-        opal_info_set(info, "accumulate_ordering", "none");
-    } else {
-        opal_info_set(info, "accumulate_ordering", "rar,war,raw,waw");
-    }
-
-    *info_used = info;
-
-    return OMPI_SUCCESS;
-}
