@@ -55,19 +55,15 @@ typedef struct mca_pml_cm_request_t mca_pml_cm_request_t;
 OBJ_CLASS_DECLARATION(mca_pml_cm_request_t);
 
 /*
- * Avoid CUDA convertor inits only for contiguous memory and if indicated by
- * the MTL. For non-contiguous memory, do not skip CUDA convertor init phases.
+ * Avoid accelerator convertor inits only for contiguous memory and if indicated by
+ * the MTL. For non-contiguous memory, do not skip accelerator convertor init phases.
  */
-#if OPAL_CUDA_SUPPORT
-#define MCA_PML_CM_SWITCH_CUDA_CONVERTOR_OFF(flags, datatype, count)            \
+#define MCA_PML_CM_SWITCH_ACCELERATOR_CONVERTOR_OFF(flags, datatype, count)            \
     {                                                                           \
         if (opal_datatype_is_contiguous_memory_layout(&datatype->super, count)  \
-            && (ompi_mtl->mtl_flags & MCA_MTL_BASE_FLAG_CUDA_INIT_DISABLE)) {   \
-            flags |= CONVERTOR_SKIP_CUDA_INIT;                                  \
+            && (ompi_mtl->mtl_flags & MCA_MTL_BASE_FLAG_ACCELERATOR_INIT_DISABLE)) {   \
+            flags |= CONVERTOR_SKIP_ACCELERATOR_INIT;                                  \
         }                                                                       \
     }
-#else
-#define MCA_PML_CM_SWITCH_CUDA_CONVERTOR_OFF(flags, datatype, count)
-#endif
 
 #endif
