@@ -13,6 +13,8 @@
  * Copyright (c) 2009      IBM Corporation.  All rights reserved.
  * Copyright (c) 2012-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -47,13 +49,14 @@
 #define MCA_PML_OB1_HDR_TYPE_GET       (MCA_BTL_TAG_PML + 7)
 #define MCA_PML_OB1_HDR_TYPE_PUT       (MCA_BTL_TAG_PML + 8)
 #define MCA_PML_OB1_HDR_TYPE_FIN       (MCA_BTL_TAG_PML + 9)
+#define MCA_PML_OB1_HDR_TYPE_CID       (MCA_BTL_TAG_PML + 10)
 
-#define MCA_PML_OB1_HDR_FLAGS_ACK     1  /* is an ack required */
-#define MCA_PML_OB1_HDR_FLAGS_NBO     2  /* is the hdr in network byte order */
-#define MCA_PML_OB1_HDR_FLAGS_PIN     4  /* is user buffer pinned */
-#define MCA_PML_OB1_HDR_FLAGS_CONTIG  8  /* is user buffer contiguous */
-#define MCA_PML_OB1_HDR_FLAGS_NORDMA  16 /* rest will be send by copy-in-out */
-#define MCA_PML_OB1_HDR_FLAGS_SIGNAL  32 /* message can be optionally signalling */
+#define MCA_PML_OB1_HDR_FLAGS_ACK     0x01  /* is an ack required */
+#define MCA_PML_OB1_HDR_FLAGS_NBO     0x02  /* is the hdr in network byte order */
+#define MCA_PML_OB1_HDR_FLAGS_PIN     0x04  /* is user buffer pinned */
+#define MCA_PML_OB1_HDR_FLAGS_CONTIG  0x08  /* is user buffer contiguous */
+#define MCA_PML_OB1_HDR_FLAGS_NORDMA  0x10  /* rest will be send by copy-in-out */
+#define MCA_PML_OB1_HDR_FLAGS_SIGNAL  0x20  /* message can be optionally signalling */
 
 /**
  * Common hdr attributes - must be first element in each hdr type
@@ -74,6 +77,26 @@ static inline void mca_pml_ob1_common_hdr_prepare (mca_pml_ob1_common_hdr_t *hdr
 #define MCA_PML_OB1_COMMON_HDR_NTOH(h)
 #define MCA_PML_OB1_COMMON_HDR_HTON(h)
 
+/**
+ * Header definition for sending a CID/local comm index combo
+ */
+struct mca_pml_ob1_cid_hdr_t {
+    mca_pml_ob1_common_hdr_t hdr_common;
+    ompi_comm_extended_cid_t hdr_cid;
+    int16_t                  hdr_src_comm_index;
+    int32_t                  hdr_src;
+};
+
+typedef struct mca_pml_ob1_cid_hdr_t mca_pml_ob1_cid_hdr_t;
+
+static inline void mca_pml_ob1_cid_hdr_prepare (mca_pml_ob1_cid_hdr_t *hdr, ompi_communicator_t *comm)
+{
+    mca_pml_ob1_common_hdr_prepare (&hdr->hdr_common, MCA_PML_OB1_HDR_TYPE_CID, 0);
+    hdr->hdr_cid = ompi_comm_get_extended_cid (comm);
+    hdr->hdr_src_comm_index = comm->c_index;
+    hdr->hdr_src = ompi_comm_rank (comm);
+}
+
 /**
  *  Header definition for the first fragment, contains the
  *  attributes required to match the corresponding posted receive.
@@ -84,11 +107,11 @@ struct mca_pml_ob1_match_hdr_t {
     int32_t  hdr_src;                      /**< source rank */
     int32_t  hdr_tag;                      /**< user tag */
     uint16_t hdr_seq;                      /**< message sequence number */
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
     uint8_t  hdr_padding[2];               /**< explicitly pad to 16 bytes.  Compilers seem to already prefer to do this, but make it explicit just in case */
 #endif
 };
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
 #define OMPI_PML_OB1_MATCH_HDR_LEN  16
 #else
 #define OMPI_PML_OB1_MATCH_HDR_LEN  14
@@ -104,7 +127,7 @@ static inline void mca_pml_ob1_match_hdr_prepare (mca_pml_ob1_match_hdr_t *hdr,
     hdr->hdr_src = hdr_src;
     hdr->hdr_tag = hdr_tag;
     hdr->hdr_seq = hdr_seq;
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT && OPAL_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     hdr->hdr_padding[0] = 0;
     hdr->hdr_padding[1] = 0;
 #endif
@@ -128,7 +151,35 @@ do { \
     (h).hdr_seq = htons((h).hdr_seq); \
 } while (0)
 
-/**
+#define MCA_PML_OB1_EXT_MATCH_HDR_NTOH(h) \
+do {\
+    MCA_PML_OB1_COMMON_HDR_NTOH((h).hdr_common); \
+    (h).hdr_cid.cid_base = ntoh64((h).hdr_cid.cid_base); \
+    (h).hdr_cid.cid_sub.u64 = ntoh64((h).hdr_cid.cid_sub.u64); \
+    (h).hdr_src_comm_index = ntohs((h).hdr_src_comm_index); \
+    (h).hdr_src = ntohl((h).hdr_src); \
+} while (0)
+
+#define MCA_PML_OB1_EXT_MATCH_HDR_HTON(h) \
+do {\
+    MCA_PML_OB1_COMMON_HDR_HTON((h).hdr_common); \
+    (h).hdr_cid.cid_base = hton64((h).hdr_cid.cid_base); \
+    (h).hdr_cid.cid_sub.u64 = hton64((h).hdr_cid.cid_sub.u64); \
+    (h).hdr_src_comm_index = htons((h).hdr_src_comm_index); \
+    (h).hdr_src = htonl((h).hdr_src); \
+} while (0)
+
+struct mca_pml_ob1_ext_match_hdr_t {
+    mca_pml_ob1_cid_hdr_t hdr_ext_cid;
+
+    /* actual match */
+    mca_pml_ob1_match_hdr_t hdr_match;
+};
+
+typedef struct mca_pml_ob1_ext_match_hdr_t mca_pml_ob1_ext_match_hdr_t;
+
+/*
+*
  * Header definition for the first fragment when an acknowledgment
  * is required. This could be the first fragment of a large message
  * or a short message that requires an ack (synchronous).
@@ -140,6 +191,14 @@ struct mca_pml_ob1_rendezvous_hdr_t {
 };
 typedef struct mca_pml_ob1_rendezvous_hdr_t mca_pml_ob1_rendezvous_hdr_t;
 
+struct mca_pml_ob1_ext_rendezvous_hdr_t {
+    mca_pml_ob1_cid_hdr_t hdr_ext_cid;
+
+    /* actual match */
+    mca_pml_ob1_rendezvous_hdr_t hdr_rndv;
+};
+typedef struct mca_pml_ob1_ext_rendezvous_hdr_t mca_pml_ob1_ext_rendezvous_hdr_t;
+
 static inline void mca_pml_ob1_rendezvous_hdr_prepare (mca_pml_ob1_rendezvous_hdr_t *hdr, uint8_t hdr_type, uint8_t hdr_flags,
                                                        uint16_t hdr_ctx, int32_t hdr_src, int32_t hdr_tag, uint16_t hdr_seq,
                                                        uint64_t hdr_msg_length, void *hdr_src_req)
@@ -169,7 +228,7 @@ static inline void mca_pml_ob1_rendezvous_hdr_prepare (mca_pml_ob1_rendezvous_hd
  */
 struct mca_pml_ob1_rget_hdr_t {
     mca_pml_ob1_rendezvous_hdr_t hdr_rndv;
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
     uint8_t hdr_padding[4];
 #endif
     opal_ptr_t hdr_frag;                      /**< source fragment (for fin) */
@@ -178,6 +237,15 @@ struct mca_pml_ob1_rget_hdr_t {
 };
 typedef struct mca_pml_ob1_rget_hdr_t mca_pml_ob1_rget_hdr_t;
 
+struct mca_pml_ob1_ext_rget_hdr_t {
+    mca_pml_ob1_cid_hdr_t hdr_ext_cid;
+
+    /* actual match */
+    mca_pml_ob1_rget_hdr_t hdr_rget;
+};
+
+typedef struct mca_pml_ob1_ext_rget_hdr_t mca_pml_ob1_ext_rget_hdr_t;
+
 static inline void mca_pml_ob1_rget_hdr_prepare (mca_pml_ob1_rget_hdr_t *hdr, uint8_t hdr_flags,
                                                  uint16_t hdr_ctx, int32_t hdr_src, int32_t hdr_tag, uint16_t hdr_seq,
                                                  uint64_t hdr_msg_length, void *hdr_src_req, void *hdr_frag,
@@ -185,7 +253,7 @@ static inline void mca_pml_ob1_rget_hdr_prepare (mca_pml_ob1_rget_hdr_t *hdr, ui
 {
     mca_pml_ob1_rendezvous_hdr_prepare (&hdr->hdr_rndv, MCA_PML_OB1_HDR_TYPE_RGET, hdr_flags,
                                         hdr_ctx, hdr_src, hdr_tag, hdr_seq, hdr_msg_length, hdr_src_req);
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT && OPAL_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     hdr->hdr_padding[0] = 0;
     hdr->hdr_padding[1] = 0;
     hdr->hdr_padding[2] = 0;
@@ -215,7 +283,7 @@ static inline void mca_pml_ob1_rget_hdr_prepare (mca_pml_ob1_rget_hdr_t *hdr, ui
  */
 struct mca_pml_ob1_frag_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;     /**< common attributes */
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
     uint8_t hdr_padding[6];
 #endif
     uint64_t hdr_frag_offset;                /**< offset into message */
@@ -229,7 +297,7 @@ static inline void mca_pml_ob1_frag_hdr_prepare (mca_pml_ob1_frag_hdr_t *hdr, ui
                                                  uint64_t hdr_dst_req)
 {
     mca_pml_ob1_common_hdr_prepare (&hdr->hdr_common, MCA_PML_OB1_HDR_TYPE_FRAG, hdr_flags);
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT && OPAL_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     hdr->hdr_padding[0] = 0;
     hdr->hdr_padding[1] = 0;
     hdr->hdr_padding[2] = 0;
@@ -260,7 +328,7 @@ static inline void mca_pml_ob1_frag_hdr_prepare (mca_pml_ob1_frag_hdr_t *hdr, ui
 
 struct mca_pml_ob1_ack_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;      /**< common attributes */
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
     uint8_t hdr_padding[6];
 #endif
     opal_ptr_t hdr_src_req;                   /**< source request */
@@ -275,7 +343,7 @@ static inline void mca_pml_ob1_ack_hdr_prepare (mca_pml_ob1_ack_hdr_t *hdr, uint
                                                 uint64_t hdr_send_offset, uint64_t hdr_send_size)
 {
     mca_pml_ob1_common_hdr_prepare (&hdr->hdr_common, MCA_PML_OB1_HDR_TYPE_ACK, hdr_flags);
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT && OPAL_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     hdr->hdr_padding[0] = 0;
     hdr->hdr_padding[1] = 0;
     hdr->hdr_padding[2] = 0;
@@ -313,8 +381,8 @@ static inline void mca_pml_ob1_ack_hdr_prepare (mca_pml_ob1_ack_hdr_t *hdr, uint
 
 struct mca_pml_ob1_rdma_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;      /**< common attributes */
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-    uint8_t hdr_padding[2];                   /** two to pad out the hdr to a 4 byte alignment.  hdr_req will then be 8 byte aligned after 4 for hdr_seg_cnt */
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
+    uint8_t hdr_padding[6];                   /** two to pad out the hdr to a 4 byte alignment.  hdr_req will then be 8 byte aligned after 4 for hdr_seg_cnt */
 #endif
     /* TODO: add real support for multiple destination segments */
     opal_ptr_t hdr_req;                       /**< destination request */
@@ -334,9 +402,13 @@ static inline void mca_pml_ob1_rdma_hdr_prepare (mca_pml_ob1_rdma_hdr_t *hdr, ui
                                                  size_t local_handle_size)
 {
     mca_pml_ob1_common_hdr_prepare (&hdr->hdr_common, MCA_PML_OB1_HDR_TYPE_PUT, hdr_flags);
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT && OPAL_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     hdr->hdr_padding[0] = 0;
     hdr->hdr_padding[1] = 0;
+    hdr->hdr_padding[2] = 0;
+    hdr->hdr_padding[3] = 0;
+    hdr->hdr_padding[4] = 0;
+    hdr->hdr_padding[5] = 0;
 #endif
     hdr->hdr_req.lval = hdr_req;
     hdr->hdr_frag.pval = hdr_frag;
@@ -371,8 +443,8 @@ static inline void mca_pml_ob1_rdma_hdr_prepare (mca_pml_ob1_rdma_hdr_t *hdr, ui
 
 struct mca_pml_ob1_fin_hdr_t {
     mca_pml_ob1_common_hdr_t hdr_common;      /**< common attributes */
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-    uint8_t hdr_padding[2];
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT || OPAL_ENABLE_DEBUG
+    uint8_t hdr_padding[6];
 #endif
     int64_t hdr_size;                         /**< number of bytes completed (positive), error code (negative) */
     opal_ptr_t hdr_frag;                      /**< completed RDMA fragment */
@@ -383,9 +455,13 @@ static inline void mca_pml_ob1_fin_hdr_prepare (mca_pml_ob1_fin_hdr_t *hdr, uint
                                                 uint64_t hdr_frag, int64_t hdr_size)
 {
     mca_pml_ob1_common_hdr_prepare (&hdr->hdr_common, MCA_PML_OB1_HDR_TYPE_FIN, hdr_flags);
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT && OPAL_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     hdr->hdr_padding[0] = 0;
     hdr->hdr_padding[1] = 0;
+    hdr->hdr_padding[2] = 0;
+    hdr->hdr_padding[3] = 0;
+    hdr->hdr_padding[4] = 0;
+    hdr->hdr_padding[5] = 0;
 #endif
     hdr->hdr_frag.lval = hdr_frag;
     hdr->hdr_size      = hdr_size;
@@ -415,11 +491,16 @@ union mca_pml_ob1_hdr_t {
     mca_pml_ob1_ack_hdr_t hdr_ack;
     mca_pml_ob1_rdma_hdr_t hdr_rdma;
     mca_pml_ob1_fin_hdr_t hdr_fin;
+    /* extended CID support */
+    mca_pml_ob1_cid_hdr_t hdr_cid;
+    mca_pml_ob1_ext_match_hdr_t hdr_ext_match;
+    mca_pml_ob1_ext_rendezvous_hdr_t hdr_ext_rndv;
+    mca_pml_ob1_ext_rget_hdr_t hdr_ext_rget;
 };
 typedef union mca_pml_ob1_hdr_t mca_pml_ob1_hdr_t;
 
 #if !defined(WORDS_BIGENDIAN) && OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-static inline __opal_attribute_always_inline__ void
+static inline void
 ob1_hdr_ntoh(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type)
 {
     if(!(hdr->hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NBO))
@@ -447,6 +528,15 @@ ob1_hdr_ntoh(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type)
         case MCA_PML_OB1_HDR_TYPE_FIN:
             MCA_PML_OB1_FIN_HDR_NTOH(hdr->hdr_fin);
             break;
+        case MCA_PML_OB1_HDR_TYPE_CID:
+	{
+	    mca_pml_ob1_hdr_t *next_hdr = (mca_pml_ob1_hdr_t *) ((uintptr_t) hdr + sizeof (hdr->hdr_cid));
+
+	    MCA_PML_OB1_EXT_MATCH_HDR_NTOH(hdr->hdr_cid);
+	    /* now swap the real header */
+	    ob1_hdr_ntoh (next_hdr, next_hdr->hdr_common.hdr_type);
+	    break;
+	}
         default:
             assert(0);
             break;
@@ -459,7 +549,7 @@ ob1_hdr_ntoh(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type)
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
 #define ob1_hdr_hton(h, t, p) \
     ob1_hdr_hton_intr((mca_pml_ob1_hdr_t*)h, t, p)
-static inline __opal_attribute_always_inline__ void
+static inline  void
 ob1_hdr_hton_intr(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type,
         const ompi_proc_t *proc)
 {
@@ -493,6 +583,15 @@ ob1_hdr_hton_intr(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type,
         case MCA_PML_OB1_HDR_TYPE_FIN:
             MCA_PML_OB1_FIN_HDR_HTON(hdr->hdr_fin);
             break;
+        case MCA_PML_OB1_HDR_TYPE_CID:
+	{
+	    mca_pml_ob1_hdr_t *next_hdr = (mca_pml_ob1_hdr_t *) ((uintptr_t) hdr + sizeof (hdr->hdr_cid));
+
+	    MCA_PML_OB1_EXT_MATCH_HDR_HTON(hdr->hdr_cid);
+	    /* now swap the real header */
+	    ob1_hdr_hton (next_hdr, next_hdr->hdr_common.hdr_type, proc);
+	    break;
+	}
         default:
             assert(0);
             break;
@@ -506,7 +605,8 @@ ob1_hdr_hton_intr(mca_pml_ob1_hdr_t *hdr, const uint8_t hdr_type,
 static inline __opal_attribute_always_inline__ void
 ob1_hdr_copy(mca_pml_ob1_hdr_t *src, mca_pml_ob1_hdr_t *dst)
 {
-    switch(src->hdr_common.hdr_type) {
+    do {
+        switch(src->hdr_common.hdr_type) {
         case MCA_PML_OB1_HDR_TYPE_MATCH:
             memcpy( &(dst->hdr_match), &(src->hdr_match), sizeof(mca_pml_ob1_match_hdr_t) );
             break;
@@ -528,10 +628,24 @@ ob1_hdr_copy(mca_pml_ob1_hdr_t *src, mca_pml_ob1_hdr_t *dst)
         case MCA_PML_OB1_HDR_TYPE_FIN:
             memcpy( &(dst->hdr_fin), &(src->hdr_fin), sizeof(mca_pml_ob1_fin_hdr_t) );
             break;
+        case MCA_PML_OB1_HDR_TYPE_CID:
+	{
+	    mca_pml_ob1_hdr_t *next_src = (mca_pml_ob1_hdr_t *) ((uintptr_t) src + sizeof (src->hdr_cid));
+	    mca_pml_ob1_hdr_t *next_dst = (mca_pml_ob1_hdr_t *) ((uintptr_t) dst + sizeof (dst->hdr_cid));
+
+	    memcpy (&dst->hdr_cid, &src->hdr_cid, sizeof (src->hdr_cid));
+            /* can't call recursively and expect inlining */
+            src = next_src;
+            dst = next_dst;
+            continue;
+	}
         default:
             memcpy( &(dst->hdr_common), &(src->hdr_common), sizeof(mca_pml_ob1_common_hdr_t) );
             break;
-    }
+        }
+
+        break;
+    } while (1);
 }
 
 #endif  /* MCA_PML_OB1_HEADER_H */
