@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2021 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -69,6 +69,7 @@
 #include "ompi/mca/mca.h"
 #include "mpi.h" /* needed for MPI_ANY_TAG */
 #include "ompi/mca/pml/pml_constants.h"
+#include "ompi/request/request.h"
 
 BEGIN_C_DECLS
 
@@ -93,25 +94,48 @@ struct ompi_proc_t;
  * indicates whether multiple threads may invoke this component
  * simultaneously or not.
  */
-typedef struct mca_pml_base_module_1_0_1_t * (*mca_pml_base_component_init_fn_t)(
+typedef struct mca_pml_base_module_2_1_0_t * (*mca_pml_base_component_init_fn_t)(
     int *priority,
     bool enable_progress_threads,
     bool enable_mpi_threads);
 
+/**
+ * @brief A transport name and device name pair used by an endpoint.
+ * @ref mca_pml.pml_get_transports
+ */
+typedef struct mca_pml_transport_entry_t {
+    // The name of a transport layer used by this endpoint.
+    const char *transport_name;
+    // The name of the device used with this transport by this endpoint.
+    const char *device_name;
+} mca_pml_transport_entry_t;
+
+/**
+ * @brief  Structure containing an array of transport layers and device names
+ * used by a PML.
+ * @ref mca_pml.pml_get_transports
+ */
+typedef struct mca_pml_transports_t {
+    // Number of transport/device name pairs.
+    unsigned int count;
+    // Pointer to array of transport/device name pairs used by this endpoint.
+    mca_pml_transport_entry_t *entries;
+} mca_pml_transports_t;
+
 typedef int (*mca_pml_base_component_finalize_fn_t)(void);
 
 /**
  * PML component version and interface functions.
  */
 
-struct mca_pml_base_component_2_0_0_t {
+struct mca_pml_base_component_2_1_0_t {
    mca_base_component_t pmlm_version;
    mca_base_component_data_t pmlm_data;
    mca_pml_base_component_init_fn_t pmlm_init;
    mca_pml_base_component_finalize_fn_t pmlm_finalize;
 };
-typedef struct mca_pml_base_component_2_0_0_t mca_pml_base_component_2_0_0_t;
-typedef mca_pml_base_component_2_0_0_t mca_pml_base_component_t;
+typedef struct mca_pml_base_component_2_1_0_t mca_pml_base_component_2_1_0_t;
+typedef mca_pml_base_component_2_1_0_t mca_pml_base_component_t;
 
 
 /**
@@ -167,6 +191,16 @@ typedef int (*mca_pml_base_module_enable_fn_t)(
 */
 typedef int (*mca_pml_base_module_progress_fn_t)(void);
 
+/**
+ * Get the set of transports and devices used to communicate with specified rank
+ *
+ * @param comm Communicator
+ * @param rank Task rank of other task
+ * @return Pointer to structure containing set of transports and device names or NULL
+ */
+typedef mca_pml_transports_t *(*mca_pml_base_module_get_transports_t)(struct ompi_communicator_t *comm,
+                                                                      int rank);
+
 /**
  * MPI Interface Functions
  */
@@ -195,6 +229,20 @@ typedef int (*mca_pml_base_module_add_comm_fn_t)(struct ompi_communicator_t* com
  */
 typedef int (*mca_pml_base_module_del_comm_fn_t)(struct ompi_communicator_t* comm);
 
+/**
+ * Downcall from MPI layer when a communicator is revoked.
+ *
+ * @param comm (INOUT)      Communicator
+ * @param coll_only (IN)    Revoke only the collective operations, or all
+ *                              non-FT operations
+ * @return                  OMPI_SUCCESS or failure status.
+ *
+ * Provides the PML the opportunity to change unexpected fragments
+ * handling and purge the queues associated with the communicator.
+ */
+/* not conditional on OPAL_ENABLE_FT_MPI for ABI */
+typedef int (*mca_pml_base_module_revoke_comm_fn_t)(struct ompi_communicator_t* comm, bool coll_only);
+
 /**
  *  Initialize a persistent receive request.
  *
@@ -350,14 +398,11 @@ typedef int (*mca_pml_base_module_send_fn_t)(
 /**
  * Initiate one or more persistent requests.
  *
- * @param count    Number of requests
- * @param request  Array of persistent requests
- * @return         OMPI_SUCCESS or failure status.
+ * @param count (IN)        Number of requests
+ * @param requests (IN/OUT) Array of persistent requests
+ * @return                  OMPI_SUCCESS or failure status.
  */
-typedef int (*mca_pml_base_module_start_fn_t)(
-    size_t count,
-    struct ompi_request_t** requests
-);
+typedef ompi_request_start_fn_t mca_pml_base_module_start_fn_t;
 
 /**
  * Probe to poll for pending recv.
@@ -472,25 +517,23 @@ typedef int (*mca_pml_base_module_dump_fn_t)(
     int verbose
 );
 
-/**
- * Fault Tolerance Awareness function
- * @param status     Checkpoint status
- * @return           OMPI_SUCCESS or failure status
- */
-typedef int (*mca_pml_base_module_ft_event_fn_t) (int status);
-
 /**
  * pml module flags
  */
 /** PML requires requires all procs in the job on the first call to
  * add_procs */
-#define MCA_PML_BASE_FLAG_REQUIRE_WORLD 0x00000001
+#define MCA_PML_BASE_FLAG_REQUIRE_WORLD    0x00000001
+
+/**
+ * PML supports the extended CID space (doesn't need a global communicator index)
+ */
+#define MCA_PML_BASE_FLAG_SUPPORTS_EXT_CID 0x00000002
 
 /**
  *  PML instance.
  */
 
-struct mca_pml_base_module_1_0_1_t {
+struct mca_pml_base_module_2_1_0_t {
 
     /* downcalls from MCA to PML */
     mca_pml_base_module_add_procs_fn_t    pml_add_procs;
@@ -501,6 +544,7 @@ struct mca_pml_base_module_1_0_1_t {
     /* downcalls from MPI to PML */
     mca_pml_base_module_add_comm_fn_t     pml_add_comm;
     mca_pml_base_module_del_comm_fn_t     pml_del_comm;
+    mca_pml_base_module_revoke_comm_fn_t  pml_revoke_comm;
     mca_pml_base_module_irecv_init_fn_t   pml_irecv_init;
     mca_pml_base_module_irecv_fn_t        pml_irecv;
     mca_pml_base_module_recv_fn_t         pml_recv;
@@ -518,22 +562,21 @@ struct mca_pml_base_module_1_0_1_t {
     /* diagnostics */
     mca_pml_base_module_dump_fn_t         pml_dump;
 
-    /* FT Event */
-    mca_pml_base_module_ft_event_fn_t     pml_ft_event;
-
     /* maximum constant sizes */
     uint32_t                              pml_max_contextid;
     int                                   pml_max_tag;
     int                                   pml_flags;
+
+    mca_pml_base_module_get_transports_t pml_get_transports;
 };
-typedef struct mca_pml_base_module_1_0_1_t mca_pml_base_module_1_0_1_t;
-typedef mca_pml_base_module_1_0_1_t mca_pml_base_module_t;
+typedef struct mca_pml_base_module_2_1_0_t mca_pml_base_module_2_1_0_t;
+typedef mca_pml_base_module_2_1_0_t mca_pml_base_module_t;
 
 /*
  * Macro for use in components that are of type pml
  */
-#define MCA_PML_BASE_VERSION_2_0_0 \
-    OMPI_MCA_BASE_VERSION_2_1_0("pml", 2, 0, 0)
+#define MCA_PML_BASE_VERSION_2_1_0 \
+    OMPI_MCA_BASE_VERSION_2_1_0("pml", 2, 1, 0)
 
     /*
      * macro for doing direct call / call through struct
@@ -557,5 +600,10 @@ static inline bool mca_pml_base_requires_world (void)
     return !!(mca_pml.pml_flags & MCA_PML_BASE_FLAG_REQUIRE_WORLD);
 }
 
+static inline bool mca_pml_base_supports_extended_cid (void)
+{
+    return !!(mca_pml.pml_flags & MCA_PML_BASE_FLAG_SUPPORTS_EXT_CID);
+}
+
 END_C_DECLS
 #endif /* MCA_PML_H */
