@@ -26,7 +26,7 @@
 #define PML_UCX_ASSERT  MCA_COMMON_UCX_ASSERT
 #define PML_UCX_ERROR   MCA_COMMON_UCX_ERROR
 #define PML_UCX_VERBOSE MCA_COMMON_UCX_VERBOSE
-
+#define PML_UCX_WARN    MCA_COMMON_UCX_WARN
 
 typedef struct mca_pml_ucx_module           mca_pml_ucx_module_t;
 typedef struct pml_ucx_persistent_request   mca_pml_ucx_persistent_request_t;
@@ -57,12 +57,11 @@ struct mca_pml_ucx_module {
     mca_pml_ucx_freelist_t    convs;
 
     int                       priority;
-    bool                      cuda_initialized;
     bool                      request_leak_check;
     uint32_t                  op_attr_nonblocking;
 };
 
-extern mca_pml_base_component_2_0_0_t mca_pml_ucx_component;
+extern mca_pml_base_component_2_1_0_t mca_pml_ucx_component;
 extern mca_pml_ucx_module_t ompi_pml_ucx;
 
 int mca_pml_ucx_open(void);
