@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2016 University of Houston. All rights reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -32,7 +34,7 @@
 
 #include "ompi/mca/common/ompio/common_ompio.h"
 
-int mca_sharedfp_base_file_unselect(mca_io_ompio_file_t *file)
+int mca_sharedfp_base_file_unselect(ompio_file_t *file)
 {
     if (NULL != file->f_sharedfp && NULL != file->f_sharedfp->sharedfp_module_finalize) {
         return file->f_sharedfp->sharedfp_module_finalize(file);
