@@ -37,7 +37,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 
-#include "opal/util/fd.h"
+#include "opal/util/output.h"
 #include "opal/util/opal_getcwd.h"
 #include "opal/util/path.h"
 #include "opal/util/os_path.h"
@@ -152,7 +152,7 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
             free (lockedfilename);
             return OMPI_ERROR;
         }
-	err = opal_fd_write (handle, sizeof(OMPI_MPI_OFFSET_TYPE), &position);
+	err = opal_best_effort_write (handle, &position, sizeof(OMPI_MPI_OFFSET_TYPE));
         if (OPAL_SUCCESS != err)  {
             free (sh);
             free (module_data);
@@ -202,7 +202,7 @@ int mca_sharedfp_lockedfile_file_close (ompio_file_t *fh)
     }
     sh = fh->f_sharedfp_data;
 
-    module_data = (lockedfile_data*)(sh->selected_module_data);
+    module_data = (lockedfile_data_global*)(sh->selected_module_data);
     if ( module_data)   {
         /*Close lockedfile handle*/
         if ( module_data->handle)  {
