@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -43,8 +43,6 @@ int MPI_Add_error_class(int *errorclass)
     int err_class;
     int rc;
 
-    OPAL_CR_NOOP_PROGRESS();
-
     if ( MPI_PARAM_CHECK ) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
@@ -56,7 +54,7 @@ int MPI_Add_error_class(int *errorclass)
 
     err_class = ompi_mpi_errclass_add();
     if ( 0 > err_class ) {
-        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_INTERN,
+        return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_INTERN,
                                       FUNC_NAME);
     }
 
@@ -73,7 +71,7 @@ int MPI_Add_error_class(int *errorclass)
                               ompi_mpi_errcode_lastused,
                               true);
     if ( MPI_SUCCESS != rc ) {
-	return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, rc, FUNC_NAME);
+	return OMPI_ERRHANDLER_NOHANDLE_INVOKE(rc, FUNC_NAME);
     }
 
     *errorclass = err_class;
