@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -47,21 +47,19 @@ int MPI_Comm_free(MPI_Comm *comm)
     if ( MPI_PARAM_CHECK ) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
-        if ( NULL == *comm  || MPI_COMM_WORLD == *comm ||
+        if ( NULL == *comm ||
              ompi_comm_invalid (*comm)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_COMM,
                                           FUNC_NAME);
-        } else if (MPI_COMM_SELF == *comm) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_SELF, MPI_ERR_COMM,
+        } else if (MPI_COMM_WORLD == *comm ||
+                   MPI_COMM_SELF == *comm) {
+            return OMPI_ERRHANDLER_INVOKE(*comm, MPI_ERR_COMM,
                                           FUNC_NAME);
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     ret = ompi_comm_free ( comm );
     OMPI_ERRHANDLER_CHECK(ret, *comm, ret, FUNC_NAME);
 
-    OPAL_CR_EXIT_LIBRARY();
     return MPI_SUCCESS;
 }
