@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -11,9 +11,10 @@
  *                         All rights reserved.
  * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012      Sandia National Laboratories. All rights reserved.
+ * Copyright (c) 2010-2012 Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2017-2022 IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -54,7 +55,7 @@ int MPI_Comm_split_type(MPI_Comm comm, int split_type, int key,
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if ( ompi_comm_invalid ( comm )) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_COMM,
                                           FUNC_NAME);
         }
 
@@ -64,6 +65,8 @@ int MPI_Comm_split_type(MPI_Comm comm, int split_type, int key,
         }
 
         if ( MPI_COMM_TYPE_SHARED != split_type && // Same as OMPI_COMM_TYPE_NODE
+             MPI_COMM_TYPE_HW_UNGUIDED != split_type &&
+             MPI_COMM_TYPE_HW_GUIDED != split_type &&
 	     OMPI_COMM_TYPE_CLUSTER != split_type &&
 	     OMPI_COMM_TYPE_CU != split_type &&
 	     OMPI_COMM_TYPE_HOST != split_type &&
@@ -87,7 +90,40 @@ int MPI_Comm_split_type(MPI_Comm comm, int split_type, int key,
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
+#if OPAL_ENABLE_FT_MPI
+    /*
+     * An early check, so as to return early if we are using a broken
+     * communicator. This is not absolutely necessary since we will
+     * check for this, and other, error conditions during the operation.
+     */
+    if( OPAL_UNLIKELY(!ompi_comm_iface_create_check(comm, &rc)) ) {
+        OMPI_ERRHANDLER_RETURN(rc, comm, rc, FUNC_NAME);
+    }
+#endif
+
+    if ( MPI_COMM_TYPE_HW_GUIDED == split_type ) {
+        int flag;
+        opal_cstring_t *value = NULL;
+
+        /* MPI_Info is required for this split_type.
+         * Not an error condition, per MPI 4.0.
+         */
+        if ( MPI_INFO_NULL == info ) {
+            *newcomm = MPI_COMM_NULL;
+            rc = MPI_SUCCESS;
+            OMPI_ERRHANDLER_RETURN ( rc, comm, rc, FUNC_NAME);
+        }
+
+        /* MPI_Info with key "mpi_hw_resource_type" is required for this split_type.
+         * Not an error condition, per MPI 4.0.
+         */
+        ompi_info_get(info, "mpi_hw_resource_type", &value, &flag);
+        if ( !flag ) {
+            *newcomm = MPI_COMM_NULL;
+            rc = MPI_SUCCESS;
+            OMPI_ERRHANDLER_RETURN ( rc, comm, rc, FUNC_NAME);
+        }
+    }
 
     if( (MPI_COMM_SELF == comm) && (MPI_UNDEFINED == split_type) ) {
         *newcomm = MPI_COMM_NULL;
