@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -64,7 +64,7 @@ int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype o
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if (ompi_win_invalid(win)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_WIN, FUNC_NAME);
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_WIN, FUNC_NAME);
         } else if (origin_count < 0 || target_count < 0) {
             rc = MPI_ERR_COUNT;
         } else if (ompi_win_peer_invalid(win, target_rank) &&
@@ -98,8 +98,8 @@ int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype o
 
                     /* GET_ACCUMULATE, unlike REDUCE, can use with derived
                        datatypes with predefinied operations, with some
-                       restrictions outlined in MPI-2:6.3.4.  The derived
-                       datatype must be composed entierly from one predefined
+                       restrictions outlined in MPI-3:11.3.4.  The derived
+                       datatype must be composed entirely from one predefined
                        datatype (so you can do all the construction you want,
                        but at the bottom, you can only use one datatype, say,
                        MPI_INT).  If the datatype at the target isn't
@@ -132,8 +132,6 @@ int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype o
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     rc = ompi_win->w_osc_module->osc_get_accumulate(origin_addr,
                                                     origin_count,
                                                     origin_datatype,
