@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -50,13 +50,11 @@ int MPI_Group_intersection(MPI_Group group1, MPI_Group group2,
       if ( (MPI_GROUP_NULL == group1) || (MPI_GROUP_NULL == group2) ||
            ( NULL == group1) || (NULL == group2) ||
            (NULL == new_group) ) {
-          return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_GROUP,
+          return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_GROUP,
                                         FUNC_NAME);
       }
   }
 
-  OPAL_CR_ENTER_LIBRARY();
-
   err = ompi_group_intersection ( group1, group2,  new_group );
-  OMPI_ERRHANDLER_RETURN(err, MPI_COMM_WORLD, err, FUNC_NAME );
+  OMPI_ERRHANDLER_NOHANDLE_RETURN(err, err, FUNC_NAME );
 }
