@@ -57,7 +57,7 @@ int MPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount,
         memchecker_comm(comm);
         memchecker_datatype(datatype);
 
-        /* check receive buffer of current proccess, whether it's addressable. */
+        /* check receive buffer of current process, whether it's addressable. */
         memchecker_call(&opal_memchecker_base_isaddressable, recvbuf,
                         recvcount, datatype);
 
@@ -75,7 +75,7 @@ int MPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount,
         err = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (ompi_comm_invalid(comm)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_COMM,
                                           FUNC_NAME);
         }
 
@@ -102,8 +102,6 @@ int MPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount,
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     /* Invoke the coll component to perform the back-end operation */
 
     err = comm->c_coll->coll_ireduce_scatter_block(sendbuf, recvbuf, recvcount,
