@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2007 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -51,16 +51,14 @@ int MPI_Type_dup (MPI_Datatype type,
       OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
       if (NULL == type || MPI_DATATYPE_NULL == type ||
           NULL == newtype) {
-        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_TYPE,
+        return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_TYPE,
                                       FUNC_NAME );
       }
    }
 
-   OPAL_CR_ENTER_LIBRARY();
-
    if (OMPI_SUCCESS != (ret = ompi_datatype_duplicate( type, newtype))) {
        ompi_datatype_destroy( newtype );
-       OMPI_ERRHANDLER_RETURN( ret, MPI_COMM_WORLD, ret, FUNC_NAME );
+       OMPI_ERRHANDLER_NOHANDLE_RETURN( ret, ret, FUNC_NAME );
    }
 
    ompi_datatype_set_args( *newtype, 0, NULL, 0, NULL, 1, &type, MPI_COMBINER_DUP );
@@ -77,11 +75,9 @@ int MPI_Type_dup (MPI_Datatype type,
                                                      type->d_keyhash,
                                                      (*newtype)->d_keyhash))) {
            ompi_datatype_destroy(newtype);
-           OMPI_ERRHANDLER_RETURN( ret, MPI_COMM_WORLD, ret, FUNC_NAME );
+           OMPI_ERRHANDLER_NOHANDLE_RETURN( ret, ret, FUNC_NAME );
        }
    }
 
-   OPAL_CR_EXIT_LIBRARY();
-
    return MPI_SUCCESS;
 }
