@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -52,26 +52,24 @@ int MPI_Type_get_attr (MPI_Datatype type,
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (NULL == type || MPI_DATATYPE_NULL == type) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_TYPE,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_TYPE,
                                           FUNC_NAME );
         } else if ((NULL == attribute_val) || (NULL == flag)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(
                                           MPI_ERR_ARG,
                                           FUNC_NAME);
         } else if (MPI_KEYVAL_INVALID == type_keyval) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_KEYVAL,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_KEYVAL,
                                           FUNC_NAME);
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     /* This stuff is very confusing.  Be sure to see
        src/attribute/attribute.c for a lengthy comment explaining Open
        MPI attribute behavior. */
 
     ret = ompi_attr_get_c(type->d_keyhash, type_keyval,
                           (void**)attribute_val, flag);
-    OMPI_ERRHANDLER_RETURN(ret, MPI_COMM_WORLD,
+    OMPI_ERRHANDLER_NOHANDLE_RETURN(ret, 
                            MPI_ERR_OTHER, FUNC_NAME);
 }
