@@ -37,7 +37,7 @@ int MPI_Win_set_info(MPI_Win win, MPI_Info info)
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if (ompi_win_invalid(win)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_WIN, FUNC_NAME);
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_WIN, FUNC_NAME);
         }
 
         if (NULL == info || MPI_INFO_NULL == info ||
@@ -46,8 +46,6 @@ int MPI_Win_set_info(MPI_Win win, MPI_Info info)
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     ret = opal_infosubscribe_change_info(&(win->super), &(info->super));
 
     OMPI_ERRHANDLER_RETURN(ret, win, ret, FUNC_NAME);
