@@ -4,6 +4,11 @@
  *                         reserved.
  * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2020      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2021      Amazon.com, Inc. or its affiliates.  All Rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -11,17 +16,17 @@
  * $HEADER$
  */
 
+#include "ompi_config.h"
+
 #include "ompi/mpi/tool/mpit-internal.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_T_pvar_session_create = PMPI_T_pvar_session_create
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/tool/profile/defines.h"
+#define MPI_T_pvar_session_create PMPI_T_pvar_session_create
 #endif
 
-
 int MPI_T_pvar_session_create(MPI_T_pvar_session *session)
 {
     int ret = MPI_SUCCESS;
@@ -34,7 +39,7 @@ int MPI_T_pvar_session_create(MPI_T_pvar_session *session)
 
     *session = OBJ_NEW(mca_base_pvar_session_t);
     if (NULL == *session) {
-        ret = MPI_ERR_NO_MEM;
+        ret = MPI_T_ERR_MEMORY;
     }
 
     ompi_mpit_unlock ();
