@@ -1,4 +1,11 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
+ * $COPYRIGHT$
+ *
+ * Additional copyrights may follow
+ *
  * $HEADER$
  */
 #if defined(c_plusplus) || defined(__cplusplus)
@@ -8,7 +15,6 @@ extern "C" {
 #include "ompi_config.h"
 
 OMPI_DECLSPEC int ompi_mpiext_init(void);
-OMPI_DECLSPEC int ompi_mpiext_fini(void);
 
 typedef int (*ompi_mpiext_init_fn_t)(void);
 typedef int (*ompi_mpiext_fini_fn_t)(void);
