@@ -15,8 +15,11 @@
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * Copyright (c) 2019      Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
  * Copyright (c) 2021      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -40,6 +43,7 @@
 #include "mpi.h"
 
 #include "opal/class/opal_object.h"
+#include "opal/util/printf.h"
 
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/mpi/fortran/base/fint_2_int.h"
@@ -64,25 +68,9 @@ typedef void (ompi_op_c_handler_fn_t)(void *, void *, int *,
 typedef void (ompi_op_fortran_handler_fn_t)(void *, void *,
                                             MPI_Fint *, MPI_Fint *);
 
-/**
- * Typedef for C++ op functions intercept (used for user-defined
- * MPI::Ops).
- *
- * See the lengthy explanation for why this is different than the C
- * intercept in ompi/mpi/cxx/intercepts.cc in the
- * ompi_mpi_cxx_op_intercept() function.
- */
-typedef void (ompi_op_cxx_handler_fn_t)(void *, void *, int *,
-                                        struct ompi_datatype_t **,
-                                        MPI_User_function * op);
-
 /**
  * Typedef for Java op functions intercept (used for user-defined
  * MPI.Ops).
- *
- * See the lengthy explanation for why this is different than the C
- * intercept in ompi/mpi/cxx/intercepts.cc in the
- * ompi_mpi_cxx_op_intercept() function.
  */
 typedef void (ompi_op_java_handler_fn_t)(void *, void *, int *,
                                          struct ompi_datatype_t **,
@@ -96,8 +84,6 @@ typedef void (ompi_op_java_handler_fn_t)(void *, void *, int *,
 #define OMPI_OP_FLAGS_INTRINSIC    0x0001
 /** Set if the callback function is in Fortran */
 #define OMPI_OP_FLAGS_FORTRAN_FUNC 0x0002
-/** Set if the callback function is in C++ */
-#define OMPI_OP_FLAGS_CXX_FUNC     0x0004
 /** Set if the callback function is in Java */
 #define OMPI_OP_FLAGS_JAVA_FUNC    0x0008
 /** Set if the callback function is associative (MAX and SUM will both
@@ -168,15 +154,7 @@ struct ompi_op_t {
         ompi_op_c_handler_fn_t *c_fn;
         /** Fortran handler function pointer */
         ompi_op_fortran_handler_fn_t *fort_fn;
-        /** C++ intercept function data -- see lengthy comment in
-            ompi/mpi/cxx/intercepts.cc::ompi_mpi_cxx_op_intercept() for
-            an explanation */
-        struct {
-            /* The user's function (it's the wrong type, but that's ok) */
-            ompi_op_c_handler_fn_t *user_fn;
-            /* The OMPI C++ callback/intercept function */
-            ompi_op_cxx_handler_fn_t *intercept_fn;
-        } cxx_data;
+        /** Java intercept function data */
         struct {
             /* The OMPI C++ callback/intercept function */
             ompi_op_java_handler_fn_t *intercept_fn;
@@ -198,7 +176,7 @@ typedef struct ompi_op_t ompi_op_t;
 OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_op_t);
 
 /**
- * Padded struct to maintain back compatibiltiy.
+ * Padded struct to maintain back compatibility.
  * See ompi/communicator/communicator.h comments with struct ompi_communicator_t
  * for full explanation why we chose the following padding construct for predefines.
  */
@@ -349,16 +327,6 @@ extern struct opal_pointer_array_t *ompi_op_f_to_c_table;
  */
 int ompi_op_init(void);
 
-/**
- * Finalize the op interface.
- *
- * @returns OMPI_SUCCESS Always
- *
- * Invokes from ompi_mpi_finalize(); tears down the op interface, and
- * destroys the F2C translation table.
- */
-int ompi_op_finalize(void);
-
 /**
  * Create a ompi_op_t with a user-defined callback (vs. creating an
  * intrinsic ompi_op_t).
@@ -390,17 +358,8 @@ ompi_op_t *ompi_op_create_user(bool commute,
                                ompi_op_fortran_handler_fn_t func);
 
 /**
- * Mark an MPI_Op as holding a C++ callback function, and cache
- * that function in the MPI_Op.  See a lenghty comment in
- * ompi/mpi/cxx/op.c::ompi_mpi_cxx_op_intercept() for a full
- * expalantion.
- */
-OMPI_DECLSPEC void ompi_op_set_cxx_callback(ompi_op_t * op,
-                                            MPI_User_function * fn);
-
-/**
- * Similar to ompi_op_set_cxx_callback(), mark an MPI_Op as holding a
- * Java calback function, and cache that function in the MPI_Op.
+ * Mark an MPI_Op as holding a Java callback function, and cache that
+ * function in the MPI_Op.
  */
 OMPI_DECLSPEC void ompi_op_set_java_callback(ompi_op_t *op,  void *jnienv,
                                              void *object, int baseType);
@@ -486,7 +445,7 @@ static inline bool ompi_op_is_valid(ompi_op_t * op, ompi_datatype_t * ddt,
             /* Intrinsic ddt on intrinsic op */
             if (-1 == ompi_op_ddt_map[ddt->id] ||
                 NULL == op->o_func.intrinsic.fns[ompi_op_ddt_map[ddt->id]]) {
-                (void) asprintf(msg,
+                (void) opal_asprintf(msg,
                                 "%s: the reduction operation %s is not defined on the %s datatype",
                                 func, op->o_name, ddt->name);
                 return false;
@@ -494,11 +453,11 @@ static inline bool ompi_op_is_valid(ompi_op_t * op, ompi_datatype_t * ddt,
         } else {
             /* Non-intrinsic ddt on intrinsic op */
             if ('\0' != ddt->name[0]) {
-                (void) asprintf(msg,
+                (void) opal_asprintf(msg,
                                 "%s: the reduction operation %s is not defined for non-intrinsic datatypes (attempted with datatype named \"%s\")",
                                 func, op->o_name, ddt->name);
             } else {
-                (void) asprintf(msg,
+                (void) opal_asprintf(msg,
                                 "%s: the reduction operation %s is not defined for non-intrinsic datatypes",
                                 func, op->o_name);
             }
@@ -622,10 +581,6 @@ static inline void ompi_op_reduce(ompi_op_t * op, void *source,
         f_count = OMPI_INT_2_FINT(count);
         op->o_func.fort_fn(source, target, &f_count, &f_dtype);
         return;
-    } else if (0 != (op->o_flags & OMPI_OP_FLAGS_CXX_FUNC)) {
-        op->o_func.cxx_data.intercept_fn(source, target, &count, &dtype,
-                                         op->o_func.cxx_data.user_fn);
-        return;
     } else if (0 != (op->o_flags & OMPI_OP_FLAGS_JAVA_FUNC)) {
         op->o_func.java_data.intercept_fn(source, target, &count, &dtype,
                                           op->o_func.java_data.baseType,
