@@ -22,11 +22,11 @@
 #include "opal_config.h"
 #include "opal/class/opal_lifo.h"
 
-static void opal_lifo_construct (opal_lifo_t *lifo)
+static void opal_lifo_construct(opal_lifo_t *lifo)
 {
     OBJ_CONSTRUCT(&lifo->opal_lifo_ghost, opal_list_item_t);
     lifo->opal_lifo_ghost.opal_list_next = &lifo->opal_lifo_ghost;
-    lifo->opal_lifo_head.data.item = &lifo->opal_lifo_ghost;
+    lifo->opal_lifo_head.data.item = (intptr_t) &lifo->opal_lifo_ghost;
     lifo->opal_lifo_head.data.counter = 0;
 }
 
