@@ -12,6 +12,8 @@
  * Copyright (c) 2007      Voltaire All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -66,14 +68,14 @@
 #define OPAL_LIST_H
 
 #include "opal_config.h"
+#include "opal/class/opal_object.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include "opal/class/opal_object.h"
 
 #if OPAL_ENABLE_DEBUG
 /* Need atomics for debugging (reference counting) */
-#include "opal/sys/atomic.h"
-#include "opal/threads/mutex.h"
+#    include "opal/mca/threads/mutex.h"
+#    include "opal/sys/atomic.h"
 #endif
 
 BEGIN_C_DECLS
@@ -91,27 +93,25 @@ OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_list_t);
  */
 OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_list_item_t);
 
-
 /**
  * \internal
  *
  * Struct of an opal_list_item_t
  */
-struct opal_list_item_t
-{
+struct opal_list_item_t {
     opal_object_t super;
     /**< Generic parent class for all Open MPI objects */
-    volatile struct opal_list_item_t * volatile opal_list_next;
+    volatile struct opal_list_item_t *volatile opal_list_next;
     /**< Pointer to next list item */
-    volatile struct opal_list_item_t * volatile opal_list_prev;
+    volatile struct opal_list_item_t *volatile opal_list_prev;
     /**< Pointer to previous list item */
     int32_t item_free;
 
 #if OPAL_ENABLE_DEBUG
     /** Atomic reference count for debugging */
-    volatile int32_t opal_list_item_refcount;
+    opal_atomic_int32_t opal_list_item_refcount;
     /** The list this item belong to */
-    volatile struct opal_list_t* opal_list_item_belong_to;
+    volatile struct opal_list_t *opal_list_item_belong_to;
 #endif
 };
 /**
@@ -119,7 +119,6 @@ struct opal_list_item_t
  */
 typedef struct opal_list_item_t opal_list_item_t;
 
-
 /**
  * Get the next item in a list.
  *
@@ -128,7 +127,7 @@ typedef struct opal_list_item_t opal_list_item_t;
  * @returns The next item in the list
  */
 #define opal_list_get_next(item) \
-    ((item) ? ((opal_list_item_t*) ((opal_list_item_t*)(item))->opal_list_next) : NULL)
+    ((item) ? ((opal_list_item_t *) ((opal_list_item_t *) (item))->opal_list_next) : NULL)
 
 /**
  * Get the next item in a list.
@@ -138,21 +137,19 @@ typedef struct opal_list_item_t opal_list_item_t;
  * @returns The next item in the list
  */
 #define opal_list_get_prev(item) \
-    ((item) ? ((opal_list_item_t*) ((opal_list_item_t*)(item))->opal_list_prev) : NULL)
-
+    ((item) ? ((opal_list_item_t *) ((opal_list_item_t *) (item))->opal_list_prev) : NULL)
 
 /**
  * \internal
  *
  * Struct of an opal_list_t
  */
-struct opal_list_t
-{
-    opal_object_t       super;
+struct opal_list_t {
+    opal_object_t super;
     /**< Generic parent class for all Open MPI objects */
-    opal_list_item_t    opal_list_sentinel;
+    opal_list_item_t opal_list_sentinel;
     /**< Head and tail item of the list */
-    volatile size_t     opal_list_length;
+    volatile size_t opal_list_length;
     /**< Quick reference to the number of items in the list */
 };
 /**
@@ -168,24 +165,27 @@ typedef struct opal_list_t opal_list_t;
  *
  * @param[in] list List to destruct or release
  */
-#define OPAL_LIST_DESTRUCT(list)                                \
-    do {                                                        \
-        opal_list_item_t *it;                                   \
-        while (NULL != (it = opal_list_remove_first(list))) {   \
-            OBJ_RELEASE(it);                                    \
-        }                                                       \
-        OBJ_DESTRUCT(list);                                     \
-    } while(0);
-
-#define OPAL_LIST_RELEASE(list)                                 \
-    do {                                                        \
-        opal_list_item_t *it;                                   \
-        while (NULL != (it = opal_list_remove_first(list))) {   \
-            OBJ_RELEASE(it);                                    \
-        }                                                       \
-        OBJ_RELEASE(list);                                      \
-    } while(0);
+#define OPAL_LIST_DESTRUCT(list)                                    \
+    do {                                                            \
+        opal_list_item_t *it;                                       \
+        if (1 == ((opal_object_t *) (list))->obj_reference_count) { \
+            while (NULL != (it = opal_list_remove_first(list))) {   \
+                OBJ_RELEASE(it);                                    \
+            }                                                       \
+        }                                                           \
+        OBJ_DESTRUCT(list);                                         \
+    } while (0);
 
+#define OPAL_LIST_RELEASE(list)                                     \
+    do {                                                            \
+        opal_list_item_t *it;                                       \
+        if (1 == ((opal_object_t *) (list))->obj_reference_count) { \
+            while (NULL != (it = opal_list_remove_first(list))) {   \
+                OBJ_RELEASE(it);                                    \
+            }                                                       \
+        }                                                           \
+        OBJ_RELEASE(list);                                          \
+    } while (0);
 
 /**
  * Loop over a list.
@@ -200,14 +200,19 @@ typedef struct opal_list_t opal_list_t;
  * Example Usage:
  *
  * class_foo_t *foo;
- * opal_list_foreach(foo, list, class_foo_t) {
- *    do something;
+ * OPAL_LIST_FOREACH(foo, list, class_foo_t) {
+ *    do something(foo);
  * }
  */
-#define OPAL_LIST_FOREACH(item, list, type)                             \
-  for (item = (type *) (list)->opal_list_sentinel.opal_list_next ;      \
-       item != (type *) &(list)->opal_list_sentinel ;                   \
-       item = (type *) ((opal_list_item_t *) (item))->opal_list_next)
+#define OPAL_LIST_FOREACH(item, list, type)                         \
+    for (item = (type *) (list)->opal_list_sentinel.opal_list_next; \
+         item != (type *) &(list)->opal_list_sentinel;              \
+         item = (type *) ((opal_list_item_t *) (item))->opal_list_next)
+
+#define OPAL_LIST_FOREACH_DECL(item, list, type)                          \
+    for (type *item = (type *) (list)->opal_list_sentinel.opal_list_next; \
+         item != (type *) &(list)->opal_list_sentinel;                    \
+         item = (type *) ((opal_list_item_t *) (item))->opal_list_next)
 
 /**
  * Loop over a list in reverse.
@@ -226,10 +231,10 @@ typedef struct opal_list_t opal_list_t;
  *    do something;
  * }
  */
-#define OPAL_LIST_FOREACH_REV(item, list, type)                         \
-  for (item = (type *) (list)->opal_list_sentinel.opal_list_prev ;      \
-       item != (type *) &(list)->opal_list_sentinel ;                   \
-       item = (type *) ((opal_list_item_t *) (item))->opal_list_prev)
+#define OPAL_LIST_FOREACH_REV(item, list, type)                     \
+    for (item = (type *) (list)->opal_list_sentinel.opal_list_prev; \
+         item != (type *) &(list)->opal_list_sentinel;              \
+         item = (type *) ((opal_list_item_t *) (item))->opal_list_prev)
 
 /**
  * Loop over a list in a *safe* way
@@ -250,11 +255,11 @@ typedef struct opal_list_t opal_list_t;
  *    opal_list_remove_item (list, (opal_list_item_t *) foo);
  * }
  */
-#define OPAL_LIST_FOREACH_SAFE(item, next, list, type)                  \
-  for (item = (type *) (list)->opal_list_sentinel.opal_list_next,       \
-         next = (type *) ((opal_list_item_t *) (item))->opal_list_next ;\
-       item != (type *) &(list)->opal_list_sentinel ;                   \
-       item = next, next = (type *) ((opal_list_item_t *) (item))->opal_list_next)
+#define OPAL_LIST_FOREACH_SAFE(item, next, list, type)                 \
+    for (item = (type *) (list)->opal_list_sentinel.opal_list_next,    \
+        next = (type *) ((opal_list_item_t *) (item))->opal_list_next; \
+         item != (type *) &(list)->opal_list_sentinel;                 \
+         item = next, next = (type *) ((opal_list_item_t *) (item))->opal_list_next)
 
 /**
  * Loop over a list in a *safe* way
@@ -275,12 +280,11 @@ typedef struct opal_list_t opal_list_t;
  *    opal_list_remove_item (list, (opal_list_item_t *) foo);
  * }
  */
-#define OPAL_LIST_FOREACH_SAFE_REV(item, prev, list, type)              \
-  for (item = (type *) (list)->opal_list_sentinel.opal_list_prev,       \
-         prev = (type *) ((opal_list_item_t *) (item))->opal_list_prev ;\
-       item != (type *) &(list)->opal_list_sentinel ;                   \
-       item = prev, prev = (type *) ((opal_list_item_t *) (item))->opal_list_prev)
-
+#define OPAL_LIST_FOREACH_SAFE_REV(item, prev, list, type)             \
+    for (item = (type *) (list)->opal_list_sentinel.opal_list_prev,    \
+        prev = (type *) ((opal_list_item_t *) (item))->opal_list_prev; \
+         item != (type *) &(list)->opal_list_sentinel;                 \
+         item = prev, prev = (type *) ((opal_list_item_t *) (item))->opal_list_prev)
 
 /**
  * Check for empty list
@@ -294,13 +298,11 @@ typedef struct opal_list_t opal_list_t;
  * This is an inlined function in compilers that support inlining,
  * so it's usually a cheap operation.
  */
-static inline bool opal_list_is_empty(opal_list_t* list)
+static inline bool opal_list_is_empty(opal_list_t *list)
 {
-    return (list->opal_list_sentinel.opal_list_next ==
-        &(list->opal_list_sentinel) ? true : false);
+    return (list->opal_list_sentinel.opal_list_next == &(list->opal_list_sentinel) ? true : false);
 }
 
-
 /**
  * Return the first item on the list (does not remove it).
  *
@@ -315,14 +317,14 @@ static inline bool opal_list_is_empty(opal_list_t* list)
  * This is an inlined function in compilers that support inlining, so
  * it's usually a cheap operation.
  */
-static inline opal_list_item_t* opal_list_get_first(opal_list_t* list)
+static inline opal_list_item_t *opal_list_get_first(opal_list_t *list)
 {
-    opal_list_item_t* item = (opal_list_item_t*)list->opal_list_sentinel.opal_list_next;
+    opal_list_item_t *item = (opal_list_item_t *) list->opal_list_sentinel.opal_list_next;
 #if OPAL_ENABLE_DEBUG
     /* Spot check: ensure that the first item is only on one list */
 
     assert(1 == item->opal_list_item_refcount);
-    assert( list == item->opal_list_item_belong_to );
+    assert(list == item->opal_list_item_belong_to);
 #endif
 
     return item;
@@ -342,14 +344,14 @@ static inline opal_list_item_t* opal_list_get_first(opal_list_t* list)
  * This is an inlined function in compilers that support inlining, so
  * it's usually a cheap operation.
  */
-static inline opal_list_item_t* opal_list_get_last(opal_list_t* list)
+static inline opal_list_item_t *opal_list_get_last(opal_list_t *list)
 {
-    opal_list_item_t* item = (opal_list_item_t *)list->opal_list_sentinel.opal_list_prev;
+    opal_list_item_t *item = (opal_list_item_t *) list->opal_list_sentinel.opal_list_prev;
 #if OPAL_ENABLE_DEBUG
     /* Spot check: ensure that the last item is only on one list */
 
-    assert( 1 == item->opal_list_item_refcount );
-    assert( list == item->opal_list_item_belong_to );
+    assert(1 == item->opal_list_item_refcount);
+    assert(list == item->opal_list_item_belong_to);
 #endif
 
     return item;
@@ -367,12 +369,12 @@ static inline opal_list_item_t* opal_list_get_last(opal_list_t* list)
  * Similar to the STL, this is a special invalid list item -- it
  * should \em not be used for storage.  It is only suitable for
  * comparison to other items in the list to see if they are valid or
- * not; it's ususally used when iterating through the items in a list.
+ * not; it's usually used when iterating through the items in a list.
  *
  * This is an inlined function in compilers that support inlining, so
  * it's usually a cheap operation.
  */
-static inline opal_list_item_t* opal_list_get_begin(opal_list_t* list)
+static inline opal_list_item_t *opal_list_get_begin(opal_list_t *list)
 {
     return &(list->opal_list_sentinel);
 }
@@ -389,17 +391,16 @@ static inline opal_list_item_t* opal_list_get_begin(opal_list_t* list)
  * Similar to the STL, this is a special invalid list item -- it
  * should \em not be used for storage.  It is only suitable for
  * comparison to other items in the list to see if they are valid or
- * not; it's ususally used when iterating through the items in a list.
+ * not; it's usually used when iterating through the items in a list.
  *
  * This is an inlined function in compilers that support inlining, so
  * it's usually a cheap operation.
  */
-static inline opal_list_item_t* opal_list_get_end(opal_list_t* list)
+static inline opal_list_item_t *opal_list_get_end(opal_list_t *list)
 {
     return &(list->opal_list_sentinel);
 }
 
-
 /**
  * Return the number of items in a list
  *
@@ -419,7 +420,7 @@ static inline opal_list_item_t* opal_list_get_end(opal_list_t* list)
  * never called on the specified list, this function will always be
  * O(1).
  */
-static inline size_t opal_list_get_size(opal_list_t* list)
+static inline size_t opal_list_get_size(opal_list_t *list)
 {
 #if OPAL_ENABLE_DEBUG && 0
     /* not sure if we really want this running in devel, as it does
@@ -429,14 +430,15 @@ static inline size_t opal_list_get_size(opal_list_t* list)
     size_t check_len = 0;
     opal_list_item_t *item;
 
-    for (item = opal_list_get_first(list) ;
-         item != opal_list_get_end(list) ;
+    for (item = opal_list_get_first(list); item != opal_list_get_end(list);
          item = opal_list_get_next(item)) {
         check_len++;
     }
 
     if (check_len != list->opal_list_length) {
-        fprintf(stderr," Error :: opal_list_get_size - opal_list_length does not match actual list length\n");
+        fprintf(
+            stderr,
+            " Error :: opal_list_get_size - opal_list_length does not match actual list length\n");
         fflush(stderr);
         abort();
     }
@@ -445,7 +447,6 @@ static inline size_t opal_list_get_size(opal_list_t* list)
     return list->opal_list_length;
 }
 
-
 /**
  * Remove an item from a list.
  *
@@ -467,51 +468,49 @@ static inline size_t opal_list_get_size(opal_list_t* list)
  * This is an inlined function in compilers that support inlining, so
  * it's usually a cheap operation.
  */
-static inline opal_list_item_t *opal_list_remove_item
-  (opal_list_t *list, opal_list_item_t *item)
+static inline opal_list_item_t *opal_list_remove_item(opal_list_t *list, opal_list_item_t *item)
 {
 #if OPAL_ENABLE_DEBUG
     opal_list_item_t *item_ptr;
     bool found = false;
 
     /* check to see that the item is in the list */
-    for (item_ptr = opal_list_get_first(list);
-            item_ptr != opal_list_get_end(list);
-            item_ptr = (opal_list_item_t *)(item_ptr->opal_list_next)) {
+    for (item_ptr = opal_list_get_first(list); item_ptr != opal_list_get_end(list);
+         item_ptr = (opal_list_item_t *) (item_ptr->opal_list_next)) {
         if (item_ptr == (opal_list_item_t *) item) {
             found = true;
             break;
         }
     }
     if (!found) {
-        fprintf(stderr," Warning :: opal_list_remove_item - the item %p is not on the list %p \n",(void*) item, (void*) list);
+        fprintf(stderr, " Warning :: opal_list_remove_item - the item %p is not on the list %p \n",
+                (void *) item, (void *) list);
         fflush(stderr);
-        return (opal_list_item_t *)NULL;
+        return (opal_list_item_t *) NULL;
     }
 
-    assert( list == item->opal_list_item_belong_to );
+    assert(list == item->opal_list_item_belong_to);
 #endif
 
     /* reset next pointer of previous element */
-    item->opal_list_prev->opal_list_next=item->opal_list_next;
+    item->opal_list_prev->opal_list_next = item->opal_list_next;
 
     /* reset previous pointer of next element */
-    item->opal_list_next->opal_list_prev=item->opal_list_prev;
+    item->opal_list_next->opal_list_prev = item->opal_list_prev;
 
     list->opal_list_length--;
 
 #if OPAL_ENABLE_DEBUG
     /* Spot check: ensure that this item is still only on one list */
 
-    OPAL_THREAD_ADD32( &(item->opal_list_item_refcount), -1 );
+    OPAL_THREAD_ADD_FETCH32(&(item->opal_list_item_refcount), -1);
     assert(0 == item->opal_list_item_refcount);
     item->opal_list_item_belong_to = NULL;
 #endif
 
-    return (opal_list_item_t *)item->opal_list_prev;
+    return (opal_list_item_t *) item->opal_list_prev;
 }
 
-
 /**
  * Append an item to the end of the list.
  *
@@ -527,55 +526,53 @@ static inline opal_list_item_t *opal_list_remove_item
  */
 
 #if OPAL_ENABLE_DEBUG
-#define opal_list_append(l,i) \
-_opal_list_append(l,i,__FILE__,__LINE__)
+#    define opal_list_append(l, i) _opal_list_append(l, i, __FILE__, __LINE__)
 #else
-#define opal_list_append(l,i) \
-_opal_list_append(l,i)
-#endif  /* OPAL_ENABLE_DEBUG */
+#    define opal_list_append(l, i) _opal_list_append(l, i)
+#endif /* OPAL_ENABLE_DEBUG */
 
 static inline void _opal_list_append(opal_list_t *list, opal_list_item_t *item
 #if OPAL_ENABLE_DEBUG
-                                     , const char* FILE_NAME, int LINENO
-#endif  /* OPAL_ENABLE_DEBUG */
-                                     )
+                                     ,
+                                     const char *FILE_NAME, int LINENO
+#endif /* OPAL_ENABLE_DEBUG */
+)
 {
-    opal_list_item_t* sentinel = &(list->opal_list_sentinel);
+    opal_list_item_t *sentinel = &(list->opal_list_sentinel);
 #if OPAL_ENABLE_DEBUG
-  /* Spot check: ensure that this item is previously on no lists */
+    /* Spot check: ensure that this item is previously on no lists */
 
-  assert(0 == item->opal_list_item_refcount);
-  assert( NULL == item->opal_list_item_belong_to );
-  item->super.cls_init_file_name = FILE_NAME;
-  item->super.cls_init_lineno    = LINENO;
+    assert(0 == item->opal_list_item_refcount);
+    assert(NULL == item->opal_list_item_belong_to);
+    item->super.cls_init_file_name = FILE_NAME;
+    item->super.cls_init_lineno = LINENO;
 #endif
 
-  /* set new element's previous pointer */
-  item->opal_list_prev = sentinel->opal_list_prev;
+    /* set new element's previous pointer */
+    item->opal_list_prev = sentinel->opal_list_prev;
 
-  /* reset previous pointer on current last element */
-  sentinel->opal_list_prev->opal_list_next = item;
+    /* reset previous pointer on current last element */
+    sentinel->opal_list_prev->opal_list_next = item;
 
-  /* reset new element's next pointer */
-  item->opal_list_next = sentinel;
+    /* reset new element's next pointer */
+    item->opal_list_next = sentinel;
 
-  /* reset the list's tail element previous pointer */
-  sentinel->opal_list_prev = item;
+    /* reset the list's tail element previous pointer */
+    sentinel->opal_list_prev = item;
 
-  /* increment list element counter */
-  list->opal_list_length++;
+    /* increment list element counter */
+    list->opal_list_length++;
 
 #if OPAL_ENABLE_DEBUG
-  /* Spot check: ensure this item is only on the list that we just
-     appended it to */
+    /* Spot check: ensure this item is only on the list that we just
+       appended it to */
 
-  OPAL_THREAD_ADD32( &(item->opal_list_item_refcount), 1 );
-  assert(1 == item->opal_list_item_refcount);
-  item->opal_list_item_belong_to = list;
+    OPAL_THREAD_ADD_FETCH32(&(item->opal_list_item_refcount), 1);
+    assert(1 == item->opal_list_item_refcount);
+    item->opal_list_item_belong_to = list;
 #endif
 }
 
-
 /**
  * Prepend an item to the beginning of the list.
  *
@@ -589,43 +586,41 @@ static inline void _opal_list_append(opal_list_t *list, opal_list_item_t *item
  * This is an inlined function in compilers that support inlining, so
  * it's usually a cheap operation.
  */
-static inline void opal_list_prepend(opal_list_t *list,
-                                     opal_list_item_t *item)
+static inline void opal_list_prepend(opal_list_t *list, opal_list_item_t *item)
 {
-    opal_list_item_t* sentinel = &(list->opal_list_sentinel);
+    opal_list_item_t *sentinel = &(list->opal_list_sentinel);
 #if OPAL_ENABLE_DEBUG
-  /* Spot check: ensure that this item is previously on no lists */
+    /* Spot check: ensure that this item is previously on no lists */
 
-  assert(0 == item->opal_list_item_refcount);
-  assert( NULL == item->opal_list_item_belong_to );
+    assert(0 == item->opal_list_item_refcount);
+    assert(NULL == item->opal_list_item_belong_to);
 #endif
 
-  /* reset item's next pointer */
-  item->opal_list_next = sentinel->opal_list_next;
+    /* reset item's next pointer */
+    item->opal_list_next = sentinel->opal_list_next;
 
-  /* reset item's previous pointer */
-  item->opal_list_prev = sentinel;
+    /* reset item's previous pointer */
+    item->opal_list_prev = sentinel;
 
-  /* reset previous first element's previous poiner */
-  sentinel->opal_list_next->opal_list_prev = item;
+    /* reset previous first element's previous pointer */
+    sentinel->opal_list_next->opal_list_prev = item;
 
-  /* reset head's next pointer */
-  sentinel->opal_list_next = item;
+    /* reset head's next pointer */
+    sentinel->opal_list_next = item;
 
-  /* increment list element counter */
-  list->opal_list_length++;
+    /* increment list element counter */
+    list->opal_list_length++;
 
 #if OPAL_ENABLE_DEBUG
-  /* Spot check: ensure this item is only on the list that we just
-     prepended it to */
+    /* Spot check: ensure this item is only on the list that we just
+       prepended it to */
 
-  OPAL_THREAD_ADD32( &(item->opal_list_item_refcount), 1 );
-  assert(1 == item->opal_list_item_refcount);
-  item->opal_list_item_belong_to = list;
+    OPAL_THREAD_ADD_FETCH32(&(item->opal_list_item_refcount), 1);
+    assert(1 == item->opal_list_item_refcount);
+    item->opal_list_item_belong_to = list;
 #endif
 }
 
-
 /**
  * Remove the first item from the list and return it.
  *
@@ -644,50 +639,49 @@ static inline void opal_list_prepend(opal_list_t *list,
  */
 static inline opal_list_item_t *opal_list_remove_first(opal_list_t *list)
 {
-  /*  Removes and returns first item on list.
-      Caller now owns the item and should release the item
-      when caller is done with it.
-  */
-  volatile opal_list_item_t *item;
-  if ( 0 == list->opal_list_length ) {
-    return (opal_list_item_t *)NULL;
-  }
+    /*  Removes and returns first item on list.
+        Caller now owns the item and should release the item
+        when caller is done with it.
+    */
+    opal_list_item_t *item;
+    if (0 == list->opal_list_length) {
+        return (opal_list_item_t *) NULL;
+    }
 
 #if OPAL_ENABLE_DEBUG
-  /* Spot check: ensure that the first item is only on this list */
+    /* Spot check: ensure that the first item is only on this list */
 
-  assert(1 == list->opal_list_sentinel.opal_list_next->opal_list_item_refcount);
+    assert(1 == list->opal_list_sentinel.opal_list_next->opal_list_item_refcount);
 #endif
 
-  /* reset list length counter */
-  list->opal_list_length--;
+    /* reset list length counter */
+    list->opal_list_length--;
 
-  /* get pointer to first element on the list */
-  item = list->opal_list_sentinel.opal_list_next;
+    /* get pointer to first element on the list */
+    item = (opal_list_item_t *) list->opal_list_sentinel.opal_list_next;
 
-  /* reset previous pointer of next item on the list */
-  item->opal_list_next->opal_list_prev = item->opal_list_prev;
+    /* reset previous pointer of next item on the list */
+    item->opal_list_next->opal_list_prev = item->opal_list_prev;
 
-  /* reset the head next pointer */
-  list->opal_list_sentinel.opal_list_next = item->opal_list_next;
+    /* reset the head next pointer */
+    list->opal_list_sentinel.opal_list_next = item->opal_list_next;
 
 #if OPAL_ENABLE_DEBUG
-  assert( list == item->opal_list_item_belong_to );
-  item->opal_list_item_belong_to = NULL;
-  item->opal_list_prev=(opal_list_item_t *)NULL;
-  item->opal_list_next=(opal_list_item_t *)NULL;
+    assert(list == item->opal_list_item_belong_to);
+    item->opal_list_item_belong_to = NULL;
+    item->opal_list_prev = (opal_list_item_t *) NULL;
+    item->opal_list_next = (opal_list_item_t *) NULL;
 
-  /* Spot check: ensure that the item we're returning is now on no
-     lists */
+    /* Spot check: ensure that the item we're returning is now on no
+       lists */
 
-  OPAL_THREAD_ADD32( &(item->opal_list_item_refcount), -1 );
-  assert(0 == item->opal_list_item_refcount);
+    OPAL_THREAD_ADD_FETCH32(&item->opal_list_item_refcount, -1);
+    assert(0 == item->opal_list_item_refcount);
 #endif
 
-  return (opal_list_item_t *) item;
+    return item;
 }
 
-
 /**
  * Remove the last item from the list and return it.
  *
@@ -706,57 +700,57 @@ static inline opal_list_item_t *opal_list_remove_first(opal_list_t *list)
  */
 static inline opal_list_item_t *opal_list_remove_last(opal_list_t *list)
 {
-  /*  Removes, releases and returns last item on list.
-      Caller now owns the item and should release the item
-      when caller is done with it.
-  */
-  volatile opal_list_item_t  *item;
-  if ( 0 == list->opal_list_length ) {
-      return (opal_list_item_t *)NULL;
-  }
+    /*  Removes, releases and returns last item on list.
+        Caller now owns the item and should release the item
+        when caller is done with it.
+    */
+    opal_list_item_t *item;
+    if (0 == list->opal_list_length) {
+        return (opal_list_item_t *) NULL;
+    }
 
 #if OPAL_ENABLE_DEBUG
-  /* Spot check: ensure that the first item is only on this list */
+    /* Spot check: ensure that the first item is only on this list */
 
-  assert(1 == list->opal_list_sentinel.opal_list_prev->opal_list_item_refcount);
+    assert(1 == list->opal_list_sentinel.opal_list_prev->opal_list_item_refcount);
 #endif
 
-  /* reset list length counter */
-  list->opal_list_length--;
+    /* reset list length counter */
+    list->opal_list_length--;
 
-  /* get item */
-  item = list->opal_list_sentinel.opal_list_prev;
+    /* get item */
+    item = (opal_list_item_t *) list->opal_list_sentinel.opal_list_prev;
 
-  /* reset previous pointer on next to last pointer */
-  item->opal_list_prev->opal_list_next = item->opal_list_next;
+    /* reset previous pointer on next to last pointer */
+    item->opal_list_prev->opal_list_next = item->opal_list_next;
 
-  /* reset tail's previous pointer */
-  list->opal_list_sentinel.opal_list_prev = item->opal_list_prev;
+    /* reset tail's previous pointer */
+    list->opal_list_sentinel.opal_list_prev = item->opal_list_prev;
 
 #if OPAL_ENABLE_DEBUG
-  assert( list == item->opal_list_item_belong_to );
-  item->opal_list_next = item->opal_list_prev = (opal_list_item_t *)NULL;
+    assert(list == item->opal_list_item_belong_to);
+    item->opal_list_next = item->opal_list_prev = (opal_list_item_t *) NULL;
 
-  /* Spot check: ensure that the item we're returning is now on no
-     lists */
+    /* Spot check: ensure that the item we're returning is now on no
+       lists */
 
-  OPAL_THREAD_ADD32( &(item->opal_list_item_refcount), -1 );
-  assert(0 == item->opal_list_item_refcount);
-  item->opal_list_item_belong_to = NULL;
+    OPAL_THREAD_ADD_FETCH32(&item->opal_list_item_refcount, -1);
+    assert(0 == item->opal_list_item_refcount);
+    item->opal_list_item_belong_to = NULL;
 #endif
 
-  return (opal_list_item_t *) item;
+    return item;
 }
 
-  /**
-   * Add an item to the list before a given element
-   *
-   * @param list The list container
-   * @param pos List element to insert \c item before
-   * @param item The item to insert
-   *
-   * Inserts \c item before \c pos.  This is an O(1) operation.
-   */
+/**
+ * Add an item to the list before a given element
+ *
+ * @param list The list container
+ * @param pos List element to insert \c item before
+ * @param item The item to insert
+ *
+ * Inserts \c item before \c pos.  This is an O(1) operation.
+ */
 static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos,
                                         opal_list_item_t *item)
 {
@@ -765,7 +759,7 @@ static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos
        not on any list */
 
     assert(0 == item->opal_list_item_refcount);
-    assert( NULL == item->opal_list_item_belong_to );
+    assert(NULL == item->opal_list_item_belong_to);
 #endif
 
     /* point item at the existing elements */
@@ -783,126 +777,101 @@ static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos
     /* Spot check: double check that this item is only on the list
        that we just added it to */
 
-    OPAL_THREAD_ADD32( &(item->opal_list_item_refcount), 1 );
+    OPAL_THREAD_ADD_FETCH32(&(item->opal_list_item_refcount), 1);
     assert(1 == item->opal_list_item_refcount);
     item->opal_list_item_belong_to = list;
 #endif
 }
 
-  /**
-   * Add an item to the list at a specific index location in the list.
-   *
-   * @param list The list container
-   * @param item The item to insert
-   * @param index Location to add the item
-   *
-   * @returns true if insertion succeeded; otherwise false
-   *
-   * This is potentially an O(N) operation to traverse down to the
-   * correct location in the list and add an item.
-   *
-   * Example: if idx = 2 and list = item1->item2->item3->item4, then
-   * after insert, list = item1->item2->item->item3->item4.
-   *
-   * If index is greater than the length of the list, no action is
-   * performed and false is returned.
-   */
-  OPAL_DECLSPEC bool opal_list_insert(opal_list_t *list, opal_list_item_t *item,
-                                      long long idx);
+/**
+ * Join a list into another list
+ *
+ * @param thislist List container for list being operated on
+ * @param pos List item in \c thislist marking the position before
+ *              which items are inserted
+ * @param xlist List container for list being spliced from
+ *
+ * Join a list into another list.  All of the elements of \c xlist
+ * are inserted before \c pos and removed from \c xlist.
+ *
+ * This operation is an O(1) operation.  Both \c thislist and \c
+ * xlist must be valid list containsers.  \c xlist will be empty
+ * but valid after the call.  All pointers to \c opal_list_item_t
+ * containers remain valid, including those that point to elements
+ * in \c xlist.
+ */
+OPAL_DECLSPEC void opal_list_join(opal_list_t *thislist, opal_list_item_t *pos, opal_list_t *xlist);
 
+/**
+ * Splice a list into another list
+ *
+ * @param thislist List container for list being operated on
+ * @param pos List item in \c thislist marking the position before
+ *             which items are inserted
+ * @param xlist List container for list being spliced from
+ * @param first List item in \c xlist marking the start of elements
+ *             to be copied into \c thislist
+ * @param last List item in \c xlist marking the end of elements
+ * to be copied into \c thislist
+ *
+ * Splice a subset of a list into another list.  The \c [first,
+ * last) elements of \c xlist are moved into \c thislist,
+ * inserting them before \c pos.  \c pos must be a valid iterator
+ * in \c thislist and \c [first, last) must be a valid range in \c
+ * xlist.  \c position must not be in the range \c [first, last).
+ * It is, however, valid for \c xlist and \c thislist to be the
+ * same list.
+ *
+ * This is an O(N) operation because the length of both lists must
+ * be recomputed.
+ */
+OPAL_DECLSPEC void opal_list_splice(opal_list_t *thislist, opal_list_item_t *pos,
+                                    opal_list_t *xlist, opal_list_item_t *first,
+                                    opal_list_item_t *last);
 
-    /**
-     * Join a list into another list
-     *
-     * @param thislist List container for list being operated on
-     * @param pos List item in \c thislist marking the position before
-     *              which items are inserted
-     * @param xlist List container for list being spliced from
-     *
-     * Join a list into another list.  All of the elements of \c xlist
-     * are inserted before \c pos and removed from \c xlist.
-     *
-     * This operation is an O(1) operation.  Both \c thislist and \c
-     * xlist must be valid list containsers.  \c xlist will be empty
-     * but valid after the call.  All pointers to \c opal_list_item_t
-     * containers remain valid, including those that point to elements
-     * in \c xlist.
-     */
-    OPAL_DECLSPEC void opal_list_join(opal_list_t *thislist, opal_list_item_t *pos,
-                                      opal_list_t *xlist);
+/**
+ * Comparison function for opal_list_sort(), below.
+ *
+ * @param a Pointer to a pointer to an opal_list_item_t.
+ * Explanation below.
+ * @param b Pointer to a pointer to an opal_list_item_t.
+ * Explanation below.
+ * @retval 1 if \em a is greater than \em b
+ * @retval 0 if \em a is equal to \em b
+ * @retval -1 if \em a is less than \em b
+ *
+ * This function is invoked by qsort(3) from within
+ * opal_list_sort().  It is important to understand what
+ * opal_list_sort() does before invoking qsort, so go read that
+ * documentation first.
+ *
+ * The important thing to realize here is that a and b will be \em
+ * double pointers to the items that you need to compare.  Here's
+ * a sample compare function to illustrate this point:
+ */
+typedef int (*opal_list_item_compare_fn_t)(opal_list_item_t **a, opal_list_item_t **b);
 
-
-    /**
-     * Splice a list into another list
-     *
-     * @param thislist List container for list being operated on
-     * @param pos List item in \c thislist marking the position before
-     *             which items are inserted
-     * @param xlist List container for list being spliced from
-     * @param first List item in \c xlist marking the start of elements
-     *             to be copied into \c thislist
-     * @param last List item in \c xlist marking the end of elements
-     * to be copied into \c thislist
-     *
-     * Splice a subset of a list into another list.  The \c [first,
-     * last) elements of \c xlist are moved into \c thislist,
-     * inserting them before \c pos.  \c pos must be a valid iterator
-     * in \c thislist and \c [first, last) must be a valid range in \c
-     * xlist.  \c postition must not be in the range \c [first, last).
-     * It is, however, valid for \c xlist and \c thislist to be the
-     * same list.
-     *
-     * This is an O(N) operation because the length of both lists must
-     * be recomputed.
-     */
-    OPAL_DECLSPEC void opal_list_splice(opal_list_t *thislist, opal_list_item_t *pos,
-                                        opal_list_t *xlist, opal_list_item_t *first,
-                                        opal_list_item_t *last);
-
-    /**
-     * Comparison function for opal_list_sort(), below.
-     *
-     * @param a Pointer to a pointer to an opal_list_item_t.
-     * Explanation below.
-     * @param b Pointer to a pointer to an opal_list_item_t.
-     * Explanation below.
-     * @retval 1 if \em a is greater than \em b
-     * @retval 0 if \em a is equal to \em b
-     * @retval 11 if \em a is less than \em b
-     *
-     * This function is invoked by qsort(3) from within
-     * opal_list_sort().  It is important to understand what
-     * opal_list_sort() does before invoking qsort, so go read that
-     * documentation first.
-     *
-     * The important thing to realize here is that a and b will be \em
-     * double pointers to the items that you need to compare.  Here's
-     * a sample compare function to illustrate this point:
-     */
-    typedef int (*opal_list_item_compare_fn_t)(opal_list_item_t **a,
-                                               opal_list_item_t **b);
-
-    /**
-     * Sort a list with a provided compare function.
-     *
-     * @param list The list to sort
-     * @param compare Compare function
-     *
-     * Put crassly, this function's complexity is O(N) + O(log(N)).
-     * Its algorithm is:
-     *
-     * - remove every item from the list and put the corresponding
-     *    (opal_list_item_t*)'s in an array
-     * - call qsort(3) with that array and your compare function
-     * - re-add every element of the now-sorted array to the list
-     *
-     * The resulting list is now ordered.  Note, however, that since
-     * an array of pointers is sorted, the comparison function must do
-     * a double de-reference to get to the actual opal_list_item_t (or
-     * whatever the underlying type is).  See the documentation of
-     * opal_list_item_compare_fn_t for an example).
-     */
-    OPAL_DECLSPEC int opal_list_sort(opal_list_t* list, opal_list_item_compare_fn_t compare);
+/**
+ * Sort a list with a provided compare function.
+ *
+ * @param list The list to sort
+ * @param compare Compare function
+ *
+ * Put crassly, this function's complexity is O(N) + O(log(N)).
+ * Its algorithm is:
+ *
+ * - remove every item from the list and put the corresponding
+ *    (opal_list_item_t*)'s in an array
+ * - call qsort(3) with that array and your compare function
+ * - re-add every element of the now-sorted array to the list
+ *
+ * The resulting list is now ordered.  Note, however, that since
+ * an array of pointers is sorted, the comparison function must do
+ * a double de-reference to get to the actual opal_list_item_t (or
+ * whatever the underlying type is).  See the documentation of
+ * opal_list_item_compare_fn_t for an example).
+ */
+OPAL_DECLSPEC int opal_list_sort(opal_list_t *list, opal_list_item_compare_fn_t compare);
 
 END_C_DECLS
 
