@@ -14,7 +14,7 @@
  *                         reserved.
  * Copyright (c) 2018-2019 Intel, Inc.  All rights reserved.
  *
- * Copyright (c) 2018-2021 Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2018-2025 Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * Copyright (c) 2020-2023 Triad National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
@@ -46,6 +46,16 @@
 
 #define MCA_BTL_OFI_REQUESTED_MR_MODE (FI_MR_ALLOCATED | FI_MR_PROV_KEY | FI_MR_VIRT_ADDR | FI_MR_ENDPOINT)
 
+/**
+ * FI_MR_BASIC (1<<0) and FI_MR_SCALABLE (1<<1) are deprecated
+ * since Libfabric 1.5 and the symbols will get removed in
+ * future Libfabric 2.x versions. Use the internal mode bits
+ * for backward compatibilities without breaking compilation
+ * with newer libfabric.
+ */
+#define MCA_BTL_OFI_MR_BASIC    (1 << 0)
+#define MCA_BTL_OFI_MR_SCALABLE (1 << 1)
+
 static char *ofi_progress_mode;
 static bool disable_sep;
 static int mca_btl_ofi_init_device(struct fi_info *info);
@@ -105,7 +115,7 @@ static int validate_info(struct fi_info *info, uint64_t required_caps, char **in
 
     mr_mode = info->domain_attr->mr_mode;
 
-    if (!(mr_mode == FI_MR_BASIC || mr_mode == FI_MR_SCALABLE
+    if (!(mr_mode == MCA_BTL_OFI_MR_BASIC || mr_mode == MCA_BTL_OFI_MR_SCALABLE
 #if defined(FI_MR_HMEM)
           || (mr_mode & ~(FI_MR_VIRT_ADDR | FI_MR_ALLOCATED | FI_MR_PROV_KEY | FI_MR_ENDPOINT | FI_MR_HMEM)) == 0)) {
 #else
@@ -647,7 +657,6 @@ static int mca_btl_ofi_init_device(struct fi_info *info)
     module->outstanding_rdma = 0;
     module->use_virt_addr = false;
     module->use_fi_mr_bind = false;
-    module->bypass_cache = false;
 
 #if defined(FI_HMEM)
     if (ofi_info->caps & FI_HMEM) {
@@ -655,7 +664,7 @@ static int mca_btl_ofi_init_device(struct fi_info *info)
     }
 #endif
 
-    if (ofi_info->domain_attr->mr_mode == FI_MR_BASIC
+    if (ofi_info->domain_attr->mr_mode == MCA_BTL_OFI_MR_BASIC
         || ofi_info->domain_attr->mr_mode & FI_MR_VIRT_ADDR) {
         module->use_virt_addr = true;
     }
@@ -664,13 +673,6 @@ static int mca_btl_ofi_init_device(struct fi_info *info)
         module->use_fi_mr_bind = true;
     }
 
-    /* Currently there is no API to query whether the libfabric provider
-     * uses an underlying registration cache. For now, just check for known
-     * providers that use registration caching. */
-    if (!strncasecmp(info->fabric_attr->prov_name, "efa", 3)) {
-        module->bypass_cache = true;
-    }
-
     /* create endpoint list */
     OBJ_CONSTRUCT(&module->endpoints, opal_list_t);
     OBJ_CONSTRUCT(&module->module_lock, opal_mutex_t);
