@@ -14,7 +14,7 @@
  * Copyright (c) 2009-2010 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2012-2013 NVIDIA Corporation.  All rights reserved.
+ * Copyright (c) 2012-2023 NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,17 +29,17 @@
 
 #include "opal_config.h"
 #include <stddef.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdint.h>
 #ifdef HAVE_SCHED_H
-#include <sched.h>
-#endif  /* HAVE_SCHED_H */
+#    include <sched.h>
+#endif /* HAVE_SCHED_H */
 
-#include "opal/util/bit_ops.h"
 #include "opal/class/opal_free_list.h"
 #include "opal/mca/btl/btl.h"
 #include "opal/mca/common/sm/common_sm.h"
+#include "opal/util/bit_ops.h"
 
 BEGIN_C_DECLS
 
@@ -71,7 +71,7 @@ BEGIN_C_DECLS
  * cachelines.
  */
 
-#define SM_FIFO_FREE  (void *) (-2)
+#define SM_FIFO_FREE (void *) (-2)
 /* We can't use opal_cache_line_size here because we need a
    compile-time constant for padding the struct.  We can't really have
    a compile-time constant that is portable, either (e.g., compile on
@@ -98,83 +98,81 @@ struct sm_fifo_t {
     volatile int tail;
     int num_to_clear;
     int lazy_free;
-    char pad4[SM_CACHE_LINE_PAD - sizeof(void **) -
-              sizeof(opal_atomic_lock_t) -
-              sizeof(int) * 3];
+    char pad4[SM_CACHE_LINE_PAD - sizeof(void **) - sizeof(opal_atomic_lock_t) - sizeof(int) * 3];
 };
 typedef struct sm_fifo_t sm_fifo_t;
 
 /*
- * Shared Memory resource managment
+ * Shared Memory resource management
  */
 
 #if OPAL_ENABLE_PROGRESS_THREADS == 1
-#define DATA (char)0
-#define DONE (char)1
+#    define DATA (char) 0
+#    define DONE (char) 1
 #endif
 
 typedef struct mca_btl_smcuda_mem_node_t {
-    mca_mpool_base_module_t* sm_mpool; /**< shared memory pool */
+    mca_mpool_base_module_t *sm_mpool; /**< shared memory pool */
 } mca_btl_smcuda_mem_node_t;
 
 /**
  * Shared Memory (SM) BTL module.
  */
 struct mca_btl_smcuda_component_t {
-    mca_btl_base_component_2_0_0_t super;  /**< base BTL component */
-    int sm_free_list_num;              /**< initial size of free lists */
-    int sm_free_list_max;              /**< maximum size of free lists */
-    int sm_free_list_inc;              /**< number of elements to alloc when growing free lists */
-    int sm_max_procs;                  /**< upper limit on the number of processes using the shared memory pool */
-    int sm_extra_procs;                /**< number of extra procs to allow */
-    char* sm_mpool_name;               /**< name of shared memory pool module */
+    mca_btl_base_component_2_0_0_t super; /**< base BTL component */
+    int sm_free_list_num;                 /**< initial size of free lists */
+    int sm_free_list_max;                 /**< maximum size of free lists */
+    int sm_free_list_inc; /**< number of elements to alloc when growing free lists */
+    int sm_max_procs;    /**< upper limit on the number of processes using the shared memory pool */
+    int sm_extra_procs;  /**< number of extra procs to allow */
+    char *sm_mpool_name; /**< name of shared memory pool module */
     mca_mpool_base_module_t **sm_mpools; /**< shared memory pools (one for each memory node) */
-    mca_mpool_base_module_t *sm_mpool; /**< mpool on local node */
-    void* sm_mpool_base;               /**< base address of shared memory pool */
-    size_t eager_limit;                /**< first fragment size */
-    size_t max_frag_size;              /**< maximum (second and beyone) fragment size */
+    mca_mpool_base_module_t *sm_mpool;   /**< mpool on local node */
+    void *sm_mpool_base;                 /**< base address of shared memory pool */
+    size_t eager_limit;                  /**< first fragment size */
+    size_t max_frag_size;                /**< maximum (second and beyond) fragment size */
     opal_mutex_t sm_lock;
-    mca_common_sm_module_t *sm_seg;   /**< description of shared memory segment */
-    volatile sm_fifo_t **shm_fifo;     /**< pointer to fifo 2D array in shared memory */
-    char **shm_bases;                  /**< pointer to base pointers in shared memory */
-    uint16_t *shm_mem_nodes;           /**< pointer to mem noded in shared memory */
-    sm_fifo_t **fifo;                  /**< cached copy of the pointer to the 2D
-                                          fifo array.  The address in the shared
-                                          memory segment sm_ctl_header is a relative,
-                                          but this one, in process private memory, is
-                                          a real virtual address */
-    uint16_t *mem_nodes;               /**< cached copy of mem nodes of each local rank */
-    unsigned int fifo_size;            /**< number of FIFO queue entries */
-    unsigned int fifo_lazy_free;       /**< number of reads before lazy fifo free is triggered */
-    int nfifos;                        /**< number of FIFOs per receiver */
-    int32_t num_smp_procs;             /**< current number of smp procs on this host */
-    int32_t my_smp_rank;               /**< My SMP process rank.  Used for accessing
-                                        *   SMP specfic data structures. */
-    opal_free_list_t sm_frags_eager;   /**< free list of sm first */
-    opal_free_list_t sm_frags_max;     /**< free list of sm second */
+    mca_common_sm_module_t *sm_seg;  /**< description of shared memory segment */
+    volatile sm_fifo_t **shm_fifo;   /**< pointer to fifo 2D array in shared memory */
+    char **shm_bases;                /**< pointer to base pointers in shared memory */
+    uint16_t *shm_mem_nodes;         /**< pointer to mem noded in shared memory */
+    sm_fifo_t **fifo;                /**< cached copy of the pointer to the 2D
+                                        fifo array.  The address in the shared
+                                        memory segment sm_ctl_header is a relative,
+                                        but this one, in process private memory, is
+                                        a real virtual address */
+    uint16_t *mem_nodes;             /**< cached copy of mem nodes of each local rank */
+    unsigned int fifo_size;          /**< number of FIFO queue entries */
+    unsigned int fifo_lazy_free;     /**< number of reads before lazy fifo free is triggered */
+    int nfifos;                      /**< number of FIFOs per receiver */
+    int32_t num_smp_procs;           /**< current number of smp procs on this host */
+    int32_t my_smp_rank;             /**< My SMP process rank.  Used for accessing
+                                      *   SMP specific data structures. */
+    opal_free_list_t sm_frags_eager; /**< free list of sm first */
+    opal_free_list_t sm_frags_max;   /**< free list of sm second */
     opal_free_list_t sm_frags_user;
-    opal_free_list_t sm_first_frags_to_progress;  /**< list of first
-                                                    fragments that are
-                                                    awaiting resources */
+    opal_free_list_t sm_first_frags_to_progress; /**< list of first
+                                                   fragments that are
+                                                   awaiting resources */
     struct mca_btl_base_endpoint_t **sm_peers;
 
     opal_free_list_t pending_send_fl;
-    int num_outstanding_frags;         /**< number of fragments sent but not yet returned to free list */
-    int num_pending_sends;             /**< total number on all of my pending-send queues */
+    opal_atomic_int32_t
+        num_outstanding_frags; /**< number of fragments sent but not yet returned to free list */
+    opal_atomic_int32_t num_pending_sends; /**< total number on all of my pending-send queues */
     int mem_node;
     int num_mem_nodes;
 
 #if OPAL_ENABLE_PROGRESS_THREADS == 1
-    char sm_fifo_path[PATH_MAX];   /**< path to fifo used to signal this process */
-    int  sm_fifo_fd;               /**< file descriptor corresponding to opened fifo */
+    char sm_fifo_path[OPAL_PATH_MAX]; /**< path to fifo used to signal this process */
+    int sm_fifo_fd;              /**< file descriptor corresponding to opened fifo */
     opal_thread_t sm_fifo_thread;
 #endif
-    struct mca_btl_smcuda_t      **sm_btls;
+    struct mca_btl_smcuda_t **sm_btls;
     struct mca_btl_smcuda_frag_t **table;
     size_t sm_num_btls;
     size_t sm_max_btls;
 
-
     /** MCA: should we be using knem or not?  neg=try but continue if
         not available, 0=don't try, 1=try and fail if not available */
     int use_knem;
@@ -200,32 +198,31 @@ struct mca_btl_smcuda_component_t {
     char *sm_mpool_rndv_file_name;
     char *sm_ctl_file_name;
     char *sm_rndv_file_name;
-#if OPAL_CUDA_SUPPORT
+
     int cuda_ipc_verbose;
     int cuda_ipc_output;
     int use_cuda_ipc;
     int use_cuda_ipc_same_gpu;
-#endif /* OPAL_CUDA_SUPPORT */
+
     unsigned long mpool_min_size;
     char *allocator;
 };
 typedef struct mca_btl_smcuda_component_t mca_btl_smcuda_component_t;
-OPAL_MODULE_DECLSPEC extern mca_btl_smcuda_component_t mca_btl_smcuda_component;
+OPAL_DECLSPEC extern mca_btl_smcuda_component_t mca_btl_smcuda_component;
 
 /**
  * SM BTL Interface
  */
 struct mca_btl_smcuda_t {
-    mca_btl_base_module_t  super;       /**< base BTL interface */
-    bool btl_inited;  /**< flag indicating if btl has been inited */
+    mca_btl_base_module_t super; /**< base BTL interface */
+    bool btl_inited;             /**< flag indicating if btl has been inited */
     mca_btl_base_module_error_cb_fn_t error_cb;
     mca_rcache_base_module_t *rcache;
 };
 typedef struct mca_btl_smcuda_t mca_btl_smcuda_t;
-OPAL_MODULE_DECLSPEC extern mca_btl_smcuda_t mca_btl_smcuda;
+OPAL_DECLSPEC extern mca_btl_smcuda_t mca_btl_smcuda;
 
-struct btl_smcuda_pending_send_item_t
-{
+struct btl_smcuda_pending_send_item_t {
     opal_free_list_item_t super;
     void *data;
 };
@@ -243,26 +240,30 @@ typedef struct btl_smcuda_pending_send_item_t btl_smcuda_pending_send_item_t;
  * we define macros to translate between relative addresses and
  * virtual addresses.
  */
-#define VIRTUAL2RELATIVE(VADDR ) ((long)(VADDR)  - (long)mca_btl_smcuda_component.shm_bases[mca_btl_smcuda_component.my_smp_rank])
-#define RELATIVE2VIRTUAL(OFFSET) ((long)(OFFSET) + (long)mca_btl_smcuda_component.shm_bases[mca_btl_smcuda_component.my_smp_rank])
+#define VIRTUAL2RELATIVE(VADDR) \
+    ((long) (VADDR)             \
+     - (long) mca_btl_smcuda_component.shm_bases[mca_btl_smcuda_component.my_smp_rank])
+#define RELATIVE2VIRTUAL(OFFSET) \
+    ((long) (OFFSET)             \
+     + (long) mca_btl_smcuda_component.shm_bases[mca_btl_smcuda_component.my_smp_rank])
 
-static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
-                               sm_fifo_t *fifo, int lazy_free)
+static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool, sm_fifo_t *fifo,
+                               int lazy_free)
 {
     int i, qsize;
 
     /* figure out the queue size (a power of two that is at least 1) */
-    qsize = opal_next_poweroftwo_inclusive (fifo_size);
+    qsize = opal_next_poweroftwo_inclusive(fifo_size);
 
     /* allocate the queue in the receiver's address space */
-    fifo->queue_recv = (volatile void **)mpool->mpool_alloc(
-            mpool, sizeof(void *) * qsize, opal_cache_line_size, 0);
-    if(NULL == fifo->queue_recv) {
+    fifo->queue_recv = (volatile void **) mpool->mpool_alloc(mpool, sizeof(void *) * qsize,
+                                                             opal_cache_line_size, 0);
+    if (NULL == fifo->queue_recv) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
 
     /* initialize the queue */
-    for ( i = 0; i < qsize; i++ )
+    for (i = 0; i < qsize; i++)
         fifo->queue_recv[i] = SM_FIFO_FREE;
 
     /* shift queue address to be relative */
@@ -271,8 +272,8 @@ static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
     /* initialize the locks */
     opal_atomic_lock_init(&(fifo->head_lock), OPAL_ATOMIC_LOCK_UNLOCKED);
     opal_atomic_lock_init(&(fifo->tail_lock), OPAL_ATOMIC_LOCK_UNLOCKED);
-    opal_atomic_unlock(&(fifo->head_lock));  /* should be unnecessary */
-    opal_atomic_unlock(&(fifo->tail_lock));  /* should be unnecessary */
+    opal_atomic_unlock(&(fifo->head_lock)); /* should be unnecessary */
+    opal_atomic_unlock(&(fifo->tail_lock)); /* should be unnecessary */
 
     /* other initializations */
     fifo->head = 0;
@@ -284,14 +285,13 @@ static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
     return OPAL_SUCCESS;
 }
 
-
 static inline int sm_fifo_write(void *value, sm_fifo_t *fifo)
 {
     volatile void **q = (volatile void **) RELATIVE2VIRTUAL(fifo->queue);
 
     /* if there is no free slot to write, report exhausted resource */
     opal_atomic_rmb();
-    if ( SM_FIFO_FREE != q[fifo->head] )
+    if (SM_FIFO_FREE != q[fifo->head])
         return OPAL_ERR_OUT_OF_RESOURCE;
 
     /* otherwise, write to the slot and advance the head index */
@@ -301,7 +301,6 @@ static inline int sm_fifo_write(void *value, sm_fifo_t *fifo)
     return OPAL_SUCCESS;
 }
 
-
 static inline void *sm_fifo_read(sm_fifo_t *fifo)
 {
     void *value;
@@ -312,18 +311,18 @@ static inline void *sm_fifo_read(sm_fifo_t *fifo)
     opal_atomic_rmb();
 
     /* if you read a non-empty slot, advance the tail pointer */
-    if ( SM_FIFO_FREE != value ) {
+    if (SM_FIFO_FREE != value) {
 
-        fifo->tail = ( fifo->tail + 1 ) & fifo->mask;
+        fifo->tail = (fifo->tail + 1) & fifo->mask;
         fifo->num_to_clear += 1;
 
         /* check if it's time to free slots, which we do lazily */
-        if ( fifo->num_to_clear >= fifo->lazy_free ) {
-            int i = (fifo->tail - fifo->num_to_clear ) & fifo->mask;
+        if (fifo->num_to_clear >= fifo->lazy_free) {
+            int i = (fifo->tail - fifo->num_to_clear) & fifo->mask;
 
-            while ( fifo->num_to_clear > 0 ) {
+            while (fifo->num_to_clear > 0) {
                 fifo->queue_recv[i] = SM_FIFO_FREE;
-                i = (i+1) & fifo->mask;
+                i = (i + 1) & fifo->mask;
                 fifo->num_to_clear -= 1;
             }
             opal_atomic_wmb();
@@ -338,8 +337,6 @@ static inline void *sm_fifo_read(sm_fifo_t *fifo)
  */
 extern int mca_btl_smcuda_component_progress(void);
 
-
-
 /**
  * Register a callback function that is called on error..
  *
@@ -347,10 +344,8 @@ extern int mca_btl_smcuda_component_progress(void);
  * @return             Status indicating if cleanup was successful
  */
 
-int mca_btl_smcuda_register_error_cb(
-    struct mca_btl_base_module_t* btl,
-    mca_btl_base_module_error_cb_fn_t cbfunc
-);
+int mca_btl_smcuda_register_error_cb(struct mca_btl_base_module_t *btl,
+                                     mca_btl_base_module_error_cb_fn_t cbfunc);
 
 /**
  * Cleanup any resources held by the BTL.
@@ -359,10 +354,7 @@ int mca_btl_smcuda_register_error_cb(
  * @return     OPAL_SUCCESS or error status on failure.
  */
 
-extern int mca_btl_smcuda_finalize(
-    struct mca_btl_base_module_t* btl
-);
-
+extern int mca_btl_smcuda_finalize(struct mca_btl_base_module_t *btl);
 
 /**
  * PML->BTL notification of change in the process list.
@@ -378,14 +370,10 @@ extern int mca_btl_smcuda_finalize(
  *
  */
 
-extern int mca_btl_smcuda_add_procs(
-    struct mca_btl_base_module_t* btl,
-    size_t nprocs,
-    struct opal_proc_t **procs,
-    struct mca_btl_base_endpoint_t** peers,
-    struct opal_bitmap_t* reachability
-);
-
+extern int mca_btl_smcuda_add_procs(struct mca_btl_base_module_t *btl, size_t nprocs,
+                                    struct opal_proc_t **procs,
+                                    struct mca_btl_base_endpoint_t **peers,
+                                    struct opal_bitmap_t *reachability);
 
 /**
  * PML->BTL notification of change in the process list.
@@ -396,13 +384,9 @@ extern int mca_btl_smcuda_add_procs(
  * @return             Status indicating if cleanup was successful
  *
  */
-extern int mca_btl_smcuda_del_procs(
-    struct mca_btl_base_module_t* btl,
-    size_t nprocs,
-    struct opal_proc_t **procs,
-    struct mca_btl_base_endpoint_t **peers
-);
-
+extern int mca_btl_smcuda_del_procs(struct mca_btl_base_module_t *btl, size_t nprocs,
+                                    struct opal_proc_t **procs,
+                                    struct mca_btl_base_endpoint_t **peers);
 
 /**
  * Allocate a segment.
@@ -410,13 +394,9 @@ extern int mca_btl_smcuda_del_procs(
  * @param btl (IN)      BTL module
  * @param size (IN)     Request segment size.
  */
-extern mca_btl_base_descriptor_t* mca_btl_smcuda_alloc(
-    struct mca_btl_base_module_t* btl,
-    struct mca_btl_base_endpoint_t* endpoint,
-    uint8_t order,
-    size_t size,
-    uint32_t flags
-);
+extern mca_btl_base_descriptor_t *mca_btl_smcuda_alloc(struct mca_btl_base_module_t *btl,
+                                                       struct mca_btl_base_endpoint_t *endpoint,
+                                                       uint8_t order, size_t size, uint32_t flags);
 
 /**
  * Return a segment allocated by this BTL.
@@ -424,11 +404,8 @@ extern mca_btl_base_descriptor_t* mca_btl_smcuda_alloc(
  * @param btl (IN)      BTL module
  * @param segment (IN)  Allocated segment.
  */
-extern int mca_btl_smcuda_free(
-    struct mca_btl_base_module_t* btl,
-    mca_btl_base_descriptor_t* segment
-);
-
+extern int mca_btl_smcuda_free(struct mca_btl_base_module_t *btl,
+                               mca_btl_base_descriptor_t *segment);
 
 /**
  * Pack data
@@ -436,16 +413,11 @@ extern int mca_btl_smcuda_free(
  * @param btl (IN)      BTL module
  * @param peer (IN)     BTL peer addressing
  */
-struct mca_btl_base_descriptor_t* mca_btl_smcuda_prepare_src(
-    struct mca_btl_base_module_t* btl,
-    struct mca_btl_base_endpoint_t* endpoint,
-    struct opal_convertor_t* convertor,
-    uint8_t order,
-    size_t reserve,
-    size_t* size,
-    uint32_t flags
-);
-
+struct mca_btl_base_descriptor_t *
+mca_btl_smcuda_prepare_src(struct mca_btl_base_module_t *btl,
+                           struct mca_btl_base_endpoint_t *endpoint,
+                           struct opal_convertor_t *convertor, uint8_t order, size_t reserve,
+                           size_t *size, uint32_t flags);
 
 /**
  * Initiate an inlined send to the peer or return a descriptor.
@@ -453,16 +425,12 @@ struct mca_btl_base_descriptor_t* mca_btl_smcuda_prepare_src(
  * @param btl (IN)      BTL module
  * @param peer (IN)     BTL peer addressing
  */
-extern int mca_btl_smcuda_sendi( struct mca_btl_base_module_t* btl,
-                             struct mca_btl_base_endpoint_t* endpoint,
-                             struct opal_convertor_t* convertor,
-                             void* header,
-                             size_t header_size,
-                             size_t payload_size,
-                             uint8_t order,
-                             uint32_t flags,
-                             mca_btl_base_tag_t tag,
-                             mca_btl_base_descriptor_t** descriptor );
+extern int mca_btl_smcuda_sendi(struct mca_btl_base_module_t *btl,
+                                struct mca_btl_base_endpoint_t *endpoint,
+                                struct opal_convertor_t *convertor, void *header,
+                                size_t header_size, size_t payload_size, uint8_t order,
+                                uint32_t flags, mca_btl_base_tag_t tag,
+                                mca_btl_base_descriptor_t **descriptor);
 
 /**
  * Initiate a send to the peer.
@@ -470,22 +438,21 @@ extern int mca_btl_smcuda_sendi( struct mca_btl_base_module_t* btl,
  * @param btl (IN)      BTL module
  * @param peer (IN)     BTL peer addressing
  */
-extern int mca_btl_smcuda_send(
-    struct mca_btl_base_module_t* btl,
-    struct mca_btl_base_endpoint_t* endpoint,
-    struct mca_btl_base_descriptor_t* descriptor,
-    mca_btl_base_tag_t tag
-);
+extern int mca_btl_smcuda_send(struct mca_btl_base_module_t *btl,
+                               struct mca_btl_base_endpoint_t *endpoint,
+                               struct mca_btl_base_descriptor_t *descriptor,
+                               mca_btl_base_tag_t tag);
+
 
-#if OPAL_CUDA_SUPPORT
 /**
  * Remote get using device memory.
  */
-int mca_btl_smcuda_get_cuda (struct mca_btl_base_module_t *btl,
-    struct mca_btl_base_endpoint_t *ep, void *local_address,
-    uint64_t remote_address, struct mca_btl_base_registration_handle_t *local_handle,
-    struct mca_btl_base_registration_handle_t *remote_handle, size_t size, int flags,
-    int order, mca_btl_base_rdma_completion_fn_t cbfunc, void *cbcontext, void *cbdata);
+int mca_btl_smcuda_get_cuda(struct mca_btl_base_module_t *btl, struct mca_btl_base_endpoint_t *ep,
+                            void *local_address, uint64_t remote_address,
+                            struct mca_btl_base_registration_handle_t *local_handle,
+                            struct mca_btl_base_registration_handle_t *remote_handle, size_t size,
+                            int flags, int order, mca_btl_base_rdma_completion_fn_t cbfunc,
+                            void *cbcontext, void *cbdata);
 
 /* CUDA IPC control message tags */
 enum ipcCtrlMsg {
@@ -496,51 +463,32 @@ enum ipcCtrlMsg {
 
 /* CUDA IPC control message */
 typedef struct ctrlhdr_st {
-        enum ipcCtrlMsg ctag;
-        int cudev;
+    enum ipcCtrlMsg ctag;
+    int cudev;
 } ctrlhdr_t;
 
 /* State of setting up CUDA IPC on an endpoint */
-enum ipcState {
-    IPC_INIT = 1,
-    IPC_SENT,
-    IPC_ACKING,
-    IPC_ACKED,
-    IPC_OK,
-    IPC_BAD
-};
+enum ipcState { IPC_INIT = 1, IPC_SENT, IPC_ACKING, IPC_ACKED, IPC_OK, IPC_BAD };
 
-#endif /* OPAL_CUDA_SUPPORT */
-
-
-extern void mca_btl_smcuda_dump(struct mca_btl_base_module_t* btl,
-                            struct mca_btl_base_endpoint_t* endpoint,
-                            int verbose);
-
-/**
- * Fault Tolerance Event Notification Function
- * @param state Checkpoint Stae
- * @return OPAL_SUCCESS or failure status
- */
-int mca_btl_smcuda_ft_event(int state);
+extern void mca_btl_smcuda_dump(struct mca_btl_base_module_t *btl,
+                                struct mca_btl_base_endpoint_t *endpoint, int verbose);
 
 #if OPAL_ENABLE_PROGRESS_THREADS == 1
-void mca_btl_smcuda_component_event_thread(opal_object_t*);
+void mca_btl_smcuda_component_event_thread(opal_object_t *);
 #endif
 
 #if OPAL_ENABLE_PROGRESS_THREADS == 1
-#define MCA_BTL_SMCUDA_SIGNAL_PEER(peer) \
-{ \
-    unsigned char cmd = DATA; \
-    if(write(peer->fifo_fd, &cmd, sizeof(cmd)) != sizeof(cmd)) { \
-        opal_output(0, "mca_btl_smcuda_send: write fifo failed: errno=%d\n", errno); \
-    } \
-}
+#    define MCA_BTL_SMCUDA_SIGNAL_PEER(peer)                                                 \
+        {                                                                                    \
+            unsigned char cmd = DATA;                                                        \
+            if (write(peer->fifo_fd, &cmd, sizeof(cmd)) != sizeof(cmd)) {                    \
+                opal_output(0, "mca_btl_smcuda_send: write fifo failed: errno=%d\n", errno); \
+            }                                                                                \
+        }
 #else
-#define MCA_BTL_SMCUDA_SIGNAL_PEER(peer)
+#    define MCA_BTL_SMCUDA_SIGNAL_PEER(peer)
 #endif
 
 END_C_DECLS
 
 #endif
-
