@@ -12,9 +12,11 @@
  *                         All rights reserved.
  * Copyright (c) 2006      Sandia National Laboratories. All rights
  *                         reserved.
- * Copyright (c) 2011-2016 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011-2019 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2015-2016 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2020      Amazon.com, Inc. or its affiliates.
+ *                         All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,30 +32,19 @@
 #include "opal_config.h"
 #include <sys/types.h>
 
-#include "opal_stdint.h"
+#include "opal/class/opal_hash_table.h"
 #include "opal/util/alfg.h"
-#include "opal/class/opal_hash_table.h"
-#include "opal/class/opal_hash_table.h"
-#include "opal/mca/event/event.h"
+#include "opal/util/event.h"
+#include "opal/util/misc.h"
+#include "opal_stdint.h"
 
-#if BTL_IN_OPAL
-#include "opal/mca/btl/btl.h"
-#include "opal/mca/btl/base/btl_base_error.h"
 #include "opal/mca/btl/base/base.h"
+#include "opal/mca/btl/base/btl_base_error.h"
+#include "opal/mca/btl/btl.h"
 #include "opal/mca/rcache/rcache.h"
 
 #include "btl_usnic_compat.h"
 
-#if RCACHE_VERSION < 30
-#include "opal/mca/mpool/grdma/mpool_grdma.h"
-#endif
-#else
-#include "ompi/mca/btl/btl.h"
-#include "ompi/mca/btl/base/btl_base_error.h"
-#include "ompi/mca/btl/base/base.h"
-#include "ompi/mca/mpool/grdma/mpool_grdma.h"
-#endif
-
 BEGIN_C_DECLS
 
 /*
@@ -67,8 +58,7 @@ extern uint64_t opal_btl_usnic_ticks;
 /* Lock for MPU_THREAD_MULTIPLE support */
 extern opal_recursive_mutex_t btl_usnic_lock;
 
-static inline uint64_t
-get_ticks(void)
+static inline uint64_t get_ticks(void)
 {
     return opal_btl_usnic_ticks;
 }
@@ -76,36 +66,35 @@ get_ticks(void)
 /* RNG buffer declaration */
 extern opal_rng_buff_t opal_btl_usnic_rand_buff;
 
-#ifndef container_of
-#define container_of(ptr, type, member) ( \
-        (type *)( ((char *)(ptr)) - offsetof(type,member) ))
-#endif
-
 #ifndef max
-#define max(a, b) (((a) > (b)) ? (a) : (b))
+#    define max(a, b) (((a) > (b)) ? (a) : (b))
 #endif
 
 /* MSGDEBUG2 prints 1 line at each BTL entry point */
-#define MSGDEBUG2 (MSGDEBUG1||0)
+#define MSGDEBUG2 (MSGDEBUG1 || 0)
 /* MSGDEBUG1 prints more info about arguments and internal functions */
 #define MSGDEBUG1 0
 
 /* output macros to declutter source */
 #if MSGDEBUG1
-#define MSGDEBUG1_OUT(...) opal_output(0, __VA_ARGS__)
+#    define MSGDEBUG1_OUT(...) opal_output(0, __VA_ARGS__)
 #else
-#define MSGDEBUG1_OUT(...) do {} while (0)
+#    define MSGDEBUG1_OUT(...) \
+        do {                   \
+        } while (0)
 #endif
 #if MSGDEBUG2
-#define MSGDEBUG2_OUT(...) opal_output(0, __VA_ARGS__)
+#    define MSGDEBUG2_OUT(...) opal_output(0, __VA_ARGS__)
 #else
-#define MSGDEBUG2_OUT(...) do {} while (0)
+#    define MSGDEBUG2_OUT(...) \
+        do {                   \
+        } while (0)
 #endif
 
 /* Set to >0 to randomly drop received frags.  The higher the number,
    the more frequent the drops. */
 #define WANT_RECV_DROPS 0
-/* Set to >0 to randomly fail to send an ACK, mimicing a lost ACK.
+/* Set to >0 to randomly fail to send an ACK, mimicking a lost ACK.
    The higher the number, the more frequent the failed-to-send-ACK. */
 #define WANT_FAIL_TO_SEND_ACK 0
 /* Set to >0 to randomly fail to resend a frag (causing it to be
@@ -114,24 +103,24 @@ extern opal_rng_buff_t opal_btl_usnic_rand_buff;
 #define WANT_FAIL_TO_RESEND_FRAG 0
 
 #if WANT_RECV_DROPS > 0
-#define FAKE_RECV_DROP (opal_rand(&opal_btl_usnic_rand_buff) < WANT_RECV_DROPS)
+#    define FAKE_RECV_DROP (opal_rand(&opal_btl_usnic_rand_buff) < WANT_RECV_DROPS)
 #else
-#define FAKE_RECV_DROP 0
+#    define FAKE_RECV_DROP 0
 #endif
 
 #if WANT_FAIL_TO_SEND_ACK > 0
-#define FAKE_FAIL_TO_SEND_ACK (opal_rand(&opal_btl_usnic_rand_buff) < WANT_FAIL_TO_SEND_ACK)
+#    define FAKE_FAIL_TO_SEND_ACK (opal_rand(&opal_btl_usnic_rand_buff) < WANT_FAIL_TO_SEND_ACK)
 #else
-#define FAKE_FAIL_TO_SEND_ACK 0
+#    define FAKE_FAIL_TO_SEND_ACK 0
 #endif
 
 #if WANT_FAIL_TO_RESEND_FRAG > 0
-#define FAKE_FAIL_TO_RESEND_FRAG (opal_rand(&opal_btl_usnic_rand_buff) < WANT_FAIL_TO_RESEND_FRAG)
+#    define FAKE_FAIL_TO_RESEND_FRAG \
+        (opal_rand(&opal_btl_usnic_rand_buff) < WANT_FAIL_TO_RESEND_FRAG)
 #else
-#define FAKE_FAIL_TO_RESEND_FRAG 0
+#    define FAKE_FAIL_TO_RESEND_FRAG 0
 #endif
 
-
 /**
  * usnic BTL component
  */
@@ -152,9 +141,9 @@ typedef struct opal_btl_usnic_component_t {
     uint64_t my_hashed_rte_name;
 
     /** array of possible BTLs (>= num_modules elements) */
-    struct opal_btl_usnic_module_t* usnic_all_modules;
+    struct opal_btl_usnic_module_t *usnic_all_modules;
     /** array of pointers to active BTLs (num_modules elements) */
-    struct opal_btl_usnic_module_t** usnic_active_modules;
+    struct opal_btl_usnic_module_t **usnic_active_modules;
 
     /** convertor packing threshold */
     int pack_lazy_threshold;
@@ -164,16 +153,11 @@ typedef struct opal_btl_usnic_component_t {
     /** list of usnic proc structures */
     opal_list_t usnic_procs;
 
-#if RCACHE_VERSION == 30
     /** memory pool hints */
-    char* usnic_mpool_hints;
+    char *usnic_mpool_hints;
 
     /** registration cache name */
     char *usnic_rcache_name;
-#else
-    /** name of memory pool */
-    char* usnic_mpool_name;
-#endif
 
     char *if_include;
     char *if_exclude;
@@ -193,7 +177,7 @@ typedef struct opal_btl_usnic_component_t {
     /** max receive descriptors per module */
     int32_t rd_num;
 
-    /** max send/receive desriptors for priority channel */
+    /** max send/receive descriptors for priority channel */
     int32_t prio_sd_num;
     int32_t prio_rd_num;
 
@@ -254,7 +238,7 @@ typedef struct opal_btl_usnic_component_t {
     opal_event_base_t *opal_evbase;
 } opal_btl_usnic_component_t;
 
-OPAL_MODULE_DECLSPEC extern opal_btl_usnic_component_t mca_btl_usnic_component;
+OPAL_DECLSPEC extern opal_btl_usnic_component_t mca_btl_usnic_component;
 
 typedef mca_btl_base_recv_reg_t opal_btl_usnic_recv_reg_t;
 
@@ -269,11 +253,11 @@ typedef uint16_t opal_btl_usnic_seq_t;
  * Relies on the fact that sequence numbers should be relatively close
  * together as compared to (1<<31)
  */
-#define SEQ_DIFF(A,B) ((int16_t)((A)-(B)))
-#define SEQ_LT(A,B) (SEQ_DIFF(A,B) < 0)
-#define SEQ_LE(A,B) (SEQ_DIFF(A,B) <= 0)
-#define SEQ_GT(A,B) (SEQ_DIFF(A,B) > 0)
-#define SEQ_GE(A,B) (SEQ_DIFF(A,B) >= 0)
+#define SEQ_DIFF(A, B) ((int16_t)((A) - (B)))
+#define SEQ_LT(A, B)   (SEQ_DIFF(A, B) < 0)
+#define SEQ_LE(A, B)   (SEQ_DIFF(A, B) <= 0)
+#define SEQ_GT(A, B)   (SEQ_DIFF(A, B) > 0)
+#define SEQ_GE(A, B)   (SEQ_DIFF(A, B) >= 0)
 
 /**
  * Register the usnic BTL MCA params
