@@ -14,19 +14,19 @@
 #include "opal_config.h"
 
 #include "opal/constants.h"
-#include "opal/util/output.h"
-#include "opal/mca/mca.h"
-#include "opal/mca/if/if.h"
 #include "opal/mca/if/base/base.h"
 #include "opal/mca/if/base/static-components.h"
+#include "opal/mca/if/if.h"
+#include "opal/mca/mca.h"
+#include "opal/util/output.h"
 
 /* instantiate the global list of interfaces */
 opal_list_t opal_if_list = {{0}};
 bool opal_if_do_not_resolve = false;
 bool opal_if_retain_loopback = false;
 
-static int opal_if_base_register (mca_base_register_flag_t flags);
-static int opal_if_base_open (mca_base_open_flag_t flags);
+static int opal_if_base_register(mca_base_register_flag_t flags);
+static int opal_if_base_open(mca_base_open_flag_t flags);
 static int opal_if_base_close(void);
 static void opal_if_construct(opal_if_t *obj);
 
@@ -35,30 +35,29 @@ static bool frameopen = false;
 /* instance the opal_if_t object */
 OBJ_CLASS_INSTANCE(opal_if_t, opal_list_item_t, opal_if_construct, NULL);
 
-MCA_BASE_FRAMEWORK_DECLARE(opal, if, NULL, opal_if_base_register, opal_if_base_open, opal_if_base_close,
-                           mca_if_base_static_components, 0);
+MCA_BASE_FRAMEWORK_DECLARE(opal, if, NULL, opal_if_base_register, opal_if_base_open,
+                           opal_if_base_close, mca_if_base_static_components, 0);
 
-static int opal_if_base_register (mca_base_register_flag_t flags)
+static int opal_if_base_register(mca_base_register_flag_t flags)
 {
     opal_if_do_not_resolve = false;
-    (void) mca_base_framework_var_register (&opal_if_base_framework, "do_not_resolve",
-                                            "If nonzero, do not attempt to resolve interfaces",
-                                            MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
-                                            OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
-                                            &opal_if_do_not_resolve);
+    (void) mca_base_framework_var_register(&opal_if_base_framework, "do_not_resolve",
+                                           "If nonzero, do not attempt to resolve interfaces",
+                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_ALL_EQ, &opal_if_do_not_resolve);
 
     opal_if_retain_loopback = false;
-    (void) mca_base_framework_var_register (&opal_if_base_framework, "retain_loopback",
-                                            "If nonzero, retain loopback interfaces",
-                                            MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
-                                            OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
-                                            &opal_if_retain_loopback);
+    (void) mca_base_framework_var_register(&opal_if_base_framework, "retain_loopback",
+                                           "If nonzero, retain loopback interfaces",
+                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_ALL_EQ, &opal_if_retain_loopback);
 
     return OPAL_SUCCESS;
 }
 
-
-static int opal_if_base_open (mca_base_open_flag_t flags)
+static int opal_if_base_open(mca_base_open_flag_t flags)
 {
     if (frameopen) {
         return OPAL_SUCCESS;
@@ -71,7 +70,6 @@ static int opal_if_base_open (mca_base_open_flag_t flags)
     return mca_base_framework_components_open(&opal_if_base_framework, flags);
 }
 
-
 static int opal_if_base_close(void)
 {
     opal_list_item_t *item;
