@@ -23,7 +23,7 @@ BEGIN_C_DECLS
 
 /* Globally exported variables */
 
-OSHMEM_MODULE_DECLSPEC extern mca_atomic_base_component_1_0_0_t
+OSHMEM_DECLSPEC extern mca_atomic_base_component_1_0_0_t
 mca_atomic_basic_component;
 
 OSHMEM_DECLSPEC void atomic_basic_lock(shmem_ctx_t ctx, int pe);
