@@ -19,9 +19,9 @@
 #include "stdio.h"
 
 #if OSHMEM_PROFILING
-#include "oshmem/shmem/fortran/profile/pbindings.h"
+#include "oshmem/shmem/fortran/pbindings.h"
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_COMPLEX_PUT, shmem_complex_put)
-#include "oshmem/shmem/fortran/profile/defines.h"
+#include "oshmem/shmem/fortran/profile-defines.h"
 #endif
 
 SHMEM_GENERATE_FORTRAN_BINDINGS_SUB (void,
@@ -37,7 +37,7 @@ void shmem_complex_put_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI
     size_t complex_type_size = 0;
     ompi_datatype_type_size(&ompi_mpi_cplex.dt, &complex_type_size);
 
-    MCA_SPML_CALL(put(FPTR_2_VOID_PTR(target),
+    MCA_SPML_CALL(put(oshmem_ctx_default, FPTR_2_VOID_PTR(target),
         OMPI_FINT_2_INT(*length) * complex_type_size,
         FPTR_2_VOID_PTR(source),
         OMPI_FINT_2_INT(*pe)));
