@@ -18,14 +18,14 @@
 #include "oshmem/op/op.h"
 
 #if OSHMEM_PROFILING
-#include "oshmem/shmem/fortran/profile/pbindings.h"
+#include "oshmem/shmem/fortran/pbindings.h"
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_INT2_MAX_TO_ALL, shmem_int2_max_to_all)
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_INT4_MAX_TO_ALL, shmem_int4_max_to_all)
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_INT8_MAX_TO_ALL, shmem_int8_max_to_all)
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_REAL4_MAX_TO_ALL, shmem_real4_max_to_all)
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_REAL8_MAX_TO_ALL, shmem_real8_max_to_all)
 SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_REAL16_MAX_TO_ALL, shmem_real16_max_to_all)
-#include "oshmem/shmem/fortran/profile/defines.h"
+#include "oshmem/shmem/fortran/profile-defines.h"
 #endif
 
 SHMEM_GENERATE_FORTRAN_BINDINGS_SUB (void,
