@@ -156,7 +156,7 @@ do { \
         for(int _r = repeats; _r > 0; _r--) { \
             memcpy(_p2, _p3, sizeof(TYPE) * (COUNT)); \
             tstart = MPI_Wtime(); \
-            MPI_Reduce_local(_p1+_k, _p2+_k, (COUNT), (MPITYPE), (MPIOP)); \
+            MPI_Reduce_local(_p1+_k, _p2+_k, (COUNT)-_k, (MPITYPE), (MPIOP)); \
             tend = MPI_Wtime(); \
             duration[_k] += (tend - tstart); \
             if( check ) { \
@@ -184,7 +184,7 @@ int main(int argc, char **argv)
     double *duration, tstart, tend;
     bool check = true;
     char type[5] = "uifd", *op = "sum", *mpi_type;
-    int lower = 1, upper = 1000000, skip_op_type;
+    int lower = 1, upper = 16*1024*1024, skip_op_type;
     MPI_Op mpi_op;
 
     while( -1 != (c = getopt(argc, argv, "l:u:r:t:o:i:s:n:1:2:vfh")) ) {
@@ -271,7 +271,9 @@ int main(int argc, char **argv)
                     " -1 <number> : (mis)alignment in elements for the first op\n"
                     " -2 <number> : (mis)alignment in elements for the result\n"
                     " -v: increase the verbosity level\n"
-                    " -h: this help message\n", argv[0]);
+                    " -f: turn off correctness checks\n"
+                    " -h: this help message\n",
+                    argv[0]);
             exit(0);
         }
     }
@@ -339,7 +341,7 @@ int main(int argc, char **argv)
                                                in_int8, inout_int8, inout_int8_for_check,
                                                count, PRId8);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  // intentionally reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_INT8_T, int8_t,
                                                in_int8, inout_int8, inout_int8_for_check,
                                                count, PRId8);
@@ -385,7 +387,7 @@ int main(int argc, char **argv)
                                                in_int16, inout_int16, inout_int16_for_check,
                                                count, PRId16);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  // intentionally reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_INT16_T, int16_t,
                                                in_int16, inout_int16, inout_int16_for_check,
                                                count, PRId16);
@@ -431,7 +433,7 @@ int main(int argc, char **argv)
                                                in_int32, inout_int32, inout_int32_for_check,
                                                count, PRId32);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  // intentionally reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_INT32_T, int32_t,
                                                in_int32, inout_int32, inout_int32_for_check,
                                                count, PRId32);
@@ -477,7 +479,7 @@ int main(int argc, char **argv)
                                                in_int64, inout_int64, inout_int64_for_check,
                                                count, PRId64);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  // intentionally reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_INT64_T, int64_t,
                                                in_int64, inout_int64, inout_int64_for_check,
                                                count, PRId64);
@@ -526,7 +528,7 @@ int main(int argc, char **argv)
                                                in_uint8, inout_uint8, inout_uint8_for_check,
                                                count, PRIu8);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  //intentionall reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_UINT8_T, uint8_t,
                                                in_uint8, inout_uint8, inout_uint8_for_check,
                                                count, PRIu8);
@@ -572,7 +574,7 @@ int main(int argc, char **argv)
                                                in_uint16, inout_uint16, inout_uint16_for_check,
                                                count, PRIu16);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  //intentionally reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_UINT16_T, uint16_t,
                                                in_uint16, inout_uint16, inout_uint16_for_check,
                                                count, PRIu16);
@@ -618,7 +620,7 @@ int main(int argc, char **argv)
                                                in_uint32, inout_uint32, inout_uint32_for_check,
                                                count, PRIu32);
                         }
-                        if( 0 == strcmp(op, "min") ) {  //intentionly reversed in and out
+                        if( 0 == strcmp(op, "min") ) {  //intentionally reversed in and out
                             MPI_OP_MINMAX_TEST(min, mpi_op,  MPI_UINT32_T, uint32_t,
                                                in_uint32, inout_uint32, inout_uint32_for_check,
                                                count, PRIu32);
